
/*
 *  @(#)ControlPanel.java 1.49 01/03/02 14:05:04
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */

package com.sun.j3d.demos.j3dfly;

import java.awt.Cursor;
import java.util.ArrayList;

import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.ProgressMonitor;

import javax.media.j3d.BranchGroup;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.Bounds;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.View;
import javax.media.j3d.Transform3D;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.PolygonAttributes;
import javax.media.j3d.DirectionalLight;
import javax.media.j3d.Node;
import javax.media.j3d.ViewPlatform;

import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;
import javax.vecmath.Color3f;
import javax.vecmath.Vector3f;

import com.sun.j3d.utils.behaviors.vp.OrbitBehavior;

import com.sun.j3d.demos.j3dfly.warp.WarpSet;
import com.sun.j3d.demos.j3dfly.warp.WarpEditorDialog;

import com.sun.j3d.demos.utils.developmenttools.DevelopmentSimpleUniverse;
import com.sun.j3d.demos.utils.loadercontrol.LoaderControl;
import com.sun.j3d.demos.utils.loadercontrol.LoaderControlListener;
import com.sun.j3d.demos.utils.vpbehaviors.HoverBehavior;
import com.sun.j3d.demos.utils.vpbehaviors.FlyBehavior;
import com.sun.j3d.demos.utils.vpbehaviors.VPBehavior;
import com.sun.j3d.demos.utils.vpbehaviors.VPDefaultCollision;
import com.sun.j3d.demos.utils.vpbehaviors.VPDriveCollision;
import com.sun.j3d.demos.utils.vpbehaviors.SimpleSweptVolume;

import com.sun.j3d.demos.utils.scenegraph.traverser.ChangePolygonAttributes;
import com.sun.j3d.demos.utils.scenegraph.traverser.ChangeTextureAttributes;
import com.sun.j3d.demos.utils.scenegraph.traverser.TreeScan;
import com.sun.j3d.demos.utils.scenegraph.traverser.ChangeMaterial;
import com.sun.j3d.demos.utils.scenegraph.traverser.NodeChangeProcessor;
import com.sun.j3d.demos.utils.scenegraph.traverser.BehaviorSetEnable;
import com.sun.j3d.demos.utils.scenegraph.io.SceneGraphFileWriter;
import com.sun.j3d.demos.utils.loadercontrol.ExampleFileFilter;
import com.sun.j3d.demos.utils.loaderwrappers.J3fLoaderListener;
import com.sun.j3d.demos.utils.loaderwrappers.J3fLoader;
import com.sun.j3d.demos.utils.gui.StatsDialog;
import com.sun.j3d.demos.utils.gui.FloatDocument;
import com.sun.j3d.demos.utils.gui.WorkingDialog;

/**
 * @author Paul Byrne
 * @version	1.49, 03/02/01
 */
public class ControlPanel extends javax.swing.JPanel implements LoaderControlListener, J3fLoaderListener {

    private DevelopmentSimpleUniverse universe;
    private LoaderControl loaderControl;
    private JFileChooser fileChooser=null;
    //private BranchGroup rootBG;
    private BranchGroup toolBG;
    private boolean collisionEnabled = false;
    private boolean collisionCapabilitySet = false;
    private VPDefaultCollision collisionControl = null;
    private BoundingSphere bigBounds;
    
    private VPBehavior viewBehavior = null;
    private ArrayList graphs;
    
    private StatsDialog statsDialog = null;
    private Cursor normalCursor;
    private WarpSet warpSet = null;
    private WarpSet animationSet = null;
    private int showAllAxis;
    
    private boolean gotValidBounds = false;
    private BoundingSphere worldBounds = null;

    /** Creates new form controlPanel */
    public ControlPanel( com.sun.j3d.demos.utils.developmenttools.DevelopmentSimpleUniverse universe ) {
        this.universe = universe;
        initComponents ();
        
        graphs = new ArrayList();
        
        if (com.sun.j3d.demos.utils.help.Help.isEnabled()) {
            com.sun.j3d.demos.utils.help.Help.addHelpButton( helpB, "j3dfly.main" );
        } else {
            paddingPanel.remove( helpB );
        }
        
        frontClipTF.setText( Double.toString(universe.getViewer().getView().getFrontClipDistance() ));
        backClipTF.setText( Double.toString(universe.getViewer().getView().getBackClipDistance() ));

        bigBounds = new BoundingSphere( new Point3d(), Float.MAX_VALUE );

        toolBG = new BranchGroup();
        universe.addBranchGraph( toolBG );
        
        javax.swing.ToolTipManager.sharedInstance().setLightWeightPopupEnabled( false );
        
        loaderControl = new LoaderControl();
        
        warpSet = new WarpSet( warpCB );
        animationSet = new WarpSet( new javax.swing.JComboBox() );
        J3fLoader.setJ3fLoaderListener( this );

        // Tell the IO system to use the Java3D super class objects for any
        // user subclasses of Java3D objects whose class is not in the
        // classpath when loading j3f files.
        System.setProperty("j3d.demo.io.UseSuperClassIfNoChildClass", "");
        
        // Use JPEG compression (which is the default) when saving
        // images.
        System.setProperty("j3d.demo.io.ImageCompression", "JPEG" );

    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        geometryPanel = new javax.swing.JPanel();
        loadB = new javax.swing.JButton();
        statisticsB = new javax.swing.JButton();
        compileB = new javax.swing.JButton();
        saveB = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        filenameTF = new javax.swing.JTextField();
        clearB = new javax.swing.JButton();
        configureLoadersB = new javax.swing.JButton();
        stripifyB = new javax.swing.JButton();
        viewPanel = new javax.swing.JPanel();
        polygonModeCB = new JComboBox( new String[] {
            "POLYGON_FILL",
            "POLYGON_LINE",
            "POLYGON_POINT" });
            lightingC = new javax.swing.JCheckBox();
            clipP = new javax.swing.JPanel();
            jLabel2 = new javax.swing.JLabel();
            jLabel3 = new javax.swing.JLabel();
            frontClipTF = new javax.swing.JTextField();
            backClipTF = new javax.swing.JTextField();
            textureEnableC = new javax.swing.JCheckBox();
            behaviorEnableC = new javax.swing.JCheckBox();
            behaviorPanel = new javax.swing.JPanel();
            viewBehaviorCB = new JComboBox( new String[] {
                "None",
                "Hover",
                "Drive",
                "Fly",
                "Orbit" } );
                enableCollisionC = new javax.swing.JCheckBox();
                showAllB = new javax.swing.JButton();
                jLabel4 = new javax.swing.JLabel();
                warpEditB = new javax.swing.JButton();
                warpCB = new javax.swing.JComboBox( new String[] { "" } );
                behaviorPropertiesB = new javax.swing.JButton();
                showAllAxisCB = new javax.swing.JComboBox( new String[] {
                    "X",
                    "Y",
                    "Z",
                    "-X",
                    "-Y",
                    "-Z" } );
                    jPanel2 = new javax.swing.JPanel();
                    paddingPanel = new javax.swing.JPanel();
                    helpB = new javax.swing.JButton();
                    exitB = new javax.swing.JButton();
                    setLayout(new javax.swing.BoxLayout(this, 1));
                    
                    geometryPanel.setLayout(new java.awt.GridBagLayout());
                    java.awt.GridBagConstraints gridBagConstraints1;
                    geometryPanel.setBorder(new javax.swing.border.TitledBorder("Geometry"));
                    
                    loadB.setToolTipText("Load a new Scene");
                      loadB.setText("Load...");
                      loadB.addActionListener(new java.awt.event.ActionListener() {
                          public void actionPerformed(java.awt.event.ActionEvent evt) {
                              loadBActionPerformed(evt);
                          }
                      }
                      );
                      gridBagConstraints1 = new java.awt.GridBagConstraints();
                      gridBagConstraints1.gridx = 0;
                      gridBagConstraints1.gridy = 1;
                      gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
                      geometryPanel.add(loadB, gridBagConstraints1);
                      
                      
                    statisticsB.setToolTipText("Geometry and performance statistics");
                      statisticsB.setText("Stats...");
                      statisticsB.addActionListener(new java.awt.event.ActionListener() {
                          public void actionPerformed(java.awt.event.ActionEvent evt) {
                              statisticsBActionPerformed(evt);
                          }
                      }
                      );
                      gridBagConstraints1 = new java.awt.GridBagConstraints();
                      gridBagConstraints1.gridx = 0;
                      gridBagConstraints1.gridy = 2;
                      gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
                      geometryPanel.add(statisticsB, gridBagConstraints1);
                      
                      
                    compileB.setText("Compile");
                      compileB.addActionListener(new java.awt.event.ActionListener() {
                          public void actionPerformed(java.awt.event.ActionEvent evt) {
                              compileBActionPerformed(evt);
                          }
                      }
                      );
                      gridBagConstraints1 = new java.awt.GridBagConstraints();
                      gridBagConstraints1.gridx = 0;
                      gridBagConstraints1.gridy = 3;
                      gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
                      geometryPanel.add(compileB, gridBagConstraints1);
                      
                      
                    saveB.setToolTipText("Output the scene graph to a file using the SceneGraphIO utility.");
                      saveB.setText("Save...");
                      saveB.addActionListener(new java.awt.event.ActionListener() {
                          public void actionPerformed(java.awt.event.ActionEvent evt) {
                              saveBActionPerformed(evt);
                          }
                      }
                      );
                      gridBagConstraints1 = new java.awt.GridBagConstraints();
                      gridBagConstraints1.gridx = 1;
                      gridBagConstraints1.gridy = 2;
                      gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
                      geometryPanel.add(saveB, gridBagConstraints1);
                      
                      
                    jPanel1.setLayout(new java.awt.BorderLayout());
                      
                      jLabel1.setText("File :");
                        jPanel1.add(jLabel1, java.awt.BorderLayout.WEST);
                        
                        
                      
                        filenameTF.setPreferredSize(new java.awt.Dimension(98, 21));
                          jScrollPane1.setViewportView(filenameTF);
                          
                          jPanel1.add(jScrollPane1, java.awt.BorderLayout.CENTER);
                        
                        gridBagConstraints1 = new java.awt.GridBagConstraints();
                      gridBagConstraints1.gridwidth = 2;
                      gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
                      geometryPanel.add(jPanel1, gridBagConstraints1);
                      
                      
                    clearB.setToolTipText("Clear existing geometry");
                      clearB.setText("Clear");
                      clearB.addActionListener(new java.awt.event.ActionListener() {
                          public void actionPerformed(java.awt.event.ActionEvent evt) {
                              clearBActionPerformed(evt);
                          }
                      }
                      );
                      gridBagConstraints1 = new java.awt.GridBagConstraints();
                      gridBagConstraints1.gridx = 1;
                      gridBagConstraints1.gridy = 1;
                      gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
                      geometryPanel.add(clearB, gridBagConstraints1);
                      
                      
                    configureLoadersB.setToolTipText("Configure which loaders are used for each file format");
                      configureLoadersB.setText("Configure Loaders...");
                      configureLoadersB.addActionListener(new java.awt.event.ActionListener() {
                          public void actionPerformed(java.awt.event.ActionEvent evt) {
                              configureLoadersBActionPerformed(evt);
                          }
                      }
                      );
                      gridBagConstraints1 = new java.awt.GridBagConstraints();
                      gridBagConstraints1.gridx = 0;
                      gridBagConstraints1.gridy = 4;
                      gridBagConstraints1.gridwidth = 2;
                      gridBagConstraints1.anchor = java.awt.GridBagConstraints.WEST;
                      geometryPanel.add(configureLoadersB, gridBagConstraints1);
                      
                      
                    stripifyB.setToolTipText("Stripify TriangleArrays");
                      stripifyB.setText("Stripify");
                      stripifyB.addActionListener(new java.awt.event.ActionListener() {
                          public void actionPerformed(java.awt.event.ActionEvent evt) {
                              stripifyBActionPerformed(evt);
                          }
                      }
                      );
                      gridBagConstraints1 = new java.awt.GridBagConstraints();
                      gridBagConstraints1.gridx = 1;
                      gridBagConstraints1.gridy = 3;
                      geometryPanel.add(stripifyB, gridBagConstraints1);
                      
                      
                    add(geometryPanel);
                    
                    
                    viewPanel.setLayout(new java.awt.GridBagLayout());
                    java.awt.GridBagConstraints gridBagConstraints2;
                    viewPanel.setBorder(new javax.swing.border.TitledBorder("Rendering Attributes"));
                    
                    polygonModeCB.setToolTipText("Select Polygon Rendering Mode");
                      polygonModeCB.addItemListener(new java.awt.event.ItemListener() {
                          public void itemStateChanged(java.awt.event.ItemEvent evt) {
                              polygonModeCBItemStateChanged(evt);
                          }
                      }
                      );
                      gridBagConstraints2 = new java.awt.GridBagConstraints();
                      gridBagConstraints2.gridx = 0;
                      gridBagConstraints2.gridy = 0;
                      viewPanel.add(polygonModeCB, gridBagConstraints2);
                      
                      
                    lightingC.setToolTipText("Enable lighting in the model");
                      lightingC.setText("Lighting Enabled");
                      lightingC.addItemListener(new java.awt.event.ItemListener() {
                          public void itemStateChanged(java.awt.event.ItemEvent evt) {
                              lightingCItemStateChanged(evt);
                          }
                      }
                      );
                      gridBagConstraints2 = new java.awt.GridBagConstraints();
                      gridBagConstraints2.gridx = 0;
                      gridBagConstraints2.gridy = 1;
                      gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
                      viewPanel.add(lightingC, gridBagConstraints2);
                      
                      
                    clipP.setLayout(new java.awt.GridBagLayout());
                      java.awt.GridBagConstraints gridBagConstraints3;
                      
                      jLabel2.setText("Front Clip :");
                        gridBagConstraints3 = new java.awt.GridBagConstraints();
                        gridBagConstraints3.anchor = java.awt.GridBagConstraints.EAST;
                        clipP.add(jLabel2, gridBagConstraints3);
                        
                        
                      jLabel3.setText("Back Clip :");
                        gridBagConstraints3 = new java.awt.GridBagConstraints();
                        gridBagConstraints3.gridx = 0;
                        gridBagConstraints3.gridy = 1;
                        gridBagConstraints3.anchor = java.awt.GridBagConstraints.EAST;
                        clipP.add(jLabel3, gridBagConstraints3);
                        
                        
                      frontClipTF.setPreferredSize(new java.awt.Dimension(80, 21));
                        frontClipTF.setText("jTextField1");
                        frontClipTF.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
                        frontClipTF.setMinimumSize(new java.awt.Dimension(80, 21));
                        frontClipTF.setDocument(new FloatDocument());
                        frontClipTF.addActionListener(new java.awt.event.ActionListener() {
                            public void actionPerformed(java.awt.event.ActionEvent evt) {
                                frontClipTFActionPerformed(evt);
                            }
                        }
                        );
                        frontClipTF.addMouseListener(new java.awt.event.MouseAdapter() {
                            public void mouseExited(java.awt.event.MouseEvent evt) {
                                frontClipTFMouseExited(evt);
                            }
                        }
                        );
                        gridBagConstraints3 = new java.awt.GridBagConstraints();
                        gridBagConstraints3.gridwidth = 0;
                        gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
                        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
                        clipP.add(frontClipTF, gridBagConstraints3);
                        
                        
                      backClipTF.setText("jTextField2");
                        backClipTF.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
                        backClipTF.setDocument(new FloatDocument());
                        backClipTF.addActionListener(new java.awt.event.ActionListener() {
                            public void actionPerformed(java.awt.event.ActionEvent evt) {
                                backClipTFActionPerformed(evt);
                            }
                        }
                        );
                        backClipTF.addMouseListener(new java.awt.event.MouseAdapter() {
                            public void mouseExited(java.awt.event.MouseEvent evt) {
                                backClipTFMouseExited(evt);
                            }
                        }
                        );
                        gridBagConstraints3 = new java.awt.GridBagConstraints();
                        gridBagConstraints3.gridx = 1;
                        gridBagConstraints3.gridy = 1;
                        gridBagConstraints3.gridwidth = 0;
                        gridBagConstraints3.fill = java.awt.GridBagConstraints.HORIZONTAL;
                        gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
                        clipP.add(backClipTF, gridBagConstraints3);
                        
                        gridBagConstraints2 = new java.awt.GridBagConstraints();
                      gridBagConstraints2.gridx = 0;
                      gridBagConstraints2.gridy = 4;
                      gridBagConstraints2.gridwidth = 0;
                      gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
                      gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
                      viewPanel.add(clipP, gridBagConstraints2);
                      
                      
                    textureEnableC.setToolTipText("Enable/Disable Texture map on geometry");
                      textureEnableC.setSelected(true);
                      textureEnableC.setText("Texture Enabled");
                      textureEnableC.addActionListener(new java.awt.event.ActionListener() {
                          public void actionPerformed(java.awt.event.ActionEvent evt) {
                              textureEnableCActionPerformed(evt);
                          }
                      }
                      );
                      gridBagConstraints2 = new java.awt.GridBagConstraints();
                      gridBagConstraints2.gridx = 0;
                      gridBagConstraints2.gridy = 2;
                      gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
                      viewPanel.add(textureEnableC, gridBagConstraints2);
                      
                      
                    behaviorEnableC.setToolTipText("Set state of all Behaviors");
                      behaviorEnableC.setSelected(true);
                      behaviorEnableC.setText("Behaviors Enabled");
                      behaviorEnableC.addActionListener(new java.awt.event.ActionListener() {
                          public void actionPerformed(java.awt.event.ActionEvent evt) {
                              behaviorEnableCActionPerformed(evt);
                          }
                      }
                      );
                      gridBagConstraints2 = new java.awt.GridBagConstraints();
                      gridBagConstraints2.gridx = 0;
                      gridBagConstraints2.gridy = 3;
                      gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
                      viewPanel.add(behaviorEnableC, gridBagConstraints2);
                      
                      
                    add(viewPanel);
                    
                    
                    behaviorPanel.setLayout(new java.awt.GridBagLayout());
                    java.awt.GridBagConstraints gridBagConstraints4;
                    behaviorPanel.setBorder(new javax.swing.border.TitledBorder("View Behavior"));
                    
                    viewBehaviorCB.setToolTipText("Select View motion controls");
                      viewBehaviorCB.addActionListener(new java.awt.event.ActionListener() {
                          public void actionPerformed(java.awt.event.ActionEvent evt) {
                              viewBehaviorCBActionPerformed(evt);
                          }
                      }
                      );
                      gridBagConstraints4 = new java.awt.GridBagConstraints();
                      gridBagConstraints4.gridx = 1;
                      gridBagConstraints4.gridy = 1;
                      gridBagConstraints4.fill = java.awt.GridBagConstraints.HORIZONTAL;
                      gridBagConstraints4.anchor = java.awt.GridBagConstraints.WEST;
                      behaviorPanel.add(viewBehaviorCB, gridBagConstraints4);
                      
                      
                    enableCollisionC.setToolTipText("Enable Collision Detection");
                      enableCollisionC.setHorizontalTextPosition(javax.swing.SwingConstants.LEFT);
                      enableCollisionC.setText("Enable Collision");
                      enableCollisionC.setEnabled(false);
                      enableCollisionC.addChangeListener(new javax.swing.event.ChangeListener() {
                          public void stateChanged(javax.swing.event.ChangeEvent evt) {
                              enableCollisionCStateChanged(evt);
                          }
                      }
                      );
                      gridBagConstraints4 = new java.awt.GridBagConstraints();
                      gridBagConstraints4.gridx = 1;
                      gridBagConstraints4.gridy = 2;
                      gridBagConstraints4.anchor = java.awt.GridBagConstraints.EAST;
                      behaviorPanel.add(enableCollisionC, gridBagConstraints4);
                      
                      
                    showAllB.setToolTipText("Set the view so all geometry is visible");
                      showAllB.setText("Show All");
                      showAllB.addActionListener(new java.awt.event.ActionListener() {
                          public void actionPerformed(java.awt.event.ActionEvent evt) {
                              showAllBActionPerformed(evt);
                          }
                      }
                      );
                      gridBagConstraints4 = new java.awt.GridBagConstraints();
                      gridBagConstraints4.gridx = 1;
                      gridBagConstraints4.gridy = 0;
                      gridBagConstraints4.anchor = java.awt.GridBagConstraints.EAST;
                      behaviorPanel.add(showAllB, gridBagConstraints4);
                      
                      
                    jLabel4.setText("Warp To");
                      gridBagConstraints4 = new java.awt.GridBagConstraints();
                      gridBagConstraints4.gridx = 0;
                      gridBagConstraints4.gridy = 4;
                      gridBagConstraints4.insets = new java.awt.Insets(0, 0, 0, 4);
                      gridBagConstraints4.anchor = java.awt.GridBagConstraints.EAST;
                      behaviorPanel.add(jLabel4, gridBagConstraints4);
                      
                      
                    warpEditB.setToolTipText("Display the Warp editor window");
                      warpEditB.setText("...");
                      warpEditB.setMargin(new java.awt.Insets(0, 0, 0, 0));
                      warpEditB.addActionListener(new java.awt.event.ActionListener() {
                          public void actionPerformed(java.awt.event.ActionEvent evt) {
                              warpEditBActionPerformed(evt);
                          }
                      }
                      );
                      gridBagConstraints4 = new java.awt.GridBagConstraints();
                      gridBagConstraints4.gridx = 2;
                      gridBagConstraints4.gridy = 4;
                      behaviorPanel.add(warpEditB, gridBagConstraints4);
                      
                      
                    warpCB.addActionListener(new java.awt.event.ActionListener() {
                          public void actionPerformed(java.awt.event.ActionEvent evt) {
                              warpCBActionPerformed(evt);
                          }
                      }
                      );
                      gridBagConstraints4 = new java.awt.GridBagConstraints();
                      gridBagConstraints4.gridx = 1;
                      gridBagConstraints4.gridy = 4;
                      gridBagConstraints4.fill = java.awt.GridBagConstraints.HORIZONTAL;
                      behaviorPanel.add(warpCB, gridBagConstraints4);
                      
                      
                    behaviorPropertiesB.setToolTipText("Edit Behavior Parameters");
                      behaviorPropertiesB.setText("...");
                      behaviorPropertiesB.setMargin(new java.awt.Insets(0, 0, 0, 0));
                      behaviorPropertiesB.addActionListener(new java.awt.event.ActionListener() {
                          public void actionPerformed(java.awt.event.ActionEvent evt) {
                              behaviorPropertiesBActionPerformed(evt);
                          }
                      }
                      );
                      gridBagConstraints4 = new java.awt.GridBagConstraints();
                      gridBagConstraints4.gridx = 2;
                      gridBagConstraints4.gridy = 1;
                      behaviorPanel.add(behaviorPropertiesB, gridBagConstraints4);
                      
                      
                    showAllAxisCB.addActionListener(new java.awt.event.ActionListener() {
                          public void actionPerformed(java.awt.event.ActionEvent evt) {
                              showAllAxisCBActionPerformed(evt);
                          }
                      }
                      );
                      gridBagConstraints4 = new java.awt.GridBagConstraints();
                      behaviorPanel.add(showAllAxisCB, gridBagConstraints4);
                      
                      
                    add(behaviorPanel);
                    
                    
                    
                    add(jPanel2);
                    
                    
                    
                    helpB.setText("Help...");
                      paddingPanel.add(helpB);
                      
                      
                    exitB.setToolTipText("Exit the application");
                      exitB.setText("Exit");
                      exitB.addActionListener(new java.awt.event.ActionListener() {
                          public void actionPerformed(java.awt.event.ActionEvent evt) {
                              exitBActionPerformed(evt);
                          }
                      }
                      );
                      paddingPanel.add(exitB);
                      
                      
                    add(paddingPanel);
                    
    }//GEN-END:initComponents

  private void exitBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitBActionPerformed
// Add your handling code here:
      System.exit(0);
  }//GEN-LAST:event_exitBActionPerformed

  private void behaviorEnableCActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_behaviorEnableCActionPerformed
// Add your handling code here:
      universe.setLive( false );
      BranchGroup bg;
      for(int i=0; i<graphs.size(); i++) {
          bg = (BranchGroup)graphs.get(i);
        BehaviorSetEnable.setBehaviorEnable( bg, behaviorEnableC.isSelected() );
      }
      universe.setLive( true );
  }//GEN-LAST:event_behaviorEnableCActionPerformed

  private void showAllAxisCBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showAllAxisCBActionPerformed
// Add your handling code here:
      showAllAxis = showAllAxisCB.getSelectedIndex();
  }//GEN-LAST:event_showAllAxisCBActionPerformed

  private void behaviorPropertiesBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_behaviorPropertiesBActionPerformed
// Add your handling code here:
      java.awt.Frame frame = (java.awt.Frame)javax.swing.SwingUtilities.getAncestorOfClass( java.awt.Frame.class, this );
      viewBehavior.editParameters( frame );
  }//GEN-LAST:event_behaviorPropertiesBActionPerformed

  private void warpEditBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_warpEditBActionPerformed
// Add your handling code here:
      WarpEditorDialog dialog = new WarpEditorDialog( (java.awt.Frame)javax.swing.SwingUtilities.getAncestorOfClass( java.awt.Frame.class, this ),
                                                      false,
                                                      warpSet,
                                                      universe.getViewingPlatform().getViewPlatformTransform() );
      dialog.setVisible( true );
  }//GEN-LAST:event_warpEditBActionPerformed

  private void warpCBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_warpCBActionPerformed
// Add your handling code here:
      String name = (String)warpCB.getSelectedItem();
      warpSet.warpTo( name, universe.getViewingPlatform().getViewPlatformTransform() );
  }//GEN-LAST:event_warpCBActionPerformed

  private void stripifyBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stripifyBActionPerformed
// Add your handling code here:
      java.awt.Frame frame = (java.awt.Frame)javax.swing.SwingUtilities.getAncestorOfClass( java.awt.Frame.class, this );
      
      StripifyWorker worker = new StripifyWorker( frame, universe, (BranchGroup[])graphs.toArray(new BranchGroup[graphs.size()]), statsDialog );
      worker.start();
  }//GEN-LAST:event_stripifyBActionPerformed

  private void textureEnableCActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_textureEnableCActionPerformed
// Add your handling code here:
      universe.setLive( false );
      BranchGroup bg;
      for(int i=0; i<graphs.size(); i++) {
          bg = (BranchGroup)graphs.get(i);
        ChangeTextureAttributes.setTextureEnable( bg, textureEnableC.isSelected() );
      }
      universe.setLive( true );
  }//GEN-LAST:event_textureEnableCActionPerformed

  private void saveBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveBActionPerformed
// Add your handling code here:
    if (fileChooser==null) {
	String currentDir = new String();
	try {
	    currentDir = System.getProperty("user.dir");
	} catch( SecurityException e ) {}
        fileChooser = new JFileChooser( currentDir );
    }     
    
    ExampleFileFilter j3dFilter = new ExampleFileFilter();
    j3dFilter.setDescription("J3dFly Files");
    j3dFilter.addExtension("j3f");
    
    fileChooser.setFileFilter( j3dFilter );

    int ret = fileChooser.showDialog( this, "Save SceneGraph" );

    if (ret==JFileChooser.APPROVE_OPTION) {
        String filename = fileChooser.getSelectedFile().getAbsolutePath();
        
        String ext = filename.substring( filename.length()-4, filename.length() );
        if (!ext.equalsIgnoreCase(".j3f"))
            filename = filename.concat(".j3f");
        
        long start = System.currentTimeMillis();
        SaveSceneGraph( new java.io.File(filename) );
        //System.out.println("Save Time "+(System.currentTimeMillis()-start));
    }
  }//GEN-LAST:event_saveBActionPerformed

  private void backClipTFActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_backClipTFActionPerformed
// Add your handling code here:
    universe.getViewer().getView().setBackClipDistance( Double.parseDouble( backClipTF.getText() ));
  }//GEN-LAST:event_backClipTFActionPerformed

  private void frontClipTFActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_frontClipTFActionPerformed
// Add your handling code here:
      universe.getViewer().getView().setFrontClipDistance( Double.parseDouble( frontClipTF.getText() ));
  }//GEN-LAST:event_frontClipTFActionPerformed

  private void backClipTFMouseExited(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_backClipTFMouseExited
// Add your handling code here:
      universe.getViewer().getView().setBackClipDistance( Double.parseDouble( backClipTF.getText() ));
  }//GEN-LAST:event_backClipTFMouseExited

  private void frontClipTFMouseExited(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_frontClipTFMouseExited
// Add your handling code here:
      universe.getViewer().getView().setFrontClipDistance( Double.parseDouble( frontClipTF.getText() ));
  }//GEN-LAST:event_frontClipTFMouseExited

  private void configureLoadersBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_configureLoadersBActionPerformed
// Add your handling code here:
      loaderControl.showSelectionDialog( findFrame( this ) );
      //loaderControl.showSelectionDialog( (java.awt.Container)SwingUtilities.getRoot( this ) );
  }//GEN-LAST:event_configureLoadersBActionPerformed

  /**
   * Return the frame which contains the component
   *
   * If component is not in a frame null is returned
   */
  private java.awt.Frame findFrame( java.awt.Component component ) {
      java.awt.Frame ret = null;
      java.awt.Component comp = component;
      
      while( ret==null && comp!=null ) {
          if (comp instanceof java.awt.Frame) {
              ret = (java.awt.Frame)comp;
          } else {
              comp = comp.getParent();
          }
      }
      
      return ret;
  }
  
    
/**
  * Remove all geometry from the scenegraph
  */
  private void clearBActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearBActionPerformed
// Add your handling code here:
      filenameTF.setText("");
      universe.setLive( false );
      BranchGroup bg;
      for(int i=0; i<graphs.size(); i++) {
          bg = (BranchGroup)graphs.get(i);
          universe.getLocale().removeBranchGraph( bg );
      }
      graphs.clear();
      warpSet.clear();
      
      saveB.setEnabled( true );
        compileB.setEnabled( true );
        polygonModeCB.setEnabled( true );
        lightingC.setEnabled( true );
        textureEnableC.setEnabled( true );
        enableCollisionC.setEnabled( true );
        behaviorEnableC.setEnabled( true );
        stripifyB.setEnabled( true );

      if (statsDialog!=null)
          statsDialog.calcGeometryStats( (BranchGroup)null );        // Clear the stats
      universe.setLive(true);
  }//GEN-LAST:event_clearBActionPerformed


/**
  * Compile the scenegraph.
  * Disable all J3dFly features which are unavailable after compile
  */
  private void compileBActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_compileBActionPerformed
// Add your handling code here:
    int response = javax.swing.JOptionPane.showConfirmDialog( this.getParent(),
        "Compiled Scene Graphs cannot be Serialized or changed.\n\nProceed ?",
        "WARNING",
        javax.swing.JOptionPane.YES_NO_OPTION,
        javax.swing.JOptionPane.WARNING_MESSAGE );
    
    if (response==javax.swing.JOptionPane.YES_OPTION) {
        Cursor origCursor = this.getParent().getCursor();
        this.getParent().setCursor( Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR ));
        universe.setLive( false );
        for(int i=0; i<graphs.size(); i++) {
            ((BranchGroup)graphs.get(i)).compile();
        }
        universe.setLive( true );
        this.getParent().setCursor( origCursor );
    }
  }//GEN-LAST:event_compileBActionPerformed

/**
  * Show the Statistics dialog
  */
      int distanceLOD=0;

  private void statisticsBActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_statisticsBActionPerformed
// Add your handling code here:
    if (statsDialog==null) {
        statsDialog = new StatsDialog(  null, toolBG );
        statsDialog.setLocationRelativeTo( this.getParent() );
        statsDialog.setEnable( false );
        statsDialog.show();
    } else {
        statsDialog.setEnable( false );
        statsDialog.show();
    }
    universe.setLive( false );
        
    statsDialog.calcGeometryStats( (BranchGroup[])graphs.toArray(new BranchGroup[graphs.size()]) );
    universe.setLive( true );
    statsDialog.setEnable( true );
  }//GEN-LAST:event_statisticsBActionPerformed

/**
  * Enable/Disable lighting for all Appearances in the scene graph
  */
  private void lightingCItemStateChanged (java.awt.event.ItemEvent evt) {//GEN-FIRST:event_lightingCItemStateChanged
// Add your handling code here:
    boolean wasLive = universe.getLive();
    universe.setLive( false );
    BranchGroup bg;
    for(int i=0; i<graphs.size(); i++) {
        bg = (BranchGroup)graphs.get(i);
        ChangeMaterial.setLightingEnable( bg, lightingC.isSelected() );
    }
    universe.setLive( wasLive );
  }//GEN-LAST:event_lightingCItemStateChanged

/**
  * Change the Polygon mode for all Appearance components in the scene graph
  */
  private void polygonModeCBItemStateChanged (java.awt.event.ItemEvent evt) {//GEN-FIRST:event_polygonModeCBItemStateChanged
// Add your handling code here:
    universe.setLive( false );
    int mode;
    switch (polygonModeCB.getSelectedIndex()) {
        case 0:
            mode = PolygonAttributes.POLYGON_FILL;
            break;
        case 1:
            mode = PolygonAttributes.POLYGON_LINE;
            break;
        case 2:
            mode = PolygonAttributes.POLYGON_POINT;
            break;
        default:
            throw new RuntimeException("Unrecognised PolygonMode");
    }
    BranchGroup bg;
    for(int i=0; i<graphs.size(); i++) {
        bg = (BranchGroup)graphs.get(i);
        ChangePolygonAttributes.setPolygonMode( bg, mode, true );
    }
    universe.setLive( true );
  }//GEN-LAST:event_polygonModeCBItemStateChanged

/**
  * Load some geometry into the SceneGraph
  */
  private void loadBActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadBActionPerformed
// Add your handling code here:
    if (fileChooser==null) {
	String currentDir = new String();
	try {
	    currentDir = System.getProperty("user.dir");
	} catch( SecurityException e ) {}
        fileChooser = new JFileChooser( currentDir );
    }     
    loaderControl.setFileFilter( fileChooser );
    

    int ret = fileChooser.showDialog( this, "Load Geometry" );

    if (ret==JFileChooser.APPROVE_OPTION) {
        loadGeometry( fileChooser.getSelectedFile() );
    }
  }//GEN-LAST:event_loadBActionPerformed

/**
  * Enable/Disable collision detection for viewplatform behavior.
  * If necessary set the capability bits in the scenegraph to 
  * enable picking of geometry which is how the collision detection is
  * implemented
  */
  private void enableCollisionCStateChanged (javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_enableCollisionCStateChanged
    // Add your handling code here:
      if (viewBehavior==null) {
          return;
      }
      
    collisionEnabled = enableCollisionC.isSelected();
    
    if (collisionEnabled && !collisionCapabilitySet ) {
        //collisionControl = new VPDriveCollision();
        collisionControl = new VPDefaultCollision();
        
        BranchGroup bg;
        for(int i=0; i<graphs.size(); i++) {
            bg = (BranchGroup)graphs.get(i);
            setCollisionCapabilityBits( bg );
        }
        collisionCapabilitySet = true;
        collisionControl.setCollisionLocale( universe.getLocale() );
        collisionControl.setVPSweptVolume( new SimpleSweptVolume() );
        viewBehavior.setCollisionControl( collisionControl );
    }
    
    viewBehavior.setCollisionEnabled( collisionEnabled );
  }//GEN-LAST:event_enableCollisionCStateChanged

  /**
   * Change the collisionController to the new object.
   * Copy the state from the old controller to the new
   */
  private void changeCollisionController( VPDefaultCollision newController ) {
      VPDefaultCollision oldController = collisionControl;
      collisionControl = newController;
      collisionControl.setCollisionLocale( universe.getLocale() );
      
      if (oldController==null) {    
        collisionControl.setVPSweptVolume( new SimpleSweptVolume() );
      } else {
          collisionControl.setVPSweptVolume( oldController.getVPSweptVolume() );
      }
  }
  
  private void setCollisionCapabilityBits( BranchGroup bg ) {
        universe.setLive(false);
        NodeChangeProcessor proc = new NodeChangeProcessor() {
          public void changeNode( Node node ) {
            com.sun.j3d.utils.picking.PickTool.setCapabilities( node, com.sun.j3d.utils.picking.PickTool.INTERSECT_COORD );
          }
        };

        TreeScan.findNode( bg, javax.media.j3d.Shape3D.class, proc, false, true );
        universe.setLive( true );
  }
  
  /** Set the ViewPlatform behavior
   * @param behaviorName The name of the desired behavior in the JComboBox
   * @param collisionEnabled Enable/Disable collision detection
   */
  public void setVPBehavior( String behaviorName, boolean collisionEnabled ) {
      Object item = null;
      for(int i=0; i<viewBehaviorCB.getItemCount() && item==null; i++)
          if ( viewBehaviorCB.getItemAt( i ).equals( behaviorName ) )
              item = viewBehaviorCB.getItemAt(i);
      
      if (item==null) return;
      
      viewBehaviorCB.setSelectedItem( item );
      enableCollisionC.setSelected( collisionEnabled );
  }
  
  private void viewBehaviorCBActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_viewBehaviorCBActionPerformed
    // Add your handling code here:
    if (viewBehaviorCB.getSelectedItem().equals("Hover")) {
        HoverBehavior hover = new HoverBehavior( universe.getViewer().getView() );
        hover.setTarget( universe.getViewingPlatform().getViewPlatformTransform() );
        hover.setSchedulingBounds( bigBounds );
        hover.setCollisionEnabled( collisionEnabled );
        if (collisionControl!=null) {
            if (collisionControl instanceof VPDriveCollision)
                changeCollisionController( new VPDefaultCollision() );
            hover.setCollisionControl( collisionControl );
        }
        viewBehavior = hover;
        universe.getViewingPlatform().setViewPlatformBehavior( hover );
        enableCollisionC.setEnabled( true );
        behaviorPropertiesB.setEnabled( true );
    } else if (viewBehaviorCB.getSelectedItem().equals("Drive")) { 
        HoverBehavior hover = new HoverBehavior( universe.getViewer().getView() );
        hover.setTarget( universe.getViewingPlatform().getViewPlatformTransform() );
        hover.setSchedulingBounds( bigBounds );
        
        hover.setCollisionEnabled( collisionEnabled );
        changeCollisionController( new VPDriveCollision() );
        hover.setCollisionControl( collisionControl );
        enableCollisionC.setSelected( true );
        collisionEnabled = true;
        
        viewBehavior = hover;
        universe.getViewingPlatform().setViewPlatformBehavior( hover );
        enableCollisionC.setEnabled( true );
        behaviorPropertiesB.setEnabled( true );
    } else if (viewBehaviorCB.getSelectedItem().equals("Fly")) {
        FlyBehavior fly = new FlyBehavior();
        fly.setTarget( universe.getViewingPlatform().getViewPlatformTransform() );
        fly.setSchedulingBounds( bigBounds );
        fly.setCollisionEnabled( collisionEnabled );
        if (collisionControl!=null) {
            if (collisionControl instanceof VPDriveCollision)
                changeCollisionController( new VPDefaultCollision() );
            fly.setCollisionControl( collisionControl );
        }
        viewBehavior = fly;
        universe.getViewingPlatform().setViewPlatformBehavior( fly );
        enableCollisionC.setEnabled( true );
        behaviorPropertiesB.setEnabled( true );
    }  else if (viewBehaviorCB.getSelectedItem().equals("Orbit")) {
        viewBehavior = null;
        OrbitBehavior orbit = new com.sun.j3d.utils.behaviors.vp.OrbitBehavior( universe.getCanvas(), OrbitBehavior.REVERSE_ROTATE | OrbitBehavior.REVERSE_TRANSLATE );
        orbit.setSchedulingBounds( bigBounds );
        computeWorldBounds();
        if (worldBounds!=null) {
            Point3d center = new Point3d();
            worldBounds.getCenter( center );
            orbit.setRotationCenter( center );
        }
        universe.getViewingPlatform().setViewPlatformBehavior( orbit );
        enableCollisionC.setEnabled( false );
        behaviorPropertiesB.setEnabled( false );
    }  else if (viewBehaviorCB.getSelectedItem().equals("None")) {
        viewBehavior = null;
        universe.getViewingPlatform().setViewPlatformBehavior( null );
        enableCollisionC.setEnabled( false );
        behaviorPropertiesB.setEnabled( false );
    }
  }//GEN-LAST:event_viewBehaviorCBActionPerformed

/**
  * Move the view platform so that all geometry fit inside view frustum
  */
  private void showAllBActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_showAllBActionPerformed
    // Add your handling code here:
      computeWorldBounds();
      if (worldBounds==null) return;
      
      setViewpoint( universe.getViewingPlatform().getViewPlatformTransform(), worldBounds, universe.getViewer().getView() );
  }//GEN-LAST:event_showAllBActionPerformed

  /**
   * Ensures the global variable worldBounds is upto date
   */
  private void computeWorldBounds() {
        if (graphs.size()==0 || gotValidBounds) return;
        
        universe.setLive( false );
        worldBounds= (BoundingSphere)((BranchGroup)graphs.get(0)).getBounds();
        for(int i=1; i<graphs.size(); i++)
            worldBounds.combine( ((BranchGroup)graphs.get(i)).getBounds() );
        
        universe.setLive( true );
        gotValidBounds=true;
  }
  
  /**
    * Load some geometry from the specified file and add it to the
    * existing scene graph
    */
    public void loadGeometry( java.io.File file ) {
        normalCursor = this.getCursor();
        this.setCursor( Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR ));
        WorkingDialog progressMonitor = new WorkingDialog( (java.awt.Frame)SwingUtilities.getAncestorOfClass( java.awt.Frame.class, this ) );
        progressMonitor.setVisible( true );
        try {
            loaderControl.loadFile( file, this, progressMonitor );
            filenameTF.setText( file.getName() );
        } catch( com.sun.j3d.demos.utils.loadercontrol.UnsupportedFormatException ef ) {
            JOptionPane.showMessageDialog( SwingUtilities.getAncestorOfClass( java.awt.Frame.class, this ), "No Loader for File format", "Error", JOptionPane.ERROR_MESSAGE );
            this.setCursor( normalCursor );
        }
    }

    /**
      * Write the rootBG subgraph to <code>file</code>
    */
    private void SaveSceneGraph( java.io.File file ) {
        universe.setLive( false );
        
        J3dFlyData flyData = new J3dFlyData();
        flyData.addProperty( "WarpSet", warpSet);
        flyData.addProperty( "AnimationSet", animationSet );
        
        try {
            SceneGraphFileWriter writer = new SceneGraphFileWriter( file , null, false, "Generated by J3dFly", flyData );
            BranchGroup[] bgs = (BranchGroup[])graphs.toArray( new BranchGroup[1] );
            for(int i=0; i<bgs.length; i++) {
                writer.writeBranchGraph( bgs[i] );
            }
            
            writer.close();
        } catch( java.io.IOException e ) {
            JOptionPane.showMessageDialog( this, "Failed to save file", "Error", JOptionPane.ERROR_MESSAGE );
        } catch( com.sun.j3d.demos.utils.scenegraph.io.UnsupportedUniverseException e2 ) {
            e2.printStackTrace();
        }
        universe.setLive( true );
    }

  /** 
    * Check if the graph contains any lights and if it needs lights
    * If it does not have lights but requires them then ask the user if they 
    * would like to add some
    *
    * Returns true if no changes made, returns false is scene graph updated
    *
    * If lightBG is not null, the lights will be added to it,
    * otherwise they will be added in a new BranchGraph
    *
    * Scene graph must not be live
    */
  private void checkForLights() {
    SceneGraphChecker checker = new SceneGraphChecker();
    boolean needsLights = false;
    
    BranchGroup bg;
    for(int i=0; i<graphs.size(); i++) {
        bg = (BranchGroup)graphs.get(i);
        needsLights |= checker.needsLights( bg );

        if (checker.getHasLights()) {
            lightingC.setSelected( true );
        }
    }
    
    if (!needsLights) return;

    int answer = JOptionPane.showConfirmDialog( null, "SceneGraph does not contain any Lights\nBut some geometry is expecting Lighting\n\nWould you like to add some ?",
                            "Add Lights ?", JOptionPane.YES_NO_OPTION );

    if (answer==JOptionPane.YES_OPTION) {
        createDefaultLights();
        lightingC.setSelected( true );
      }
  }

  private void createDefaultLights() {
        // Set up the directional lights
        BranchGroup lightBG = new BranchGroup();
        BoundingSphere bounds = new BoundingSphere( new Point3d(), Float.MAX_VALUE );
        Color3f light1Color = new Color3f(0.7f, 0.7f, 0.7f);
        Vector3f light1Direction  = new Vector3f(4.0f, -7.0f, -12.0f);
        Color3f light2Color = new Color3f(0.3f, 0.3f, 0.4f);
        Vector3f light2Direction  = new Vector3f(-4.0f, 7.0f, 12.0f);

        DirectionalLight light1 = new DirectionalLight(light1Color, light1Direction);
        light1.setInfluencingBounds(bounds);
        lightBG.addChild(light1);

        DirectionalLight light2  = new DirectionalLight(light2Color, light2Direction);
        light2.setInfluencingBounds(bounds);
        lightBG.addChild(light2);
        
        universe.addBranchGraph( lightBG );
    }

/**
 * Set the transform so that everything inside the BoundingSphere is
 * visible within the view frustum
 */
private void setViewpoint( TransformGroup tg, BoundingSphere sceneBounds, View view ) {
    Transform3D viewTrans = new Transform3D();
    Transform3D eyeTrans = new Transform3D();

    // point the view at the center of the object
    Point3d center = new Point3d();
    sceneBounds.getCenter(center);
    double radius = sceneBounds.getRadius();
    Point3d eyePos = new Point3d(center);
    Vector3d up = new Vector3d();

    // pull the eye back far enough to see the whole object
    double eyeDist = radius / Math.tan(view.getFieldOfView() / 2.0);
    switch( showAllAxis ) {
        case 0 :
            eyePos.x += eyeDist;
            up.y = 1;
            break;
        case 1 :
            eyePos.y += eyeDist;
            up.z = -1;
            break;
        case 2 :
            eyePos.z += eyeDist;
            up.y = 1;
            break;
        case 3 :
            eyePos.x -= eyeDist;
            up.y = 1;
            break;
        case 4 :
            eyePos.y -= eyeDist;
            up.z = -1;
            break;
        case 5 :
            eyePos.z -= eyeDist;
            up.y = 1;
            break;
    }
            
    viewTrans.setIdentity();
    viewTrans.lookAt( eyePos, center, up ); 
    viewTrans.invert();
    // If we have moved the model behind the back clip distance then change
    // the back clip distance
    if (universe.getViewer().getView().getBackClipDistance()<eyeDist) {
        universe.getViewer().getView().setBackClipDistance( eyeDist );
        backClipTF.setText( Double.toString( eyeDist ));
    }

    // set the view transform
    tg.setTransform(viewTrans);
}
// Variables declaration - do not modify//GEN-BEGIN:variables
private javax.swing.JPanel geometryPanel;
private javax.swing.JButton loadB;
private javax.swing.JButton statisticsB;
private javax.swing.JButton compileB;
private javax.swing.JButton saveB;
private javax.swing.JPanel jPanel1;
private javax.swing.JLabel jLabel1;
private javax.swing.JScrollPane jScrollPane1;
private javax.swing.JTextField filenameTF;
private javax.swing.JButton clearB;
private javax.swing.JButton configureLoadersB;
private javax.swing.JButton stripifyB;
private javax.swing.JPanel viewPanel;
private javax.swing.JComboBox polygonModeCB;
private javax.swing.JCheckBox lightingC;
private javax.swing.JPanel clipP;
private javax.swing.JLabel jLabel2;
private javax.swing.JLabel jLabel3;
private javax.swing.JTextField frontClipTF;
private javax.swing.JTextField backClipTF;
private javax.swing.JCheckBox textureEnableC;
private javax.swing.JCheckBox behaviorEnableC;
private javax.swing.JPanel behaviorPanel;
private javax.swing.JComboBox viewBehaviorCB;
private javax.swing.JCheckBox enableCollisionC;
private javax.swing.JButton showAllB;
private javax.swing.JLabel jLabel4;
private javax.swing.JButton warpEditB;
private javax.swing.JComboBox warpCB;
private javax.swing.JButton behaviorPropertiesB;
private javax.swing.JComboBox showAllAxisCB;
private javax.swing.JPanel jPanel2;
private javax.swing.JPanel paddingPanel;
private javax.swing.JButton helpB;
private javax.swing.JButton exitB;
// End of variables declaration//GEN-END:variables

/**
  * Callback from scene loader when load is complete
  */
public void sceneLoaded( int status, com.sun.j3d.loaders.Scene scene,int loaderID, java.io.File file, Exception ex ) {
    if (status==LoaderControlListener.LOAD_FAILED) {
        JOptionPane.showMessageDialog( this, "Error Loading file "+file.getName(), "Error", JOptionPane.ERROR_MESSAGE );
        if (ex!=null) ex.printStackTrace();
        this.setCursor( normalCursor );
        return;
    }
    
    if (collisionCapabilitySet) {
        setCollisionCapabilityBits( scene.getSceneGroup() );
    }
    
    universe.setLive( false );
    
    scene.getSceneGroup().setCapability( BranchGroup.ALLOW_DETACH );

    // Benchmarking code
    //com.sun.j3d.demos.utils.benchmark.SpinBehavior test = new com.sun.j3d.demos.utils.benchmark.SpinBehavior( universe.getViewer().getViewingPlatform().getViewPlatformTransform(), scene.getSceneGroup() );

    universe.addBranchGraph( scene.getSceneGroup() );
    graphs.add( scene.getSceneGroup() );


    if (warpSet!=null) {
        warpSet.warpTo( warpSet.getDefaultWarp(), universe.getViewingPlatform().getViewPlatformTransform() );
    }    
    
    checkForLights();
    if (statsDialog!=null) {
        statsDialog.calcGeometryStats( (BranchGroup[])graphs.toArray(new BranchGroup[graphs.size()]) );
    }
    
    /*
     // Show the bounds of the object just loaded
    BranchGroup t = new BranchGroup();
    com.sun.j3d.demos.j3dedit.j3deditor.ShowBoundsBehavior b = new com.sun.j3d.demos.j3dedit.j3deditor.ShowBoundsBehavior( t );
    b.showBounds( (BranchGroup)graphs.get(0) );
    universe.addBranchGraph( t );
     */
    
    gotValidBounds = false;
    
    if (warpSet==null || warpSet.getDefaultWarp()==null ) {        
        int confirm = JOptionPane.showConfirmDialog( null, "Set View to show entire scene ?", "Set View Position", JOptionPane.YES_NO_OPTION );
        
        if (confirm==JOptionPane.YES_OPTION) {
            computeWorldBounds();
            if (worldBounds==null) return;

            setViewpoint( universe.getViewingPlatform().getViewPlatformTransform(), worldBounds, universe.getViewer().getView() );
        }
    }   
    universe.setLive( true );
    this.setCursor( normalCursor );
    }
    
    /**
     * Callback for J3fLoaderListener
     */
    public void loadingJ3f( J3fLoader loader ) {
        Object data=null;
        try {
            data = loader.getFileUserData();
        } catch( java.io.IOException e ) {}
        
        if (data instanceof J3dFlyData) {
            // Merge existing warpSet with the new one
            Object prop = ((J3dFlyData)data).getProperty("WarpSet");
            if (prop!=null)
                warpSet.merge( (WarpSet)prop );
        }
    }
}

