/*
 *  @(#)J3dfly.java 1.14 01/03/01 16:11:08
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.j3dfly;

import javax.swing.JWindow;
import javax.swing.JFrame;
import java.lang.Package;

import javax.media.j3d.Canvas3D;
import com.sun.j3d.demos.utils.developmenttools.DevelopmentSimpleUniverse;
import com.sun.j3d.demos.utils.help.Help;

/**
 * @author Paul Byrne
 * @version	1.14, 03/01/01
 */

public class J3dfly implements java.awt.event.WindowListener {

    private ControlPanel controlPanel;
    private java.awt.Container container;
    
    private boolean fullscreen = false;
    
    /** Creates new form j3dfly */
    public J3dfly( String[] args) {
        
        checkProperties( args );
        Help.initialize( "com/sun/j3d/demos/j3dfly/help/J3dFlyHelp" );
        
        new J3dVersionCheck( this );
        
        if (fullscreen) {
            container = new javax.swing.JWindow();
        } else {
            container = new javax.swing.JFrame();
        }
        
        Canvas3D canvas3D = new Canvas3D( DevelopmentSimpleUniverse.getPreferredConfiguration() );
        canvas3D.setSize( 500, 500 );
        DevelopmentSimpleUniverse universe = new DevelopmentSimpleUniverse( canvas3D );
        
        universe.getViewer().getView().setBackClipDistance( 100.0 );
        controlPanel = new ControlPanel( universe );
        
        // Set the defauly ViewPlatform behavior
        controlPanel.setVPBehavior( "Orbit", false );
        
        if (container instanceof javax.swing.JWindow) {
            ((JWindow)container).getContentPane().add( java.awt.BorderLayout.WEST, controlPanel );
            ((JWindow)container).getContentPane().add( java.awt.BorderLayout.CENTER, canvas3D );
            showControls(false);
            ((JWindow)container).addWindowListener( this );
            container.setSize( java.awt.Toolkit.getDefaultToolkit().getScreenSize() );
        } else if (container instanceof javax.swing.JFrame ) {
            ((JFrame)container).getContentPane().add( java.awt.BorderLayout.WEST, controlPanel );
            ((JFrame)container).getContentPane().add( java.awt.BorderLayout.CENTER, canvas3D );
            ((JFrame)container).pack();
            ((JFrame)container).addWindowListener( this );
            ((JFrame)container).setTitle( "Sun Java 3D(tm) Fly Through 1.0 beta" );
        } else
            throw new RuntimeException("Internal Error");
        
        canvas3D.addKeyListener( new KeyControls( this, controlPanel ) );
        
        if (args.length!=0 && !args[args.length-1].startsWith("-")) {
            controlPanel.loadGeometry( new java.io.File( args[args.length-1] ));
        }
        
        container.show();
    }
    
    /**
      * Show/Hide the control panel
      */
    protected void showControls( boolean visible ) {
        if (visible) {
            if (controlPanel.getParent()==null) {
                // Make the control panel visible
                if (container instanceof javax.swing.JWindow) {
                    ((JWindow)container).getContentPane().add( java.awt.BorderLayout.WEST, controlPanel );
                } else if (container instanceof javax.swing.JFrame) {
                    ((JFrame)container).getContentPane().add( java.awt.BorderLayout.WEST, controlPanel );
                } else
                    throw new RuntimeException("Unsupported window class");
                container.validate();
            }
        } else {
            if (controlPanel.getParent()!=null) {
                // Make the control panel invisible
                if (container instanceof javax.swing.JWindow) {
                    ((JWindow)container).getContentPane().remove( controlPanel );
                } else if (container instanceof javax.swing.JFrame) {
                    ((JFrame)container).getContentPane().remove( controlPanel );
                } else
                    throw new RuntimeException("Unsupported window class");
                container.validate();
            }
        }
    }
    
    /**
      * Check the value of any properties associated with J3dFly
      */
    private void checkProperties( String[] args ) {
        if (System.getProperty("j3dfly.fullscreen")!=null) fullscreen = true;
        
        for(int i=0; i<args.length; i++) {
            if (args[i].equalsIgnoreCase("-fullscreen")) {
                fullscreen = true;
            } else if (args[i].equalsIgnoreCase("-help")) { 
                System.out.println("Usage : fly [options] [filename]");
                System.out.println("Options : -fullscreen");
            }
        }
        
    }

    /**
      * Exit the app
      */
    protected void exit() {
        System.exit(0);
    }
    
    /**
      * Return the top level container of J3dFly
      */
    protected java.awt.Container getContainer() {
        return container;
    }
    
    /**
    * @param args the command line arguments
    */
    public static void main (String args[]) {
        new J3dfly ( args );
    }


    public void windowDeactivated(final java.awt.event.WindowEvent p1) {
    }
    public void windowClosed(final java.awt.event.WindowEvent p1) {
        exit();
    }
    public void windowDeiconified(final java.awt.event.WindowEvent p1) {
    }
    public void windowOpened(final java.awt.event.WindowEvent p1) {
    }
    public void windowIconified(final java.awt.event.WindowEvent p1) {
    }
    public void windowClosing(final java.awt.event.WindowEvent p1) {
        if (container instanceof javax.swing.JWindow) {
            ((JWindow)container).dispose();
        } else if (container instanceof javax.swing.JFrame ) {
            ((JFrame)container).dispose();
        } else
            throw new RuntimeException("Internal Error");
     }
    public void windowActivated(final java.awt.event.WindowEvent p1) {
    }




/**
 * Check which version of Java3D is installed 
 */
class J3dVersionCheck {
    private boolean patchNext = false;
    
    private int getTokenEnd( int start, String str ) {
        int ret = start;
        boolean finished = false;
        char c;
        
        while( ret < str.length() && !finished ) {
            c = str.charAt( ret );
            if ( c=='.' || c==' ')
                finished = true;
            else if ( c=='_' ) {
                finished = true;
                patchNext = true;
            } else {
                ret++;
            }
        }
        
        return ret;
    }
    
    /**
     * Check which version of Java3D we are using
     */
    public J3dVersionCheck( J3dfly fly ) {
        Package p = Package.getPackage( "javax.media.j3d" );
        String version = p.getImplementationVersion();

	if (version==null) {
	    System.out.println("Unable to determine Java3D version");
	    return;
	}
        
        // Syntax of version number is <int>.<int>[.<int[_<int>]][_<int>] [<str>]
        // 1.2   1.2.1  1.2.1_001, 1.3_001,  1.2 text
        
        //version = "1.2";
        //version = "1.2.1";
        //version = "1.2.1_001";
        //version = "1.3_001";
        //version = "1.3_001 test data";
        //version = "1.2 test data";
        //version = "1.2.1 test data";
        //version = "1.2.1_001 test data";
       
        int majorV=0;
        int minorV=0;
        int subV=0;
        int patchV=0;
        
        int tokenStart = 0;
        int tokenEnd = getTokenEnd( tokenStart, version );
        
        try {
            majorV = Integer.parseInt( version.substring( tokenStart, tokenEnd ));

            tokenStart = tokenEnd+1;
            tokenEnd = getTokenEnd( tokenStart, version );
            
            minorV = Integer.parseInt( version.substring( tokenStart, tokenEnd ));

            if (patchNext) {
                tokenStart = tokenEnd+1;
                tokenEnd = getTokenEnd( tokenStart, version );
                if (tokenStart!=tokenEnd)
                    patchV = Integer.parseInt( version.substring( tokenStart, tokenEnd ));
            } else {                    
                tokenStart = tokenEnd+1;
                tokenEnd = getTokenEnd( tokenStart, version );

                if (tokenStart!=tokenEnd) {
                        subV = Integer.parseInt( version.substring( tokenStart, tokenEnd ));
                    } else
                        patchV = Integer.parseInt( version.substring( tokenStart, tokenEnd ));
                
                tokenStart = tokenEnd+1;
                tokenEnd = getTokenEnd( tokenStart, version );

                if (tokenStart!=tokenEnd)
                    patchV = Integer.parseInt( version.substring( tokenStart, tokenEnd ));
            }
        } catch(NumberFormatException e) {
        }
        
        if ( minorV<3 && subV<1 ) {      
            // Not 1.2.1 or greater
            String[] options = new String[] { "Continue", "Exit" };
            if (javax.swing.JOptionPane.showOptionDialog( null, "J3dFly is designed for use with Java3D 1.2.1 or later.\nUsing it with older versions of Java3D will cause exceptions.",
                "Old Java3D Version", javax.swing.JOptionPane.DEFAULT_OPTION, 
                javax.swing.JOptionPane.WARNING_MESSAGE, null, options, options[0] )==1)
                    fly.exit();
        }
    }
}

}
