/*
 *  @(#)KeyControls.java 1.4 00/12/19 15:41:44
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.j3dfly;

import java.awt.event.KeyEvent;
import java.awt.BorderLayout;

/**
 *  Monitors user key strokes in the canvas3D checking for sequences
 *  that control the app. This does not perform any java3D user motion
 *
 * @author  Paul Byrne
 * @version 1.4, 12/19/00
 */
public class KeyControls implements java.awt.event.KeyListener {

    private J3dfly j3dfly;
    private ControlPanel controlPanel;
    
    private static final int showControlsKey = KeyEvent.VK_C;   // Show/Hide controls
    private static final int helpKey = KeyEvent.VK_H;           // Show help  
    private static final int quitKey = KeyEvent.VK_Q;           // Quit the app
    
    /** Creates new KeyControls */
    public KeyControls( J3dfly j3dfly, ControlPanel controlPanel ) {
        this.j3dfly = j3dfly;
        this.controlPanel = controlPanel;
    }

    public void keyReleased(final java.awt.event.KeyEvent p1) {
    }
    
    public void keyPressed(final java.awt.event.KeyEvent evt) {
        switch (evt.getKeyCode()) {
            case showControlsKey:
                if (controlPanel.getParent()==null) {
                    j3dfly.showControls( true );
                } else {
                    j3dfly.showControls( false );
                }
                break;
            case helpKey:
                showHelpDialog();
                break;
            case KeyEvent.VK_HELP:
                showHelpDialog();
                break;
            case quitKey:
                j3dfly.exit();
                break;
        }
    }
    
    public void keyTyped(final java.awt.event.KeyEvent evt) {
    }
    
    public void showHelpDialog() {
        javax.swing.JOptionPane.showMessageDialog( j3dfly.getContainer(), 
            "c - Show/Hide Control Panel \n h - Display this Help Message \n q - Quit the application \n esc - Enable/Disable View Behaviors \n Home - Move to original location",
            "J3dFly Help", 
            javax.swing.JOptionPane.INFORMATION_MESSAGE );
    }
}

