/*
 *  @(#)Capabilities.java 1.4 00/12/19 15:46:49
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.capability;

import java.util.ArrayList;

/** 
 * Convenience class for extracting capability information from a SceneGraph Object
 *
 * @author Paul Byrne
 * @version	1.4, 12/19/00
 */
public class Capabilities extends Object {
 
  /** Print a list of SET capabilities to stdout, one capability per line
 * @param obj The Object for which to print the capabilities
 */
  public static void printCapabilities( javax.media.j3d.SceneGraphObject obj ) {
    ArrayList list = new ArrayList();
    getCapabilities( obj, list );
    java.util.Iterator it = list.iterator();
    while(it.hasNext())
      System.out.println( (String)it.next() );
  }
  
  /** Extract the names of all the SET capabilities in the object.
 * The names (Strings) are appended to the arrayList
 * @param obj The object for which to extract the capability strings
 * @param capabilityStrings The ArrayList to which the capability names will be appended
 */
  public static void getCapabilities( javax.media.j3d.SceneGraphObject obj,
                                         java.util.ArrayList capabilityStrings ) {

    int value;
    String str;
    Class cl = obj.getClass();

    java.lang.reflect.Field[] fields = cl.getFields();

    try {
      for(int i=0; i<fields.length; i++) {
        str = fields[i].getName();
        value = fields[i].getInt( fields[i] );
        if (str.indexOf("ALLOW") != -1 || str.indexOf("ENABLE_")!=-1) {
          if (obj.getCapability(value))
          capabilityStrings.add( str );
        }
      }
    } catch(Exception e) {
      e.printStackTrace();
      throw new RuntimeException("Internal Error");
    }
  }
  
  /**
   * Return an array of capability bits for the object
   *
   * If no capabilities are set then an array of length 0 is returned
   *
   * @param obj The object for which to extract the capability bits
   */
  public static int[] getCapabilities( javax.media.j3d.SceneGraphObject obj ) {
      ArrayList bits = new ArrayList();
      
    int value;
    String str;
    Class cl = obj.getClass();

    java.lang.reflect.Field[] fields = cl.getFields();

    try {
      for(int i=0; i<fields.length; i++) {
        str = fields[i].getName();
        value = fields[i].getInt( fields[i] );
        if (str.indexOf("ALLOW_") != -1 || str.indexOf("ENABLE_")!=-1 ) {
          if (obj.getCapability(value))
            bits.add( new Integer(value) );
        }
      }
    } catch(Exception e) {
      e.printStackTrace();
      throw new RuntimeException("Internal Error");
    }
    
    int[] ret = new int[ bits.size() ];
    for(int i=0; i<ret.length; i++)
        ret[i] = ((Integer)bits.get(i)).intValue();
    
    return ret;
  }
}

