/*
 *  @(#)DevelopmentLocale.java 1.3 00/12/19 15:48:24
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
 
package com.sun.j3d.demos.utils.developmenttools;

import java.util.ArrayList;
import java.util.Iterator;

import javax.media.j3d.VirtualUniverse;
import javax.media.j3d.HiResCoord;
import javax.media.j3d.BranchGroup;

/** 
 *
 * @author  Paul Byrne
 * @version 1.3 12/19/00
 */
public class DevelopmentLocale extends javax.media.j3d.Locale  {
  
  //private SceneGraphLiveListener liveListener = null;
  //private SceneGraphChangeListener changeListener = null;
  
  private ArrayList liveListeners = new ArrayList();
  private ArrayList changeListeners = new ArrayList();
  
  private ArrayList branchGraphs = new ArrayList();
  private BranchGroup branchGraphArray[] = new BranchGroup[0];
  private BranchGroup hiddenBranchGraph = null;
  private boolean isLive=true;
    
  /** Creates new Locale */
  public DevelopmentLocale( VirtualUniverse universe ) {
    super(universe);
    init();
  }
  
  public DevelopmentLocale( VirtualUniverse universe, HiResCoord hiRes ) {
    super( universe, hiRes );
    init();
  }
  
  public DevelopmentLocale( VirtualUniverse universe ,int[] x, int[] y, int[] z ) {
    super( universe, x, y, z );
    init();
  }
  
  private void init() {
    isLive = true;
  }
  
  /**
    * Hidden BranchGraph is always the last branchgraph in the locale.
    * It is not returned by getBranchGraphs(), it is not included in numBranchGraphs 
    * and no listeners are notified when it is set
    * The Hidden BranchGraph is made live and not live when setLive is called
    */
  public void setHiddenBranchGraph( BranchGroup hiddenBranchGraph ) {
    this.hiddenBranchGraph = hiddenBranchGraph;
    hiddenBranchGraph.setCapability( BranchGroup.ALLOW_DETACH );
    if (isLive)
      super.addBranchGraph( hiddenBranchGraph );
  }
  
  /**
    * Return the hidden branch graph
    */
  public BranchGroup getHiddenBranchGraph() {
    return hiddenBranchGraph;
  }
  
  public void addBranchGraph( BranchGroup branchGroup ) {
    
    boolean wasLive = isLive;
    
    //System.out.println("Adding "+branchGroup );
    
    if (isLive) setLive(false);
    
    if (branchGroup.isCompiled())
      javax.swing.JOptionPane.showMessageDialog(null, "Compiled Branch Group's can not be viewed, remove the .compile() from your source code.", 
            "Compiled Branch Group", javax.swing.JOptionPane.WARNING_MESSAGE );
    else {
      branchGroup.setCapability( BranchGroup.ALLOW_DETACH );
      branchGraphs.add( branchGroup );
    }
  
    updateBranchGraphArray();
    
    for(int i=0; i<changeListeners.size(); i++)
      ((SceneGraphChangeListener)changeListeners.get(i)).graphAdded( branchGroup );
        
    if (isLive)
      super.addBranchGraph( branchGroup );
    
    if (wasLive)
      setLive( true );
  }
  
  public void removeBranchGraph( BranchGroup branchGroup ) {
    
    branchGraphs.remove( branchGraphs.indexOf( branchGroup ) );
    
    updateBranchGraphArray();
    
    if (isLive)
      super.removeBranchGraph( branchGroup );
    
    for(int i=0; i<changeListeners.size(); i++) 
      ((SceneGraphChangeListener)changeListeners.get(i)).graphRemoved( branchGroup );

  }
  
  public void replaceBranchGraph( BranchGroup oldGroup, BranchGroup newGroup ) {
    for(int i=0; i<changeListeners.size(); i++)
      ((SceneGraphChangeListener)changeListeners.get(i)).graphReplaced( oldGroup, newGroup );
        
    newGroup.setCapability( BranchGroup.ALLOW_DETACH );
    branchGraphs.set( branchGraphs.indexOf( oldGroup ), newGroup );
    
    updateBranchGraphArray();
    if (isLive)
      super.replaceBranchGraph( oldGroup, newGroup );    
  }
  
  public int numBranchGraphs() {
    return branchGraphs.size();
  }
    
  public synchronized void setLive( boolean live ) {
    if ( isLive == live ) return;
    
    BranchGroup bg;
    
    Iterator it = branchGraphs.iterator();
    while( it.hasNext() ) {
      bg = (BranchGroup)it.next();
      //System.out.println("BranchGroup "+bg);
      if (live) {
        super.addBranchGraph( bg );
      } else {
        bg.detach();
      }
    }
    
    if (hiddenBranchGraph!=null) {
      if (live)
        super.addBranchGraph( hiddenBranchGraph );
      else
        hiddenBranchGraph.detach();
    }
    
    isLive = live;
    
    for(int i=0; i<liveListeners.size(); i++)
      ((SceneGraphLiveListener)liveListeners.get(i)).sceneGraphLive( live );
    
  }
  
  public boolean getLive() {
    return isLive;
  }
  
  public BranchGroup[] getBranchGraphs() {
    return branchGraphArray;
  }

  public void addLiveListener( SceneGraphLiveListener listener ) {
    liveListeners.add(listener);
  }
  
  public void addGraphChangeListener( SceneGraphChangeListener listener ) {
    changeListeners.add( listener );
  }
  
  public void removeLiveListener( SceneGraphLiveListener listener ) {
    if (!liveListeners.remove(listener))
      throw new RuntimeException("Removing unknown listener");
  }
  
  public void removeGraphChangeListener( SceneGraphChangeListener listener ) {
    if (!changeListeners.remove( listener ))
      throw new RuntimeException("Removing unknown listener");
  }
  
  public void notifyGraphChanged( BranchGroup graph, javax.media.j3d.Node node ) {
    if (node instanceof javax.media.j3d.Group ||
        node instanceof javax.media.j3d.Behavior ||
        node instanceof javax.media.j3d.Link ) {
      for(int i=0; i<changeListeners.size(); i++) {
        ((SceneGraphChangeListener)changeListeners.get(i)).graphChanged( graph, node );
      }
    } else 
      throw new RuntimeException( "Node MUST be either Group, Behavior or Link");
  }
  
  private void updateBranchGraphArray() {
    branchGraphs.trimToSize();
    if (branchGraphs.size() != branchGraphArray.length)
      branchGraphArray = new BranchGroup[ branchGraphs.size() ];
    branchGraphArray = (BranchGroup[])branchGraphs.toArray( branchGraphArray );
  }
 
}

