/*
 *  @(#)GeometryInfoGenerator.java 1.3 00/12/19 15:49:23
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
 
package com.sun.j3d.demos.utils.geometry;

import javax.media.j3d.GeometryArray;
import javax.media.j3d.GeometryStripArray;
import javax.media.j3d.TriangleFanArray;
import javax.media.j3d.TriangleStripArray;
import javax.media.j3d.TriangleArray;
import javax.media.j3d.QuadArray;
import javax.media.j3d.IndexedGeometryArray;
import javax.media.j3d.IndexedGeometryStripArray;
import javax.media.j3d.IndexedQuadArray;
import javax.media.j3d.IndexedTriangleArray;
import javax.media.j3d.IndexedTriangleFanArray;
import javax.media.j3d.IndexedTriangleStripArray;

import com.sun.j3d.utils.geometry.GeometryInfo;

/** 
 * Populate a GeometryInfo object from the Geometry provided
 *
 * @author Paul Byrne
 * @version	1.3, 12/19/00
 */
public class GeometryInfoGenerator extends Object {
  
  public static void create( GeometryInfo geomInfo, GeometryArray geomArray ) {
    if (geomArray instanceof GeometryStripArray) 
      create( geomInfo, (GeometryStripArray)geomArray);
    else if (geomArray instanceof TriangleArray) {
      geomInfo.reset( GeometryInfo.TRIANGLE_ARRAY );
      processGeometryArray( geomInfo, geomArray );
    } else if (geomArray instanceof QuadArray ) {
      geomInfo.reset( GeometryInfo.QUAD_ARRAY );
      processGeometryArray( geomInfo, geomArray );
    } else if (geomArray instanceof IndexedGeometryArray)
      create( geomInfo, (IndexedGeometryArray)geomArray );
    else
      throw new RuntimeException("Unsupported Geometry type "+
                                        geomArray.getClass().getName() );
        
  }
  
  private static void create( GeometryInfo geomInfo, GeometryStripArray geomArray ) {
    if (geomArray instanceof TriangleFanArray) {
      geomInfo.reset( GeometryInfo.TRIANGLE_FAN_ARRAY );
    } else if (geomArray instanceof TriangleStripArray) {
      geomInfo.reset( GeometryInfo.TRIANGLE_STRIP_ARRAY );
    } else
      throw new RuntimeException("Unsupported Geometry type "+
                                        geomArray.getClass().getName() );
    
    processGeometryArray( geomInfo, geomArray ); 
    processStripArray( geomInfo, geomArray );
  }
  
  private static void create( GeometryInfo geomInfo, IndexedGeometryArray geomArray ) {
    if (geomArray instanceof IndexedQuadArray) {
      geomInfo.reset( GeometryInfo.QUAD_ARRAY );
    } else if (geomArray instanceof IndexedTriangleArray) {
      geomInfo.reset( GeometryInfo.TRIANGLE_ARRAY );
    } else if (geomArray instanceof IndexedTriangleFanArray) {
      geomInfo.reset( GeometryInfo.TRIANGLE_FAN_ARRAY );
      processIndexStripArray( geomInfo, (IndexedGeometryStripArray)geomArray );
    } else if (geomArray instanceof IndexedTriangleStripArray) {
      geomInfo.reset( GeometryInfo.TRIANGLE_STRIP_ARRAY );
      processIndexStripArray( geomInfo, (IndexedGeometryStripArray)geomArray );
    }
    
    processGeometryArray( geomInfo, geomArray );
    processIndexedArray( geomInfo, geomArray );
  }
  
  private static void processGeometryArray( GeometryInfo geomInfo, GeometryArray geomArray ) {
    int vertexFormat = geomArray.getVertexFormat();
    int vertexCount = geomArray.getVertexCount();
    
    double[] coords = new double[ vertexCount*3 ];
    geomArray.getCoordinates( 0, coords );
    geomInfo.setCoordinates( coords );
    
    if ((vertexFormat & GeometryArray.NORMALS)==GeometryArray.NORMALS) {
      float[] normals = new float[ vertexCount*3 ];
      geomArray.getNormals( 0, normals );
      geomInfo.setNormals( normals );
    }
    
    if ((vertexFormat & GeometryArray.COLOR_4)==GeometryArray.COLOR_4) {
      float[] colors = new float[ vertexCount*4 ];
      geomArray.getColors( 0, colors );
      geomInfo.setColors4( colors );
    } else if ((vertexFormat & GeometryArray.COLOR_3)!=0) {
      float[] colors = new float[ vertexCount*3 ];
      geomArray.getColors( 0, colors );
      geomInfo.setColors3( colors );
    }
    
    
    if ((vertexFormat & GeometryArray.TEXTURE_COORDINATE_3)==GeometryArray.TEXTURE_COORDINATE_3) {
      float[] tex = new float[ vertexCount * 3 ];
      geomArray.getTextureCoordinates( 0, tex );
      geomInfo.setTextureCoordinates3( tex );
    } else if ((vertexFormat & GeometryArray.TEXTURE_COORDINATE_2)!=0) {
      float[] tex = new float[ vertexCount * 2 ];
      geomArray.getTextureCoordinates( 0, tex );
      geomInfo.setTextureCoordinates2( tex );
    }
    

  }
  
  private static void processIndexedArray( GeometryInfo geomInfo, IndexedGeometryArray geomArray ) {
    int indexCount = geomArray.getIndexCount();
    int vertexFormat = geomArray.getVertexFormat();
    
    int[] coordI = new int[ indexCount ];
    
    geomArray.getCoordinateIndices( 0, coordI );
    geomInfo.setCoordinateIndices( coordI );
    
    if ((vertexFormat & GeometryArray.NORMALS)!=0) {
      int[] normalI = new int[indexCount];
      geomArray.getNormalIndices( 0, normalI );
      geomInfo.setNormalIndices( normalI );
    }
    
    if ((vertexFormat & GeometryArray.COLOR_3)!=0 ||
        (vertexFormat & GeometryArray.COLOR_4)!=0 ) {
      int[] colorI = new int[ indexCount ];
      geomArray.getColorIndices( 0, colorI );
      geomInfo.setColorIndices( colorI );
    }
    
    if ((vertexFormat & GeometryArray.TEXTURE_COORDINATE_2)!=0 ||
        (vertexFormat & GeometryArray.TEXTURE_COORDINATE_3)!=0) {
      int[] texI = new int[indexCount];
      geomArray.getTextureCoordinateIndices( 0, texI );
      geomInfo.setTextureCoordinateIndices( texI );
    }
    
  
  }
  
  private static void processStripArray( GeometryInfo geomInfo, 
                                  GeometryStripArray geomArray ) {
    int[] strips = new int[ geomArray.getNumStrips() ];
    geomArray.getStripVertexCounts( strips );
    
    geomInfo.setStripCounts( strips );
  }
  
  private static void processIndexStripArray( GeometryInfo geomInfo, 
                                  IndexedGeometryStripArray geomArray ) {
    int[] strips = new int[ geomArray.getNumStrips() ];
    geomArray.getStripIndexCounts( strips );
    
    geomInfo.setStripCounts( strips );
  }
}

