/*
 *  @(#)StatsDialog.java 1.13 00/12/20 14:24:22
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */package com.sun.j3d.demos.utils.gui;

import com.sun.j3d.demos.utils.scenegraph.traverser.TreeScan;
import com.sun.j3d.demos.utils.scenegraph.traverser.GeometrySizeProcessor;

import com.sun.j3d.demos.utils.behaviors.PerfBehaviorListener;
import com.sun.j3d.demos.utils.behaviors.PerfBehavior;


/**
 *
 * @author Paul Byrne
 * @version	1.13, 12/20/00
 */
public class StatsDialog extends javax.swing.JDialog implements PerfBehaviorListener {

    private long triangleCount = 0;
    private javax.media.j3d.BranchGroup toolBG; 
    private PerfBehavior perfBehavior=null;
    
    /** Creates new StatsDialog and add the performance measuring behavior
     * as a child of toolBG
     */
    public StatsDialog(java.awt.Frame parent, javax.media.j3d.BranchGroup toolBG ) {
        super (parent, false);
        this.toolBG = toolBG;
        initComponents ();
        pack ();
    }
    
    public void show() {
        super.show();
        if (perfBehavior!=null) {
            fpsTF.setText( "Calibrating" );
            //trianglespsTF.setText( "Calibrating" );
	    perfBehavior.calibrate();
	}
    }
    
    /**
     * Enable the Frame-per-second gathering of the Stats
     */
    public void setEnable( boolean enable ) {
        if (perfBehavior!=null) {
            perfBehavior.setEnable( enable );
        }
    }
    
    /**
     * Extract Geometry statistics from all the branch groups
     * If any of the graphs are compiled then Not Available (N/A) will
     * appear in the dialog.
     */
    public void calcGeometryStats( javax.media.j3d.BranchGroup[] branchGroups ) {
        boolean compiled = false;
        for(int i=0; i<branchGroups.length; i++)
            compiled |= branchGroups[i].isCompiled();
        
        if (!compiled) {
            GeometrySizeProcessor processor = new GeometrySizeProcessor();
            TreeScan scan = new TreeScan();
            Class geometryClass = javax.media.j3d.Shape3D.class;

            for(int i=0; i<branchGroups.length; i++) {
                scan.findNode( branchGroups[i], geometryClass, processor, false, true );
            }
            
            triangleCountTF.setText( Long.toString(processor.getTriangleCount() ));
            quadCountTF.setText( Long.toString(processor.getQuadCount() ));
            lineCountTF.setText( Long.toString(processor.getLineCount() ));
            pointCountTF.setText( Long.toString(processor.getPointCount() ));

            vertexInStripsTF.setText( Float.toString(processor.getStripPercentage() ));
            stripLengthTF.setText( Float.toString(processor.getAverageStripLength()) );

            triangleCount = processor.getTriangleCount();
            triangleCount += processor.getQuadCount()*2;
            
            shape3DcountTF.setText( Integer.toString(processor.getShape3DCount() ));
            appearanceCountTF.setText( Integer.toString( processor.getAppearanceCount() ));
        } else {
            String naStr = new String("N/A");
            triangleCountTF.setText( naStr );
            quadCountTF.setText( naStr );
            lineCountTF.setText( naStr );
            pointCountTF.setText( naStr );
            vertexInStripsTF.setText( naStr );
            stripLengthTF.setText( naStr );
            shape3DcountTF.setText( naStr );           
            appearanceCountTF.setText( naStr );
        }
        if (perfBehavior==null) {
            perfBehavior = new PerfBehavior();
            perfBehavior.addPerfBehaviorListener( this );
            perfBehavior.setEnable( false );
            toolBG.addChild( perfBehavior );
        } else {
            perfBehavior.calibrate();
        }
    }    
    
    /**
      * Extract the geometry statistics from the graph 
     *
     * If bg is null 0 is displayed in the stats and the 
      */
    public void calcGeometryStats( javax.media.j3d.BranchGroup bg ) {
        
        if (bg != null && !bg.isCompiled()) {
            GeometrySizeProcessor processor = new GeometrySizeProcessor();
            TreeScan scan = new TreeScan();
            Class geometryClass = javax.media.j3d.Shape3D.class;


            scan.findNode( bg, geometryClass, processor, true, true );
            triangleCountTF.setText( Long.toString(processor.getTriangleCount() ));
            quadCountTF.setText( Long.toString(processor.getQuadCount() ));
            lineCountTF.setText( Long.toString(processor.getLineCount() ));
            pointCountTF.setText( Long.toString(processor.getPointCount() ));

            vertexInStripsTF.setText( Float.toString(processor.getStripPercentage() ));
            stripLengthTF.setText( Float.toString(processor.getAverageStripLength()) );

            triangleCount = processor.getTriangleCount();
            triangleCount += processor.getQuadCount()*2;
            
            shape3DcountTF.setText( Integer.toString(processor.getShape3DCount() ));
            appearanceCountTF.setText( Integer.toString( processor.getAppearanceCount() ));
        } else {
            String naStr;
            if (bg==null)
                naStr = new String("0");
            else
                naStr = new String("N/A");
            triangleCountTF.setText( naStr );
            quadCountTF.setText( naStr );
            lineCountTF.setText( naStr );
            pointCountTF.setText( naStr );
            vertexInStripsTF.setText( naStr );
            stripLengthTF.setText( naStr );
            shape3DcountTF.setText( naStr );           
            appearanceCountTF.setText( naStr );
        }
        
        if (perfBehavior==null) {
            perfBehavior = new PerfBehavior();
            perfBehavior.addPerfBehaviorListener( this );
            toolBG.addChild( perfBehavior );
        } else
            if (bg==null) {
                perfBehavior.setEnable( false );
            } else {
                perfBehavior.calibrate();
                perfBehavior.setEnable( false );
            }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        buttonP = new javax.swing.JPanel();
        closeB = new javax.swing.JButton();
        statsP = new javax.swing.JPanel();
        geometryStatsP = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        triangleCountTF = new javax.swing.JTextField();
        quadCountTF = new javax.swing.JTextField();
        lineCountTF = new javax.swing.JTextField();
        pointCountTF = new javax.swing.JTextField();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        vertexInStripsTF = new javax.swing.JTextField();
        stripLengthTF = new javax.swing.JTextField();
        jLabel7 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        performanceP = new javax.swing.JPanel();
        jLabel9 = new javax.swing.JLabel();
        fpsTF = new javax.swing.JTextField();
        scenegraphStatsP = new javax.swing.JPanel();
        jLabel11 = new javax.swing.JLabel();
        shape3DcountTF = new javax.swing.JTextField();
        jLabel12 = new javax.swing.JLabel();
        appearanceCountTF = new javax.swing.JTextField();
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        }
        );
        
        
        closeB.setText("Close");
          closeB.addActionListener(new java.awt.event.ActionListener() {
              public void actionPerformed(java.awt.event.ActionEvent evt) {
                  closeBActionPerformed(evt);
              }
          }
          );
          buttonP.add(closeB);
          
          
        getContentPane().add(buttonP, java.awt.BorderLayout.SOUTH);
        
        
        statsP.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        geometryStatsP.setLayout(new java.awt.GridBagLayout());
          java.awt.GridBagConstraints gridBagConstraints2;
          geometryStatsP.setBorder(new javax.swing.border.TitledBorder(
          new javax.swing.border.EtchedBorder(), "Geometry Statistics"));
          
          jLabel1.setText("Triangle Count :");
            gridBagConstraints2 = new java.awt.GridBagConstraints();
            gridBagConstraints2.gridx = 0;
            gridBagConstraints2.gridy = 1;
            gridBagConstraints2.insets = new java.awt.Insets(0, 0, 0, 4);
            gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
            geometryStatsP.add(jLabel1, gridBagConstraints2);
            
            
          jLabel2.setText("Quad Count :");
            gridBagConstraints2 = new java.awt.GridBagConstraints();
            gridBagConstraints2.gridx = 0;
            gridBagConstraints2.gridy = 2;
            gridBagConstraints2.insets = new java.awt.Insets(0, 0, 0, 4);
            gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
            geometryStatsP.add(jLabel2, gridBagConstraints2);
            
            
          jLabel3.setText("Line Count :");
            gridBagConstraints2 = new java.awt.GridBagConstraints();
            gridBagConstraints2.gridx = 0;
            gridBagConstraints2.gridy = 3;
            gridBagConstraints2.insets = new java.awt.Insets(0, 0, 0, 4);
            gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
            geometryStatsP.add(jLabel3, gridBagConstraints2);
            
            
          jLabel4.setText("Point Count :");
            gridBagConstraints2 = new java.awt.GridBagConstraints();
            gridBagConstraints2.gridx = 0;
            gridBagConstraints2.gridy = 4;
            gridBagConstraints2.insets = new java.awt.Insets(0, 0, 0, 4);
            gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
            geometryStatsP.add(jLabel4, gridBagConstraints2);
            
            
          triangleCountTF.setEditable(false);
            triangleCountTF.setColumns(7);
            triangleCountTF.setText("jTextField1");
            triangleCountTF.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
            triangleCountTF.setMinimumSize(new java.awt.Dimension(63, 21));
            gridBagConstraints2 = new java.awt.GridBagConstraints();
            gridBagConstraints2.gridx = 1;
            gridBagConstraints2.gridy = 1;
            geometryStatsP.add(triangleCountTF, gridBagConstraints2);
            
            
          quadCountTF.setEditable(false);
            quadCountTF.setColumns(7);
            quadCountTF.setText("jTextField1");
            quadCountTF.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
            quadCountTF.setMinimumSize(new java.awt.Dimension(63, 21));
            gridBagConstraints2 = new java.awt.GridBagConstraints();
            gridBagConstraints2.gridx = 1;
            gridBagConstraints2.gridy = 2;
            geometryStatsP.add(quadCountTF, gridBagConstraints2);
            
            
          lineCountTF.setEditable(false);
            lineCountTF.setColumns(7);
            lineCountTF.setText("jTextField1");
            lineCountTF.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
            lineCountTF.setMinimumSize(new java.awt.Dimension(63, 21));
            gridBagConstraints2 = new java.awt.GridBagConstraints();
            gridBagConstraints2.gridx = 1;
            gridBagConstraints2.gridy = 3;
            geometryStatsP.add(lineCountTF, gridBagConstraints2);
            
            
          pointCountTF.setEditable(false);
            pointCountTF.setColumns(7);
            pointCountTF.setText("jTextField1");
            pointCountTF.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
            pointCountTF.setMinimumSize(new java.awt.Dimension(63, 21));
            pointCountTF.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    jTextField4ActionPerformed(evt);
                }
            }
            );
            gridBagConstraints2 = new java.awt.GridBagConstraints();
            gridBagConstraints2.gridx = 1;
            gridBagConstraints2.gridy = 4;
            geometryStatsP.add(pointCountTF, gridBagConstraints2);
            
            
          jLabel5.setText("Vertex In Strips :");
            gridBagConstraints2 = new java.awt.GridBagConstraints();
            gridBagConstraints2.gridx = 0;
            gridBagConstraints2.gridy = 5;
            gridBagConstraints2.insets = new java.awt.Insets(0, 0, 0, 4);
            gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
            geometryStatsP.add(jLabel5, gridBagConstraints2);
            
            
          jLabel6.setText("Average Strip Length :");
            gridBagConstraints2 = new java.awt.GridBagConstraints();
            gridBagConstraints2.gridx = 0;
            gridBagConstraints2.gridy = 6;
            gridBagConstraints2.insets = new java.awt.Insets(0, 0, 0, 4);
            gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
            geometryStatsP.add(jLabel6, gridBagConstraints2);
            
            
          vertexInStripsTF.setEditable(false);
            vertexInStripsTF.setColumns(4);
            vertexInStripsTF.setText("jTextField6");
            vertexInStripsTF.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
            vertexInStripsTF.setMinimumSize(new java.awt.Dimension(36, 21));
            gridBagConstraints2 = new java.awt.GridBagConstraints();
            gridBagConstraints2.gridx = 1;
            gridBagConstraints2.gridy = 5;
            gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
            geometryStatsP.add(vertexInStripsTF, gridBagConstraints2);
            
            
          stripLengthTF.setEditable(false);
            stripLengthTF.setColumns(7);
            stripLengthTF.setText("jTextField7");
            stripLengthTF.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
            stripLengthTF.setMinimumSize(new java.awt.Dimension(63, 21));
            gridBagConstraints2 = new java.awt.GridBagConstraints();
            gridBagConstraints2.gridx = 1;
            gridBagConstraints2.gridy = 6;
            geometryStatsP.add(stripLengthTF, gridBagConstraints2);
            
            
          jLabel7.setText("%");
            gridBagConstraints2 = new java.awt.GridBagConstraints();
            gridBagConstraints2.gridx = 2;
            gridBagConstraints2.gridy = 5;
            gridBagConstraints2.insets = new java.awt.Insets(0, 4, 0, 0);
            gridBagConstraints2.anchor = java.awt.GridBagConstraints.WEST;
            geometryStatsP.add(jLabel7, gridBagConstraints2);
            
            
          jLabel8.setText("Stats for all Geometry in Scene Graph");
            gridBagConstraints2 = new java.awt.GridBagConstraints();
            gridBagConstraints2.gridwidth = 3;
            gridBagConstraints2.insets = new java.awt.Insets(0, 0, 4, 0);
            geometryStatsP.add(jLabel8, gridBagConstraints2);
            
            gridBagConstraints1 = new java.awt.GridBagConstraints();
          gridBagConstraints1.gridheight = 2;
          gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
          statsP.add(geometryStatsP, gridBagConstraints1);
          
          
        performanceP.setLayout(new java.awt.GridBagLayout());
          java.awt.GridBagConstraints gridBagConstraints3;
          performanceP.setBorder(new javax.swing.border.TitledBorder(
          new javax.swing.border.EtchedBorder(), "Performance"));
          
          jLabel9.setText("Frames per second :");
            gridBagConstraints3 = new java.awt.GridBagConstraints();
            gridBagConstraints3.insets = new java.awt.Insets(0, 0, 0, 3);
            gridBagConstraints3.anchor = java.awt.GridBagConstraints.EAST;
            performanceP.add(jLabel9, gridBagConstraints3);
            
            
          fpsTF.setPreferredSize(new java.awt.Dimension(63, 21));
            fpsTF.setEditable(false);
            fpsTF.setColumns(7);
            fpsTF.setText("Calibrating");
            fpsTF.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
            fpsTF.setMinimumSize(new java.awt.Dimension(60, 21));
            gridBagConstraints3 = new java.awt.GridBagConstraints();
            gridBagConstraints3.anchor = java.awt.GridBagConstraints.WEST;
            performanceP.add(fpsTF, gridBagConstraints3);
            
            gridBagConstraints1 = new java.awt.GridBagConstraints();
          gridBagConstraints1.gridx = 1;
          gridBagConstraints1.gridy = 0;
          gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
          gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTH;
          statsP.add(performanceP, gridBagConstraints1);
          
          
        scenegraphStatsP.setLayout(new java.awt.GridBagLayout());
          java.awt.GridBagConstraints gridBagConstraints4;
          scenegraphStatsP.setBorder(new javax.swing.border.TitledBorder("SceneGraph Stats"));
          
          jLabel11.setText("Shape3D Count :");
            gridBagConstraints4 = new java.awt.GridBagConstraints();
            gridBagConstraints4.insets = new java.awt.Insets(0, 0, 0, 4);
            gridBagConstraints4.anchor = java.awt.GridBagConstraints.EAST;
            scenegraphStatsP.add(jLabel11, gridBagConstraints4);
            
            
          shape3DcountTF.setEditable(false);
            shape3DcountTF.setColumns(5);
            shape3DcountTF.setText("jTextField1");
            shape3DcountTF.setMinimumSize(new java.awt.Dimension(45, 21));
            gridBagConstraints4 = new java.awt.GridBagConstraints();
            gridBagConstraints4.anchor = java.awt.GridBagConstraints.WEST;
            scenegraphStatsP.add(shape3DcountTF, gridBagConstraints4);
            
            
          jLabel12.setText("Appearance Count :");
            gridBagConstraints4 = new java.awt.GridBagConstraints();
            gridBagConstraints4.gridx = 0;
            gridBagConstraints4.gridy = 1;
            gridBagConstraints4.insets = new java.awt.Insets(0, 0, 0, 4);
            gridBagConstraints4.anchor = java.awt.GridBagConstraints.EAST;
            scenegraphStatsP.add(jLabel12, gridBagConstraints4);
            
            
          appearanceCountTF.setEditable(false);
            appearanceCountTF.setColumns(5);
            appearanceCountTF.setText("jTextField2");
            appearanceCountTF.setMinimumSize(new java.awt.Dimension(45, 21));
            gridBagConstraints4 = new java.awt.GridBagConstraints();
            gridBagConstraints4.gridx = 1;
            gridBagConstraints4.gridy = 1;
            gridBagConstraints4.anchor = java.awt.GridBagConstraints.WEST;
            scenegraphStatsP.add(appearanceCountTF, gridBagConstraints4);
            
            gridBagConstraints1 = new java.awt.GridBagConstraints();
          gridBagConstraints1.gridx = 1;
          gridBagConstraints1.gridy = 1;
          gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
          gridBagConstraints1.anchor = java.awt.GridBagConstraints.NORTHWEST;
          statsP.add(scenegraphStatsP, gridBagConstraints1);
          
          
        getContentPane().add(statsP, java.awt.BorderLayout.CENTER);
        
    }//GEN-END:initComponents

  private void jTextField4ActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextField4ActionPerformed
// Add your handling code here:
  }//GEN-LAST:event_jTextField4ActionPerformed

  private void closeBActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_closeBActionPerformed
// Add your handling code here:
    closeDialog();
  }//GEN-LAST:event_closeBActionPerformed

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        closeDialog();
    }//GEN-LAST:event_closeDialog

    private void closeDialog() {
        setVisible (false);
        perfBehavior.setEnable( false );
        dispose ();
    }


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel buttonP;
    private javax.swing.JButton closeB;
    private javax.swing.JPanel statsP;
    private javax.swing.JPanel geometryStatsP;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JTextField triangleCountTF;
    private javax.swing.JTextField quadCountTF;
    private javax.swing.JTextField lineCountTF;
    private javax.swing.JTextField pointCountTF;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JTextField vertexInStripsTF;
    private javax.swing.JTextField stripLengthTF;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JPanel performanceP;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JTextField fpsTF;
    private javax.swing.JPanel scenegraphStatsP;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JTextField shape3DcountTF;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JTextField appearanceCountTF;
    // End of variables declaration//GEN-END:variables

    /** Called when new performance figures are available
     */
    public void updatePerformanceFigures(float framesPerSecond) {
        fpsTF.setText( Float.toString( framesPerSecond ) );
        //trianglespsTF.setText( Double.toString( framesPerSecond * triangleCount ));
    }
}

