/*
 *  @(#)LoaderControl.java 1.20 01/01/03 09:22:03
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.loadercontrol;

import java.io.File;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.filechooser.FileFilter;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import com.sun.j3d.loaders.Scene;
import com.sun.j3d.loaders.Loader;

import com.sun.j3d.demos.utils.loadercontrol.UnsupportedFormatException;

/**
 * Provides a single interface for using all the Java3D loaders.
 *
 * @author Paul Byrne
 * @version	1.20, 01/03/01
 *
  */
public class LoaderControl extends Object {

  private ArrayList fileFilters = null;       
  private LoaderSelectionDialog selectionDialog = null;
 // private JFileChooser fileChooser = null;
  private LoaderControlListener menuLoaderListener = null;
                            
  // List of available Loaders
  // Name, file extension, loader class, loader flags, enabled (Y/N), loader interface (Reader, URL, Filename, Nonstandard)
  private String[][] loaders = new String[][] {
    {"J3D Fly", "j3f", "com.sun.j3d.demos.utils.loaderwrappers.J3fLoader", "LOAD_ALL", "Y", "Filename" },
    {"Alias Obj", "obj", "com.sun.j3d.loaders.objectfile.ObjectFile", "STRIPIFY TRIANGULATE", "Y", "Reader" },
    {"Alias Obj (Reversed Tris)", "obj", "com.sun.j3d.loaders.objectfile.ObjectFile", "REVERSE STRIPIFY TRIANGULATE", "N", "Reader" },
    {"Lightwave", "lws", "com.sun.j3d.loaders.lw3d.Lw3dLoader", "LOAD_ALL", "Y", "Filename" },
    {"Vrml97", "wrl", "com.sun.j3d.loaders.vrml97.VrmlLoader", "LOAD_ALL", "N", "URL" },
    {"Starfire Research 3DS", "3ds", "com.mnstarfire.loaders3d.Loader3DS", "", "Y", "Filename" },
    {"NCSA 3DS", "3ds", "ncsa.j3d.loaders.ModelLoader", "LOAD_ALL", "N", "Nonstandard" },
    {"NCSA COB", "cob", "ncsa.j3d.loaders.ModelLoader", "LOAD_ALL", "Y", "Nonstandard" },
    {"NCSA DEM", "dem", "ncsa.j3d.loaders.ModelLoader", "LOAD_ALL", "Y", "Nonstandard" },
    {"NCSA DXF", "dxf", "ncsa.j3d.loaders.ModelLoader", "LOAD_ALL", "Y", "Nonstandard"},
    {"NCSA IOB", "iob", "ncsa.j3d.loaders.ModelLoader", "LOAD_ALL", "Y", "Nonstandard"},
    {"NCSA LWS", "lws", "ncsa.j3d.loaders.ModelLoader", "LOAD_ALL", "N", "Nonstandard"},
    {"NCSA OBJ", "obj", "ncsa.j3d.loaders.ModelLoader", "LOAD_ALL", "N", "Nonstandard"},
    {"NCSA PDB", "pdb", "ncsa.j3d.loaders.ModelLoader", "LOAD_ALL", "Y", "Nonstandard"},
    {"NCSA PLAY", "play", "ncsa.j3d.loaders.ModelLoader", "LOAD_ALL", "Y", "Nonstandard"},
    {"NCSA VRML", "wrl", "ncsa.j3d.loaders.ModelLoader", "LOAD_ALL", "Y", "Nonstandard"},
    {"FullSail FLT", "flt", "com.fullsail.j3d.loaders.flt.FltLoader", "FORCE_LOAD USE_TEXTURES", "Y", "Filename" }
  };

  private boolean installed[] = new boolean[ loaders.length ];
  private boolean enabled[] = new boolean[ loaders.length ];
  private int loaderMethod[] = new int[ loaders.length ];

  /** Creates new LoaderControl */
  public LoaderControl() {
      checkInstalled();
  }

    /**
      * Set the fileChooser to use when loaders are selected from
      * the loader Menu
      *
      * The FileFilter will be overwritten with a filter listing all 
      * available geometry types whenever a menu item is selected
      *
      * If no fileChooser is set a new fileChooser will be created when the
      * first loaderMenu item is selected
      */
  /*
    public void setFileChooser( JFileChooser fileChooser ) {
        this.fileChooser = fileChooser;
    }
   **/

    /**  
      * Show the selection dialog which allows the user to choose
      * which loaders to use
      */

    public void showSelectionDialog( java.awt.Frame parent ) {
        if (selectionDialog==null) {
            selectionDialog = new LoaderSelectionDialog(parent, true);
            boolean enabled;
            for(int i=0; i<loaders.length; i++) {
                if (loaders[i][4].equals("Y")) {
                    enabled = true;
                } else {
                    enabled = false;
                }
                selectionDialog.addLoader( loaders[i][0], 
                                           loaders[i][1],
                                           loaders[i][3],
                                           enabled,
                                           installed[i] );
            }
        }
        
        if (selectionDialog.showDialog()) {
            selectionDialog.getEnabledLoaders( enabled );
       
            if (fileFilters!=null) {
                // Remove all the file filters fileFilter will be update when setFileFilters is called
                fileFilters.clear();
            }
                                        
        }
    }
  
        
    /**
      * Populate </code>menu</code> with MenuItems representing all the
      * available loaders
      * 
      * This class handles all ActionEvents on the menu, when an item is
      * loaded the loaderListener will be called
      */
    /*
  public void setupMenu( JMenu menu, LoaderControlListener loaderListener  ) {
    menuLoaderListener = loaderListener;
    for(int i=0; i<loaders.length; i++) {
      JMenuItem item = new JMenuItem( loaders[i][0] );
      item.addActionListener( this );
      item.setActionCommand( Integer.toString(i) );
      
      item.setEnabled( installed[i] );
      
      menu.add( item );
    }
  }
     **/
  
  /**
   *  Check which loaders are installed in the Classpath and
   * setup the enabledList
   */
  private void checkInstalled() {
    for(int i=0; i<loaders.length; i++) {      
      try {   // Disable menu item if class not available
        Class cl = Class.forName( loaders[i][2] );
        installed[i] = true;
      } catch( ClassNotFoundException e ) {
          //System.out.println( "Not Installed "+loaders[i][2] );
          installed[i] = false;
          loaders[i][4] = "N";
      } catch( NoClassDefFoundError e ) {
          //System.out.println( "Not Installed "+loaders[i][2] );
          installed[i] = false;
          loaders[i][4] = "N";
      }
      if (loaders[i][4].equals("Y")) {
          enabled[i] = true;
      } else {
          enabled[i] = false;
      }
      
      if (loaders[i][5].equalsIgnoreCase("Reader"))
          loaderMethod[i] = SceneLoader.USE_READER;
      else if (loaders[i][5].equalsIgnoreCase("Filename"))
          loaderMethod[i] = SceneLoader.USE_FILENAME;
      else if (loaders[i][5].equalsIgnoreCase("URL"))
          loaderMethod[i] = SceneLoader.USE_URL;
      else if (loaders[i][5].equalsIgnoreCase("Nonstandard"))
          loaderMethod[i] = SceneLoader.USE_NONSTANDARD;
      else {
          System.out.println("WARNING - LoaderControl has invalid LoaderMethod for "+loaders[i][2] );
          loaderMethod[i] = SceneLoader.USE_READER;
      }
          
    }
  }
 
  /**
    * Sets the file filters in the JFileChooser to the various geometry
    * file formats available in the classpath of the app.
    *
    * Previous file filters are cleared from the chooser.
    *
    * This method should be called whenever the fileChooser is 
    * displayed because the FileFilter may change depending on
    * user updates made in showSelectionDialog().
    */
  public void setFileFilter(javax.swing.JFileChooser fileChooser ) {
    if (fileFilters==null) {
        fileFilters = new ArrayList();
    }
    
    if (fileFilters.size()==0) {
        
        ExampleFileFilter allFilter = new ExampleFileFilter();
        allFilter.setDescription("All Geometry Files");
        allFilter.setExtensionListInDescription( false );
        for(int i=0; i<loaders.length; i++) {
            if (installed[i] & enabled[i]) {
                allFilter.addExtension( loaders[i][1] );
                fileFilters.add( new ExampleFileFilter( loaders[i][1], loaders[i][0] ));
            }
        }
        
        fileFilters.add( allFilter );
    }
    
    fileChooser.resetChoosableFileFilters();
    
    Iterator it = fileFilters.iterator();
    while( it.hasNext() )
        fileChooser.addChoosableFileFilter( (FileFilter)it.next() );
    
    // Set the current filter to the allFilter
    fileChooser.setFileFilter( (FileFilter)fileFilters.get( fileFilters.size()-1) );

}
  
    /** Listener for menu selections
      * This method should not be called from user code
      */
/*
  public void actionPerformed(final java.awt.event.ActionEvent evt) {
    Loader actualLoader = null;
    JMenuItem menuItem = (JMenuItem)evt.getSource();
    int loaderID = Integer.parseInt(menuItem.getActionCommand());
    File currentFile=null;

    actualLoader = getLoader( loaderID );
    
    if (fileChooser==null) {
        fileChooser = new JFileChooser();
    }
    
    setFileFilter( fileChooser );
        
    fileChooser.setDialogTitle( "Load 3D Content" );
    fileChooser.setFileSelectionMode( JFileChooser.FILES_AND_DIRECTORIES );
    int ret = fileChooser.showOpenDialog( null );
    if (ret==JFileChooser.APPROVE_OPTION) {
        currentFile = fileChooser.getSelectedFile();
    }
    
    if (currentFile==null) {
      return;
    }
    
    ///SceneLoader sceneLoader = new SceneLoader( actualLoader, menuLoaderListener, currentFile,
    //                                           loaderID );
    try {
        loadFileAndWait( currentFile, menuLoaderListener, null );
    } catch( UnsupportedFormatException e ) {
        System.out.println("Error loading file");
    }
  }
 **/
  
  /**
   * Return the Class Name for the specified loader
   */
  public String getLoaderClassName( int loaderID ) {
      return loaders[loaderID][2];
  }
  
  /**
    * Return the loader for the given loaderID
    */
  private Loader getLoader( int loaderID ) {
    Loader actualLoader=null;
    
    try {
      Class loaderClass = Class.forName( loaders[loaderID][2] );
      actualLoader = (Loader)loaderClass.newInstance();
      int loaderFlags = 0;
      if (loaders[loaderID][3]!=null) {
        String tokens = loaders[loaderID][3];
        int nextWord = tokens.indexOf( ' ', 0 );
        int startWord = 0;
        String str;
        if (tokens.length()!=0) {
            do {
              if (nextWord==-1) 
                nextWord = tokens.length();
              str = tokens.substring( startWord, nextWord );
              Field field = loaderClass.getField( str );
              loaderFlags |= field.getInt( actualLoader );
              startWord = nextWord+1;
              nextWord = tokens.indexOf( ' ', startWord );
            } while( startWord!=0 && startWord<tokens.length() );
        }
      }
      actualLoader.setFlags( 0 );
      actualLoader.setFlags( loaderFlags );
    } catch( ClassNotFoundException e ) {
      e.printStackTrace();
      System.out.println("Error Recovered");
    } catch( IllegalAccessException ex ) {
      ex.printStackTrace();
      System.out.println("Error Recovered");
    } catch( InstantiationException exc ) {
      exc.printStackTrace();
      System.out.println("Error Recovered");
   } catch( NoSuchFieldException exce ) {
      exce.printStackTrace();
      System.out.println("Error Recovered");
    }
    
    return actualLoader;
  }
     
  /**
    * Load the file and don't return until the load is complete
   * @param file The file to load
   * @param listener The listener to call when the load is complete
   * @param workMonitor A progress monitor for the load, can be null
    */
  public void loadFileAndWait( File file, LoaderControlListener listener, com.sun.j3d.demos.utils.gui.WorkMonitor workMonitor ) throws UnsupportedFormatException {
    actualLoadFile( file, listener, workMonitor ).waitForCompletion();
  }
  
  /**
    * Load the specified file, listener is notified when load is complete
    * Loading takes place in a seperate thread, this method will return
    * without waiting for the load to complete.
   *
   * @param file The file to load
   * @param listener The listener to call when the load is complete
   * @param workMonitor A progress monitor for the load, can be null
    *
    */
  public void loadFile( File file, LoaderControlListener listener, com.sun.j3d.demos.utils.gui.WorkMonitor workMonitor ) throws UnsupportedFormatException {
    actualLoadFile( file, listener, workMonitor );
  }
    
  private SceneLoader actualLoadFile( File file, LoaderControlListener listener, com.sun.j3d.demos.utils.gui.WorkMonitor workMonitor ) throws UnsupportedFormatException {
    String filename = file.getName();
    String extension;
    int loaderID = -1;
    try {
      extension = filename.substring( filename.lastIndexOf('.')+1, filename.length() );
    } catch( Exception e ) {
      throw new UnsupportedFormatException( "Unknown file extension "+filename);
    }
    
    for(int i=0; i<loaders.length && loaderID==-1; i++) {
      if ( loaders[i][1].compareToIgnoreCase(extension)==0 && enabled[i] )
        loaderID = i;
    }
    
    if (loaderID==-1)
      throw new UnsupportedFormatException("Unsupported file extension "+extension);
    
    Loader actualLoader = getLoader( loaderID );
    SceneLoader sceneLoader = new SceneLoader( actualLoader, listener, file,
                                               loaderID, loaderMethod[loaderID],
                                               workMonitor );
    return sceneLoader;
  }
  
    /**
      * Returns the class name for the loader with specified ID
      */
    public String getLoaderClass( int loaderID ) {
        return loaders[loaderID][2];
    }
    
    public String getLoaderFileExtension( int loaderID ) {
        return loaders[loaderID][1];
    }
  
}

