/*
 *  @(#)LoaderSelectionDialog.java 1.4 00/12/19 16:52:37
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.loadercontrol;

import java.util.HashMap;
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.event.TableModelEvent;

/**
 * A Dialog which allows users to choose which loaders are used for
 * a particular file format.
 *
 * @author Paul Byrne
 * @version	1.4, 12/19/00
 */
public class LoaderSelectionDialog extends javax.swing.JDialog implements javax.swing.event.TableModelListener {

    private DefaultTableModel tableModel;
    private LoaderTableCellRenderer cellRenderer = new LoaderTableCellRenderer();
   
    private HashMap loaderTypes;
    private int currentRow;
    
    boolean okQuit = false;
    
    /** Creates new form LoaderSelectionDialog */
    public LoaderSelectionDialog(java.awt.Frame parent, boolean modal) {
        super (parent, modal);
        initComponents ();
        
        if (com.sun.j3d.demos.utils.help.Help.isEnabled()) {
            com.sun.j3d.demos.utils.help.Help.addHelpButton( helpB, "loader.selection.dialog" );
        } else {
            buttonPanel.remove( helpB );
        }
        
        pack ();
            
        tableModel = (DefaultTableModel)loaderTable.getModel();
        currentRow = 0;
        tableModel.addTableModelListener( this );
        
        loaderTypes = new HashMap();
        
        TableColumn nameColumn = loaderTable.getColumn("Loader Name");
        nameColumn.setCellRenderer( cellRenderer );
        
        TableColumn extColumn = loaderTable.getColumn("File Extension");
        extColumn.setCellRenderer( cellRenderer );
        
        TableColumn paramColumn = loaderTable.getColumn("Parameters");
        paramColumn.setCellRenderer( cellRenderer );
    }
        
    /**
     * Show the dialog,
     * return true if Apply was pressed
     */
    public boolean showDialog() {
        okQuit = false;
        super.show();
        
        return okQuit;
    }
    
    /**
      * Add a loader to the list of available loaders
      */
    public void addLoader( String name, String fileExtension, String parameters, boolean enabled, boolean available ) {

        if (!available) {
            name = name.concat(".");
            fileExtension = name.concat(".");
            parameters = name.concat(".");
            enabled = false;
        }
        
        Boolean en = new Boolean(enabled);
        Boolean avail = new Boolean(available);
        tableModel.insertRow( currentRow, new Object[] {name, fileExtension, parameters, en, avail} );
        if (available) {
            recordType( fileExtension, currentRow );
        }
        currentRow++;
     }
     
     /**
      * Record which types are in which table row
      */
     private void recordType( String fileExtension, int row ) {
         ArrayList rows = (ArrayList)loaderTypes.get( fileExtension );
         if (rows==null) {
            rows = new ArrayList();
            rows.add( new Integer(row) );
            loaderTypes.put( fileExtension, rows );
         } else {
             rows.add( new Integer(row));
         }         
     }
     
     /**
      * Enable the loader in the specified row
      * Disable all other loaders that take the same file extension
      */
     private void enableLoader( int row ) {
         String fileExtension = (String)tableModel.getValueAt( row, 1 );
         ArrayList rowList = (ArrayList)loaderTypes.get( fileExtension );
         if (rowList!=null) {
             Iterator it = rowList.iterator();
             while( it.hasNext() ) {
                 Integer i = (Integer)it.next();
                 if (i.intValue()!=row) {
                     tableModel.setValueAt( new Boolean(false), i.intValue(), 3 );
                 }
             }
         }
     }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
     private void initComponents() {//GEN-BEGIN:initComponents
         jPanel1 = new javax.swing.JPanel();
         jScrollPane1 = new javax.swing.JScrollPane();
         loaderTable = new javax.swing.JTable();
         buttonPanel = new javax.swing.JPanel();
         applyB = new javax.swing.JButton();
         cancelB = new javax.swing.JButton();
         helpB = new javax.swing.JButton();
         addWindowListener(new java.awt.event.WindowAdapter() {
             public void windowClosing(java.awt.event.WindowEvent evt) {
                 closeDialog(evt);
             }
         }
         );
         
         
         
           loaderTable.setModel(new javax.swing.table.DefaultTableModel (
             new Object [][] {
                 
             },
             new String [] {
                 "Loader Name", "File Extension", "Parameters", "Enabled", "Available"
             }
             ) {
                 Class[] types = new Class [] {
                     java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.Boolean.class, java.lang.Boolean.class
                 };
                 boolean[] canEdit = new boolean [] {
                     false, false, false, true, false
                 };
                 
                 public Class getColumnClass (int columnIndex) {
                     return types [columnIndex];
                 }
                 
                 public boolean isCellEditable (int rowIndex, int columnIndex) {
                     return canEdit [columnIndex];
                 }
             });
             jScrollPane1.setViewportView(loaderTable);
             
             jPanel1.add(jScrollPane1);
           
           
         getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);
         
         
         
         applyB.setText("Apply");
           applyB.addActionListener(new java.awt.event.ActionListener() {
               public void actionPerformed(java.awt.event.ActionEvent evt) {
                   applyBActionPerformed(evt);
               }
           }
           );
           buttonPanel.add(applyB);
           
           
         cancelB.setText("Cancel");
           cancelB.addActionListener(new java.awt.event.ActionListener() {
               public void actionPerformed(java.awt.event.ActionEvent evt) {
                   cancelBActionPerformed(evt);
               }
           }
           );
           buttonPanel.add(cancelB);
           
           
         helpB.setText("Help...");
           buttonPanel.add(helpB);
           
           
         getContentPane().add(buttonPanel, java.awt.BorderLayout.SOUTH);
         
     }//GEN-END:initComponents

  private void cancelBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelBActionPerformed
// Add your handling code here:
      setVisible( false );
      dispose();
      okQuit = false;
  }//GEN-LAST:event_cancelBActionPerformed

  private void applyBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_applyBActionPerformed
// Add your handling code here:
      setVisible( false );
      dispose();
      okQuit = true;
  }//GEN-LAST:event_applyBActionPerformed

    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible (false);
        dispose ();
    }//GEN-LAST:event_closeDialog

    /**
    * @param args the command line arguments
    */
    public static void main (String args[]) {
        new LoaderSelectionDialog (new javax.swing.JFrame (), true).show ();
    }
    
    /**
     * Return true if user applied changes, false otherwise
     */
    public boolean changesApplied() {
        return okQuit;
    }

    public void tableChanged(final javax.swing.event.TableModelEvent evt) {
        if (evt.getType()==TableModelEvent.UPDATE) {
            //System.out.println( evt.getColumn()+"  "+evt.getFirstRow() );
            if ( ((Boolean)tableModel.getValueAt( evt.getFirstRow(), 3 )).booleanValue())
                enableLoader( evt.getFirstRow() );
        }
    }  
    
    /**
     * Set enabledList to represent the enable state of each loader
     */
    public void getEnabledLoaders( boolean[] enabledList ) {
        for(int i=0; i<enabledList.length; i++) {
            enabledList[i] = ((Boolean)tableModel.getValueAt( i, 3)).booleanValue();
        }
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTable loaderTable;
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JButton applyB;
    private javax.swing.JButton cancelB;
    private javax.swing.JButton helpB;
    // End of variables declaration//GEN-END:variables

}

