/*
 *  @(#)SceneLoader.java 1.14 01/02/07 15:17:36
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.loadercontrol;

import com.sun.j3d.loaders.Loader;
import com.sun.j3d.loaders.Scene;
import java.io.*;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import java.awt.event.ActionEvent;
import java.lang.reflect.Method;

class SceneLoader extends Object {
    
    public static final int USE_READER = 0;
    public static final int USE_FILENAME = 1;
    public static final int USE_URL = 2;
    public static final int USE_NONSTANDARD = 3;
    
    private InputStream inputStream;
    private Thread worker;
    private Timer timer;
    private String filename;
    private int progress;
    private Runnable setProgress;
    
  /** Creates new SceneLoader */
    public SceneLoader( final Loader loader, final LoaderControlListener loaderControl,
    final File file, final int loaderID, final int useLoaderInterface,
    final com.sun.j3d.demos.utils.gui.WorkMonitor monitor ) {
        
        if (monitor!=null) {
            setProgress = new Runnable() {
                public void run() {
                    monitor.setProgress( progress );
                }
            };
        } else
            setProgress = null;
        
        worker = new Thread() {
            public void run() {
                long startTime = System.currentTimeMillis();
                progress = 0;
                if (setProgress!=null)
                    SwingUtilities.invokeLater( setProgress );
                try {
                    Scene scene=null;
                    
                    switch( useLoaderInterface ) {
                        case USE_READER :
                            try {
                                inputStream = new BufferedInputStream(
                                new FileInputStream( file ));
                            } catch(FileNotFoundException e ) {
                                loaderControl.sceneLoaded( LoaderControlListener.LOAD_FAILED, null, loaderID, file, e );
                                return;
                            }
                            loader.setBasePath( file.getAbsolutePath().substring(0, file.getAbsolutePath().lastIndexOf( java.io.File.separator )) );
                            scene = loader.load( new java.io.InputStreamReader(inputStream) );
                            break;
                        case USE_FILENAME :
                            scene = loader.load( file.getAbsolutePath() );
                            break;
                            
                        case USE_URL :
                            scene = loader.load( file.toURL() );
                            break;
                        case USE_NONSTANDARD :
                            if ( loader.getClass().getName().equals("ncsa.j3d.loaders.ModelLoader") ) {
                                String fileext = file.getName().substring( file.getName().lastIndexOf('.')+1 );
                                Method loadMethod = loader.getClass().getMethod( "load", new Class[] { java.io.Reader.class, String.class } );
                                try {
                                    inputStream = new BufferedInputStream(
                                    new FileInputStream( file ));
                                } catch(FileNotFoundException e ) {
                                    loaderControl.sceneLoaded( LoaderControlListener.LOAD_FAILED, null, loaderID, file, e );
                                    return;
                                }
                                scene = (com.sun.j3d.loaders.Scene)loadMethod.invoke( loader, new Object[] { new java.io.InputStreamReader(inputStream), fileext } );
                            } else {
                                RuntimeException e = new RuntimeException("No Nonstandard loader for "+loader.getClass().getName() );
                                loaderControl.sceneLoaded( LoaderControlListener.LOAD_FAILED, null, loaderID, file, e );
                                progress = -1;
                                timer.stop();
                                if (setProgress!=null)
                                    SwingUtilities.invokeLater( setProgress );
                                return;
                            }
                            break;
                    }
                    
                    //System.out.println("Load Time "+(System.currentTimeMillis()-startTime));
                    if (setProgress!=null) {
                        progress = -1;
                        timer.stop();
                        SwingUtilities.invokeLater( setProgress );
                    }
                    
                    // The NCSA Loader sometimes fails without throwing an exception
                    // and returns a null scene
                    if (scene!=null) {
                        loaderControl.sceneLoaded( LoaderControlListener.LOAD_COMPLETE, scene, loaderID, file, null );
                    } else {
                        loaderControl.sceneLoaded( LoaderControlListener.LOAD_FAILED, null, loaderID, file, null );
                    }
                } catch( java.io.FileNotFoundException e) {
                    if (setProgress!=null) {
                        progress = -1;
                        timer.stop();
                        SwingUtilities.invokeLater( setProgress );
                    }
                    loaderControl.sceneLoaded( LoaderControlListener.LOAD_FAILED, null, loaderID, file, e );
                    return;
                    //e.printStackTrace();
                } catch( IOException ex ) {
                    if (setProgress!=null) {
                        progress = -1;
                        timer.stop();
                        SwingUtilities.invokeLater( setProgress );
                    }
                    loaderControl.sceneLoaded( LoaderControlListener.LOAD_FAILED, null, loaderID, file, ex );
                    return;
                    //System.out.println("User Cancel");
                } catch( Exception exc ) {
                    //exc.printStackTrace();
                    if (setProgress!=null) {
                        progress = -1;
                        timer.stop();
                        SwingUtilities.invokeLater( setProgress );
                    }
                    loaderControl.sceneLoaded( LoaderControlListener.LOAD_FAILED, null, loaderID, file, exc );
                    return;
                }
                
            }
        };
        
        if (setProgress!=null) {
            timer = new Timer( 1000, new ProgressTimer() );
            timer.start();
        }
        
        worker.start();
    }
    
  /**
   * Wait for the load to complete
   */
    public void waitForCompletion() {
        try {
            worker.join();
        } catch( InterruptedException e ) {
            e.printStackTrace();
        }
    }
    
    class ProgressTimer implements java.awt.event.ActionListener {
        public void actionPerformed( ActionEvent e ) {
            progress++;
            
            SwingUtilities.invokeLater( setProgress );
        }
    }
    
}

