/*
 *  @(#)J3fLoader.java 1.4 01/01/25 15:52:53
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.loaderwrappers;

import com.sun.j3d.loaders.Scene;
import com.sun.j3d.loaders.SceneBase;

import javax.media.j3d.BranchGroup;

import com.sun.j3d.demos.utils.scenegraph.io.SceneGraphFileReader;

/**
  * Provides a Loader which can load Java3D scene graphs saved using the
 * SceneGraphFileWriter.
 *
 * The Java3D LoaderInterface implemented by this class does not provide access
 * to all the features that can be stored in a .j3f file.
 * Most importantly the file may contain a universe which will be ignored
 * by this loader.
 *
 * @author Paul Byrne
 * @version	1.4, 01/25/01
  */
public class J3fLoader extends com.sun.j3d.loaders.LoaderBase {

    private SceneGraphFileReader reader;
    private static J3fLoaderListener loaderListener = null;
    private BranchGroup[] graphs = null;
    
    /**
     * Setup a listener to get notified when this loader is used
     */
    public static void setJ3fLoaderListener( J3fLoaderListener listener ) {
        loaderListener = listener;
    }
    
    /**
     * Load all the BranchGraphs from the file called filename and return
     * a Scene containing all the BranchGraphs
     */
    public Scene load( String filename ) throws java.io.FileNotFoundException,
                                                com.sun.j3d.loaders.IncorrectFormatException,
                                                com.sun.j3d.loaders.ParsingErrorException {

        SceneBase scenebase = null;
        Scene scene=null;
        try {      
            reader = new SceneGraphFileReader( new java.io.File( filename ));
            int size = reader.getBranchGraphCount();
            scenebase = new SceneBase();

            BranchGroup bg = new BranchGroup();
            graphs = new BranchGroup[ size ];
            
            if (size>1) {            
                for(int i=0; i<size; i++) {
                    graphs[i] = reader.readBranchGraph( i )[0];
                    bg.addChild( graphs[i] );
                }
            } else {
                bg = reader.readBranchGraph(0)[0];
                graphs[0] = bg;
            }
            
            scenebase.setSceneGroup( bg );
            
            String[] objectNames = reader.getNames();
            for(int i=0; i<objectNames.length; i++)
                try {
                    scenebase.addNamedObject( objectNames[i], reader.getNamedObject( objectNames[i] ) );
                } catch( com.sun.j3d.demos.utils.scenegraph.io.ObjectNotLoadedException e ) {}
                  catch( com.sun.j3d.demos.utils.scenegraph.io.NamedObjectException e ) {}

        } catch( java.io.InvalidClassException ex ) {
            throw new com.sun.j3d.loaders.IncorrectFormatException( ex.getMessage() );
        } catch( java.io.FileNotFoundException e ) {
            throw new java.io.FileNotFoundException( filename );
        } catch( java.io.IOException exc ) {
            throw new java.io.FileNotFoundException( exc.getMessage() );
        }
        
        if (loaderListener!=null)
            loaderListener.loadingJ3f( this );
        
        if (scenebase==null)
        return scene;
        else return scenebase;
    }
    
    /**
     * Scene only supports a single sceneBase.
     *
     * For any J3f files containing multiple BranchGraphs all the graphs
     * will be added as children of SceneBase.
     *
     * This call will return the actual set of BranchGraphs (this will
     * still be children of the sceneBase ). If their is only a single
     * branchgraph getBranchGrapgs()[0] == sceneBase
     */
    public BranchGroup[] getBranchGraphs() {
        return graphs;
    }
    
    /**
     * Get the user object for the file, should be called from within a J3fLoaderListener only
     */
    public Object getFileUserData() throws java.io.IOException {
        return reader.readUserData();
    }

    /** Not Implemented
     */
    public Scene load( java.net.URL url ) throws java.io.FileNotFoundException,
                                                com.sun.j3d.loaders.IncorrectFormatException,
                                                com.sun.j3d.loaders.ParsingErrorException {
        throw new RuntimeException("NOT IMPLEMENTED");                                            
    }
    
    /** Not Implemented
     */
    public Scene load( java.io.Reader reader ) throws java.io.FileNotFoundException,
                                                com.sun.j3d.loaders.IncorrectFormatException,
                                                com.sun.j3d.loaders.ParsingErrorException {

        throw new RuntimeException("NOT IMPLEMENTED");                                            
    }
    
    
}

