/*
 *  @(#)SceneGraphFileReader.java 1.6 01/02/06 13:04:59
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io;

import java.io.File;
import java.io.IOException;

import javax.media.j3d.VirtualUniverse;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.SceneGraphObject;

import com.sun.j3d.demos.utils.scenegraph.io.retained.RandomAccessFileControl;

/**
 * Read Java3D BranchGraphs and/or Universe from a file. Individual branchgraphs or an
 * entire Universe can be read and references (shared nodes and components)
 * between the graphs are handled correctly.
 *
 * @author Paul Byrne
 * @version	1.6, 02/06/01
 */
public class SceneGraphFileReader extends java.lang.Object {

    private RandomAccessFileControl fileControl;
    
    /** Creates new SceneGraphFileWriter */
    public SceneGraphFileReader( java.io.File file ) throws IOException {
        fileControl = new RandomAccessFileControl();
        fileControl.openFile( file );
    }

    /**
     * Create and return a universe with the parameters in this file. If the file does
     * not contain a universe null is returned.
     *
     * If attachBranchGraphs is true load and attach all the branchgraphs
     * to the universe
     *
     */
    public VirtualUniverse readUniverse(boolean attachBranchGraphs ) throws IOException {
        return fileControl.readUniverse( attachBranchGraphs );
    }
    
    /**
     * Get the UserData in the File header
     */
    public Object readUserData() throws IOException {
        return fileControl.getUserData();
    }
    
    /**
     * Get the Description of this files contents
     */
    public String readDescription() throws IOException {
        return fileControl.readFileDescription();
    }
    
    /**
     * Return the number of BranchGraphs in the file
     */
    public int getBranchGraphCount() {
        return fileControl.getBranchGraphCount();
    }
    
    /**
     * Read the BranchGraph at index in the file, if the graph
     * contains references to nodes in other BranchGraphs that have not already been
     * loaded then they will also be loaded and returned.
     *
     * The requested graph will always be the first element in the array.
     *
     * The file index of all the Graphs can be discovered using getBranchGraphPosition
     *
     * @param index The index of the Graph in the file. First graph is at index 0
     *
     * @see getBranchGraphPosition( BranchGroup graph )
     *
     */
    public BranchGroup[] readBranchGraph(int index) throws IOException {
        return fileControl.readBranchGraph( index );
    }
    
    /**
     * Read and return all the branchgraphs in the file
     */
    public BranchGroup[] readAllBranchGraphs() throws IOException {
        return fileControl.readAllBranchGraphs();
    }
    
    /**
     * Remove the IO systems reference to this branchgraph and all it's nodes.
     *
     * References to all loaded graphs are maintained by the IO system in
     * order to facilitate node and component sharing between the graphs.
     *
     * This call removes the references to graph <code>index</code>
     *
     * NOT CURRENTLY IMPLEMENTED
     */
    public void dereferenceBranchGraph( BranchGroup graph ) {
        throw new RuntimeException("Not implemented");
    }
    
    /**
     * Given a BranchGraph that has been loaded return the index of the
     * graph in the file
     *
     * NOT CURRENTLY IMPLEMENTED
     */
    public int getBranchGraphPosition( BranchGroup graph ) {
        throw new RuntimeException("Not Impletemented");
    }
    
    /**
     * Read the userdata for the branchgraph at 'index' in the file
     *
     * @param index the index of the graph in the file
     */
    public Object readBranchGraphUserData( int index ) throws IOException {
        return fileControl.readBranchGraphUserData( index );
    }
    
    /**
     * Return the names of all the named objects
     */
    public String[] getNames() {
        return fileControl.getNames();
    }
    
    /**
     * Return the named object. 
     *
     * @param name The name of the object
     *
     * @exception NamedObjectException is thrown if the name is not known to the system
     * @exception ObjectNotLoadedException is thrown if the named object has not been loaded yet
     */
    public SceneGraphObject getNamedObject( String name ) throws NamedObjectException, ObjectNotLoadedException {
        return fileControl.getNamedObject( name );
    }
    
    /**
     * Close the file and cleanup internal data structures
     */
    public void close() throws IOException {
        fileControl.close();
    }
    
}
