/*
 *  @(#)SceneGraphIO.java 1.4 00/12/19 17:22:00
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io;

/**
 * Implement this interface in any classes that subclass a Java3D SceneGraphObject
 * in order to have your class handled correctly by scenegraph.io.
 *
 * More information and example code is provided <A href="doc-files/extensibility.html">here.</a>
 *
 * Classes that implement this interface MUST have a no-arg constructor
 *
 * @author  Paul Byrne
 * @version 1.4 12/19/00
 */
public interface SceneGraphIO {

    /**
     * The method is called before writeSGObject and gives the user the chance
     * to create references to other Nodes and NodeComponents.
     *
     * References take the form of a nodeID, of type integer. Every SceneGraphObject
     * is assigned a unique ID.
     *
     * The user must save the reference information in writeSGObject
     *
     * @param ref provides methods to create references to a SceneGraphObject
     */
    public void createSceneGraphObjectReferences( SceneGraphObjectReferenceControl ref );
    
    /**
     * Within this method the user should restore references to the SceneGraphObjects
     * whose nodeID's were created with createSceneGraphObjectReferences
     * This method is called once the all objects in the scenegraph have been loaded.
     *
     *
     * @param ref provides methods to resolve references to a SceneGraphObject
     */
    public void restoreSceneGraphObjectReferences( SceneGraphObjectReferenceControl ref );
    
    /**
     * This method should store all the local state of the object and any references
     * to other SceneGraphObjects into <code>out</code>.
     *
     * This is called after data for the parent SceneGraphObject has been written to
     * the <code>out</code>. 
     *
     * @param out the output stream
     */
    public void writeSceneGraphObject( java.io.DataOutput out ) throws java.io.IOException;
    
    /**
     * This is called after the object has been constructed and the superclass SceneGraphObject
     * data has been read from <code>in</code>.
     *
     * The user should restore all state infomation written in writeSGObject
     *
     * @param in the input stream
     */
    public void readSceneGraphObject( java.io.DataInput in ) throws java.io.IOException;
    
    /**
     * Flag indicating for children of this object should be saved
     *
     * This method only has an effect if this is a subclass of Group.
     *
     * If this returns true then all children of this Group will be saved.
     *
     * If it returns false then the children are not saved.
     */
    public boolean saveChildren();
}

