/*
 *  @(#)SceneGraphStreamWriter.java 1.5 00/12/19 17:22:05
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io;

import java.io.File;
import java.io.IOException;
import java.io.DataOutputStream;
import java.util.HashMap;

import javax.media.j3d.VirtualUniverse;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.CapabilityNotSetException;
import javax.media.j3d.DanglingReferenceException;
import com.sun.j3d.demos.utils.scenegraph.io.retained.StreamControl;

/**
 * Writes a Java3D SceneGraph to a Java OutputStream.
 * Using this class to write to a FileOutputStream is not recommended, use
 * SceneGraphFileWriter for writing files inorder to achieve maximum read
 * performance and flexibility.
 *
 *
 *
 * @author Paul Byrne
 * @version	1.5, 12/19/00
 */
public class SceneGraphStreamWriter extends java.lang.Object {
    
    private StreamControl control;
    private DataOutputStream out;

    /** Creates new SceneGraphStreamWriter which will output to the supplied stream */
    public SceneGraphStreamWriter(java.io.OutputStream outputStream ) {
        this.out = new java.io.DataOutputStream( outputStream );
        control = new StreamControl( out );
    }
    

    /**
     * Write <code>universe</code> to the Stream.  
     *
     * If <writeContent> is true then all the BranchGraphs attached to the
     * universe will be saved. If it is false then only the universe
     * data structures will be ouput.
     *
     * If <code>writeContent/code> is true then all the BranchGraphs
     * attached to the Locales of the universe must have the
     * ALLOW_DETACH capability set. If they do not a CapabilityNotSetException
     * will be thrown
     *
     * @param universe The universe to write
     * @param writeContent Flag enabling the BranchGraphs to be written
     *
     * @exception IOException
     * @exception UnsupportedUniverseException Thrown if the universe class is not
     * supported by this implementation
     */
    public void writeUniverse( VirtualUniverse universe, boolean writeContent ) throws IOException, UnsupportedUniverseException {
        control.writeUniverse( out, universe, writeContent );
    }
    
    /**
     * Write the entire graph to the stream.
     *
     * The API will correctly handle NodeComponents that are shared
     * between seperate graphs. However Node's cannot be referenced
     * in other Graphs.
     *
     * If a reference to a Node in another graph is encountered a
     * DanglingReferenceException will be thrown.
     *
     * namedObjects can contain a mapping between a key and a SceneGraphObject
     * in the graph. During the read process this can be used to locate nodes
     * in the graph.
     */
    public void writeBranchGraph( BranchGroup graph, HashMap namedObjects ) throws IOException, DanglingReferenceException, NamedObjectException {
        // TODO Add namedObjects to SymbolTable
        control.addNamedObjects( namedObjects );
        control.writeBranchGraph( graph, null );
    }
    
    /** 
     * Close the SceneGraphStreamWriter, but does not close the Stream
     */
    public void close() throws IOException {
        control.close();
    }
}
