/*
 *  @(#)StreamControl.java 1.6 01/03/02 14:04:24
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io.retained;

import java.io.RandomAccessFile;
import java.io.IOException;
import java.io.DataOutput;
import java.io.DataInput;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.util.HashMap;
import java.util.Iterator;

import javax.media.j3d.VirtualUniverse;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.SceneGraphObject;

import com.sun.j3d.demos.utils.scenegraph.io.state.javax.media.j3d.SceneGraphObjectState;
import com.sun.j3d.demos.utils.scenegraph.io.state.javax.media.j3d.NodeComponentState;
import com.sun.j3d.demos.utils.scenegraph.io.state.javax.media.j3d.BranchGroupState;
import com.sun.j3d.demos.utils.scenegraph.io.UnsupportedUniverseException;

/**
 * Provides the infrastructure for ScenGraphStream Reader and Writer
 *
 * @author Paul Byrne
 * @version	1.6, 03/02/01
 */
public class StreamControl extends Controller {
        
    private DataInputStream inputStream;
    private DataOutputStream outputStream;
        
    public StreamControl( DataOutputStream out ) {
        super();
        outputStream = out;
        symbolTable = new SymbolTable( this );
    }
    
    public StreamControl( DataInputStream in ) {
        super();
        inputStream = in;
        symbolTable = new SymbolTable( this );
    }
        
    /**
     * Add the named objects to the symbol table
     */
    public void addNamedObjects( HashMap namedObjects ) {
        symbolTable.addNamedObjects( namedObjects );
    }
     
    /** userData is not supported for Streams and will be ignored
     */
    public void writeBranchGraph( BranchGroup bg, java.io.Serializable userData ) throws IOException {
        SymbolTableData symbol = symbolTable.getSymbol( bg );
        
        if (symbol==null) {
            symbol = symbolTable.createSymbol( bg );
            symbol.branchGraphID = -1;          // This is a new BranchGraph so set the ID to -1
        }                                       // which will cause setBranchGraphRoot to assign a new ID.
        
        symbolTable.setBranchGraphRoot( symbol, 0 );        
        symbolTable.startUnsavedNodeComponentFrame();
        SceneGraphObjectState state = createState( bg, symbol );
        writeObject( outputStream, state );
        writeNodeComponents( outputStream );
        symbolTable.endUnsavedNodeComponentFrame();
        
        if (symbolTable.branchGraphHasDependencies( symbol.branchGraphID ))
            throw new javax.media.j3d.DanglingReferenceException();
       
        symbolTable.clearUnshared();
        symbolTable.writeTable( outputStream );
    }
    
    public BranchGroup readBranchGraph( HashMap namedObjects ) throws IOException {
        SceneGraphObjectState state = readObject( inputStream );
        readNodeComponents( inputStream );
        symbolTable.readTable( inputStream, true );
        
        symbolTable.setBranchGraphRoot( state.getSymbol(), 0 );
        
        state.buildGraph();
        
        symbolTable.getNamedObjectMap( namedObjects );
        
        return (BranchGroup)state.getNode();
    }
    
    /**
     * Get the file version of the file we are reading
     */
    public int getCurrentFileVersion() {
        return getOutputFileVersion();
    }
    /**
     * Read the set of branchgraps.
     *
     * Used by readUniverse
     *
     * RandomAccessFileControl will read the graphs in the array,
     * StreamControl expects the graphs to follow the universe in the
     * stream so it will read graphs.length branchgraphs.
     */
    protected void readBranchGraphs( int[] graphs ) throws IOException {
        for(int i=0; i<graphs.length; i++)
            readBranchGraph( null );
    }
    
    /**
     * Used by SymbolTable to load a node component that is not in current
     * graph
     */
    public void loadNodeComponent( SymbolTableData symbol ) throws IOException {
        throw new RuntimeException("Unable to load individual NodeComponents from Stream");
    }
    
    public void close() throws IOException {
        super.reset();
    }
    
    /**
     * Implementation of abstract method from Controller.
     *
     * Always returns 0
     */
    public long getFilePointer() {
        return 0L;
    }
    
}
