/*
 *  @(#)SymbolTableData.java 1.4 00/12/19 16:58:28
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io.retained;

import java.io.DataOutput;
import java.io.DataInput;
import java.io.IOException;
import javax.media.j3d.SceneGraphObject;
import com.sun.j3d.demos.utils.scenegraph.io.state.javax.media.j3d.SceneGraphObjectState;

/**
 * Encapsulates all the data for a node which is stored in the Symbol table
 *
 * @author  Paul Byrne
 * @version 1.4 12/19/00
 */
public class SymbolTableData extends java.lang.Object {

    public int nodeID;
    public SceneGraphObjectState nodeState;
    public SceneGraphObject j3dNode;
    public int referenceCount;
    public long filePosition;
    public boolean isNodeComponent;
    public int branchGraphID;
    public long branchGraphFilePointer;
    public boolean graphBuilt = false;
    
    /** Creates new SymbolTableData */
    public SymbolTableData( int nodeID, 
                            SceneGraphObject j3dNode,
                            SceneGraphObjectState nodeState,
                            int branchGraphID ) {
        this.nodeID = nodeID;
        this.j3dNode = j3dNode;
        this.nodeState = nodeState;
        this.branchGraphID = branchGraphID;
        this.referenceCount = 1;
        this.isNodeComponent = false;
    }

    public void writeObject( DataOutput out ) throws IOException {
        out.writeInt( nodeID );
        out.writeInt( referenceCount );
        out.writeLong( filePosition );
        out.writeBoolean( isNodeComponent );
        out.writeInt( branchGraphID );
        out.writeLong( branchGraphFilePointer );
    }
    
    public void readObject( DataInput in ) throws IOException {
        nodeID = in.readInt();
        referenceCount = in.readInt();
        filePosition = in.readLong();
        isNodeComponent = in.readBoolean();
        branchGraphID = in.readInt();
        branchGraphFilePointer = in.readLong();
    }
    
    public final int getNodeID() {
        return nodeID;
    }
    
    public final SceneGraphObjectState getNodeState() {
        return nodeState;
    }
    
    public final void setNodeState( SceneGraphObjectState state ) {
        nodeState = state;
    }
    
    public final SceneGraphObject getJ3dNode() {
        return j3dNode;
    }
    
    public final long getFilePosition() {
        return filePosition;
    }
    
    public final int getReferenceCount() {
        return referenceCount;
    }
    
    public final void incrementReferenceCount() {
        referenceCount++;
    }
    
    public final boolean isNodeComponent() {
        return isNodeComponent;
    }
    
    public String toString() {
        return new String(nodeID +" "+ filePosition+"  "+j3dNode);
    }
}
