/*
 *  @(#)NcFont3DState.java 1.4 01/03/02 15:32:51
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io.state.com.sun.j3d.demos.utils.scenegraph.io.retained;

import java.io.IOException;
import java.io.DataInput;
import java.io.DataOutput;
import java.util.Map;
import javax.media.j3d.Font3D;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.FontExtrusion;
import com.sun.j3d.demos.utils.scenegraph.io.retained.Controller;
import com.sun.j3d.demos.utils.scenegraph.io.retained.SymbolTableData;
import com.sun.j3d.demos.utils.scenegraph.io.retained.NcFont3D;
import com.sun.j3d.demos.utils.scenegraph.io.state.javax.media.j3d.NodeComponentState;

/**
 * Font3D is not a subclass of NodeComponent however it is important that the
 * API supports sharing of Font3D's between graphs.
 *
 * Therefore this class uses NcFont3D which is a subclass of NodeComponent and
 * provides a representation of a Font3D.
 *
 * @author Paul Byrne
 * @version 1.4 03/02/01
 */
public class NcFont3DState extends NodeComponentState {

    private java.awt.Font font;
    private FontExtrusion fontExtrusion;
    private double tessellationTolerance;
    
    
    public NcFont3DState(SymbolTableData symbol,Controller control) {
        super( symbol, control );
        
    }
    
    public void writeConstructorParams( DataOutput out ) throws IOException {
        super.writeConstructorParams( out );
        
        fontExtrusion = new FontExtrusion();
        ((NcFont3D)node).getFontExtrusion( fontExtrusion );
        
        writeFontExtrusion( out );
        
        font = ((NcFont3D)node).getFont();
        
        writeFont( out );
        
        out.writeDouble( ((NcFont3D)node).getTessellationTolerance() );
    }
    
    public void readConstructorParams( DataInput in ) throws IOException {
        super.readConstructorParams( in );
        
        readFontExtrusion( in );
        
        readFont( in );
        
        tessellationTolerance = in.readDouble();
    }
    
    private void writeFontExtrusion( DataOutput out ) throws IOException {
        java.awt.Shape shape = fontExtrusion.getExtrusionShape();
        
        if ( shape!=null && !(shape instanceof java.io.Serializable) )
            throw new IOException("FontExtrusion Shape is not Serializable");
        
        control.writeSerializedData( out, (java.io.Serializable)shape );
        
        out.writeDouble( fontExtrusion.getTessellationTolerance() );
    }
    
    private void readFontExtrusion( DataInput in ) throws IOException {
        java.awt.Shape shape = (java.awt.Shape)control.readSerializedData( in );
        double tessellationTol = in.readDouble();
        
        fontExtrusion = new FontExtrusion( shape, tessellationTol );
    }
     
    private void writeFont( DataOutput out ) throws IOException {
        Map map = font.getAttributes();
        control.writeSerializedData( out, (java.io.Serializable)map );
    }
    
    private void readFont( DataInput in ) throws IOException {
        Map map = (Map)control.readSerializedData( in );
        font = new java.awt.Font( map );
    }
    
    public SceneGraphObject createNode( Class j3dClass ) {
        return createNode( j3dClass, new Class[] { java.awt.Font.class,
                                                    Double.TYPE,
                                                    FontExtrusion.class },
                                      new Object[] { font,
                                                     new Double( tessellationTolerance ),
                                                     fontExtrusion } );
         
    }
    
    protected javax.media.j3d.SceneGraphObject createNode() {
        return new NcFont3D( font, tessellationTolerance, fontExtrusion );
    }

}
