/*
 *  @(#)BoxState.java 1.6 01/03/02 15:32:59
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io.state.com.sun.j3d.utils.geometry;

import java.io.*;
import com.sun.j3d.utils.geometry.Box;
import javax.media.j3d.Appearance;
import javax.media.j3d.SceneGraphObject;
import javax.vecmath.Color3f;
import com.sun.j3d.demos.utils.scenegraph.io.retained.Controller;
import com.sun.j3d.demos.utils.scenegraph.io.retained.SymbolTableData;

/**
 * @author Paul Byrne
 * @version	1.6, 03/02/01
 *
 */
public class BoxState extends PrimitiveState {
    
    private float xdim;
    private float ydim;
    private float zdim;
    private int frontAppearance;
    private int backAppearance;
    private int topAppearance;
    private int bottomAppearance;
    private int leftAppearance;
    private int rightAppearance;

    public BoxState( SymbolTableData symbol, Controller control ) {
	super( symbol, control );
        
        if (node!=null) {
            frontAppearance = control.getSymbolTable().addReference( ((Box)node).getShape( Box.FRONT ).getAppearance() );
            backAppearance = control.getSymbolTable().addReference( ((Box)node).getShape( Box.BACK ).getAppearance() );
            topAppearance = control.getSymbolTable().addReference( ((Box)node).getShape( Box.TOP ).getAppearance() );
            bottomAppearance = control.getSymbolTable().addReference( ((Box)node).getShape( Box.BOTTOM ).getAppearance() );
            leftAppearance = control.getSymbolTable().addReference( ((Box)node).getShape( Box.LEFT ).getAppearance() );
            rightAppearance = control.getSymbolTable().addReference( ((Box)node).getShape( Box.RIGHT ).getAppearance() );
        }
    }

    public void writeObject( DataOutput out ) throws IOException {
	super.writeObject( out );
                
        out.writeInt( frontAppearance );
        out.writeInt( backAppearance );
        out.writeInt( topAppearance );
        out.writeInt( bottomAppearance );
        out.writeInt( leftAppearance );
        out.writeInt( rightAppearance );
    }

    public void readObject( DataInput in ) throws IOException {
       super.readObject(in);
       
       frontAppearance = in.readInt();
       backAppearance = in.readInt();
       topAppearance = in.readInt();
       bottomAppearance = in.readInt();
       leftAppearance = in.readInt();
       rightAppearance = in.readInt();
    }
    
    public void writeConstructorParams( DataOutput out ) throws IOException {
	super.writeConstructorParams( out );
                
        out.writeFloat( ((Box)node).getXdimension() );
        out.writeFloat( ((Box)node).getYdimension() );
        out.writeFloat( ((Box)node).getZdimension() );
    }

    public void readConstructorParams( DataInput in ) throws IOException {
       super.readConstructorParams(in);
       
       xdim = in.readFloat();
       ydim = in.readFloat();
       zdim = in.readFloat();
    }
    
    public void buildGraph() {
        super.buildGraph();
        
        if (frontAppearance == backAppearance &&
            frontAppearance == topAppearance &&
            frontAppearance == bottomAppearance &&
            frontAppearance == leftAppearance &&
            frontAppearance == rightAppearance ) {
            ((Box)node).setAppearance( (Appearance)control.getSymbolTable().getJ3dNode( frontAppearance ));
        } else {
            ((Box)node).setAppearance( Box.FRONT, (Appearance)control.getSymbolTable().getJ3dNode( frontAppearance ));
            ((Box)node).setAppearance( Box.BACK, (Appearance)control.getSymbolTable().getJ3dNode( backAppearance ));
            ((Box)node).setAppearance( Box.TOP, (Appearance)control.getSymbolTable().getJ3dNode( topAppearance ));
            ((Box)node).setAppearance( Box.BOTTOM, (Appearance)control.getSymbolTable().getJ3dNode( bottomAppearance ));
            ((Box)node).setAppearance( Box.LEFT, (Appearance)control.getSymbolTable().getJ3dNode( leftAppearance ));
            ((Box)node).setAppearance( Box.RIGHT, (Appearance)control.getSymbolTable().getJ3dNode( rightAppearance ));
        }
    }

    public SceneGraphObject createNode( Class j3dClass ) {
        Box box = (Box)super.createNode( j3dClass, new Class[] { Float.TYPE,
                                                   Float.TYPE,
                                                   Float.TYPE,
                                                   Integer.TYPE,
                                                   Appearance.class },
                                     new Object[] { new Float( xdim ),
                                                    new Float( ydim ),
                                                    new Float( zdim ),
                                                    new Integer( primflags ),
                                                    null } );
        return box;
    }
    
    protected javax.media.j3d.SceneGraphObject createNode() {
        return new Box( xdim, ydim, zdim, primflags, null );
    }

}
