/*
 *  @(#)ConeState.java 1.6 01/03/02 15:33:11
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io.state.com.sun.j3d.utils.geometry;

import java.io.*;
import com.sun.j3d.utils.geometry.Cone;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.Appearance;
import javax.media.j3d.PolygonAttributes;
import javax.media.j3d.AmbientLight;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.Material;
import com.sun.j3d.demos.utils.scenegraph.io.retained.Controller;
import com.sun.j3d.demos.utils.scenegraph.io.retained.SymbolTableData;

/**
 * @author Paul Byrne
 * @version	1.6, 03/02/01
 *
 */
public class ConeState extends PrimitiveState {

    private float radius=1f;
    private float height=2f;
    private int xdivision=15;
    private int ydivision=1;
    private int bodyAppearance=0;
    private int capAppearance=0;
    
    public ConeState(SymbolTableData symbol,Controller control) {
	super( symbol, control );
        
        if (node!=null) {
            bodyAppearance = control.getSymbolTable().addReference( ((Cone)node).getShape( Cone.BODY ).getAppearance() );
            capAppearance = control.getSymbolTable().addReference( ((Cone)node).getShape( Cone.CAP ).getAppearance() );
        }
    }

    public void writeObject( DataOutput out ) throws IOException {
	super.writeObject( out );
        out.writeInt( bodyAppearance );
        out.writeInt( capAppearance );
    }

    public void readObject( DataInput in ) throws IOException {
       super.readObject(in);
       bodyAppearance = in.readInt();
       capAppearance = in.readInt();
    }
    
    public void writeConstructorParams( DataOutput out ) throws IOException {
	super.writeConstructorParams( out );
        
        out.writeFloat( ((Cone)node).getRadius() );
        out.writeFloat( ((Cone)node).getHeight() );
        out.writeInt( ((Cone)node).getXdivisions() );
        out.writeInt( ((Cone)node).getYdivisions() );
    }

    public void readConstructorParams( DataInput in ) throws IOException {
       super.readConstructorParams(in);
       
       radius = in.readFloat();
       height = in.readFloat();
       xdivision = in.readInt();
       ydivision = in.readInt();
    }
    
    public void buildGraph() {
        super.buildGraph();

        if (bodyAppearance == capAppearance ) {
            ((Cone)node).setAppearance( (Appearance)control.getSymbolTable().getJ3dNode( bodyAppearance ));
        } else {
            ((Cone)node).setAppearance( Cone.BODY, (Appearance)control.getSymbolTable().getJ3dNode( bodyAppearance ));
            ((Cone)node).setAppearance( Cone.CAP, (Appearance)control.getSymbolTable().getJ3dNode( capAppearance ));
        }
        
    }
    
    public SceneGraphObject createNode( Class j3dClass ) {
        // Create the node with a null appearance, we will add the appearance
        // during build graph
        Cone cone = (Cone)createNode( j3dClass, new Class[] {
                                                Float.TYPE,
                                                Float.TYPE,
                                                Integer.TYPE,
                                                Integer.TYPE,
                                                Integer.TYPE,
                                                Appearance.class },
                                        new Object[] {
                                                new Float( radius ),
                                                new Float( height ),
                                                new Integer( primflags ),
                                                new Integer( xdivision ),
                                                new Integer( ydivision ),
                                                null } );
                                                
        return cone;
    }
    
    protected javax.media.j3d.SceneGraphObject createNode() {
        return new Cone( radius, height, primflags, xdivision, ydivision, null );
    }

    
}
