/*
 *  @(#)SphereState.java 1.6 01/03/02 15:33:24
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io.state.com.sun.j3d.utils.geometry;

import java.io.*;
import com.sun.j3d.utils.geometry.Sphere;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.Appearance;
import com.sun.j3d.demos.utils.scenegraph.io.retained.Controller;
import com.sun.j3d.demos.utils.scenegraph.io.retained.SymbolTableData;

/**
 * @author Paul Byrne
 * @version	1.6, 03/02/01
 *
 */
public class SphereState extends PrimitiveState {

    private float radius;
    private int divisions;
    private int bodyAppearance;
    
    public SphereState( SymbolTableData symbol, Controller control ) {
	super( symbol, control );
        
        if (node!=null) {
            bodyAppearance = control.getSymbolTable().addReference( ((Sphere)node).getShape( Sphere.BODY ).getAppearance() );
        }
    }

    public void writeObject( DataOutput out ) throws IOException {
	super.writeObject( out );
        
        out.writeInt( bodyAppearance );
    }

    public void readObject( DataInput in ) throws IOException {
       super.readObject(in);
       
       bodyAppearance = in.readInt();
    }
    
    public void writeConstructorParams( DataOutput out ) throws IOException {
	super.writeConstructorParams( out );
        
        out.writeFloat( ((Sphere)node).getRadius() );
        out.writeInt( ((Sphere)node).getDivisions() );        
    }

    public void readConstructorParams( DataInput in ) throws IOException {
       super.readConstructorParams(in);
       
       radius = in.readFloat();
       divisions = in.readInt();
    }

    public void buildGraph() {
        super.buildGraph();
        ((Sphere)node).setAppearance( (Appearance)control.getSymbolTable().getJ3dNode( bodyAppearance ));
    }
    
    public SceneGraphObject createNode( Class j3dClass ) {
        // Create the node with a null appearance, we will add the appearance
        // during build graph
        Sphere sphere = (Sphere)createNode( j3dClass, new Class[] {
                                                Float.TYPE,
                                                Integer.TYPE,
                                                Integer.TYPE,
                                                javax.media.j3d.Appearance.class },
                                        new Object[] {
                                                new Float( radius ),
                                                new Integer( primflags ),
                                                new Integer( divisions ),
                                                null } );
                                                
        return sphere;
    }
    
    protected javax.media.j3d.SceneGraphObject createNode() {
        return new Sphere( radius, primflags, divisions, null );
    }

}
