/*
 *  @(#)CompressedGeometryState.java 1.4 01/03/02 15:34:44
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io.state.javax.media.j3d;

import java.io.IOException;
import java.io.DataInput;
import java.io.DataOutput;
import javax.media.j3d.CompressedGeometry;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.CompressedGeometryHeader;
import com.sun.j3d.demos.utils.scenegraph.io.retained.Controller;
import com.sun.j3d.demos.utils.scenegraph.io.retained.SymbolTableData;

/**
 * @author Paul Byrne
 * @version 1.4 03/02/01
 */
public class CompressedGeometryState extends BehaviorState {
    
    private byte[] bytes;
    private boolean isByReference;
    private CompressedGeometryHeader header;
    
    
    public CompressedGeometryState(SymbolTableData symbol,Controller control) {
        super( symbol, control );
        
    }
    
    public void writeConstructorParams( DataOutput out ) throws IOException {
        super.writeConstructorParams( out );
        
        out.writeBoolean( ((CompressedGeometry)node).isByReference() );
        
        int size = ((CompressedGeometry)node).getByteCount();
        out.writeInt( size );
        bytes = new byte[ size ];
        ((CompressedGeometry)node).getCompressedGeometry( bytes );
        out.write( bytes );
        
        header = new CompressedGeometryHeader();
        ((CompressedGeometry)node).getCompressedGeometryHeader( header );
        writeCompressedGeometryHeader( out );
    }
    
    public void readConstructorParams( DataInput in ) throws IOException {
        super.readConstructorParams( in );
        
        isByReference = in.readBoolean();
        bytes = new byte[ in.readInt() ];
        in.readFully( bytes );
        
        header = new CompressedGeometryHeader();
        readCompressedGeometryHeader( in );
    }
    
    private void writeCompressedGeometryHeader( DataOutput out ) throws IOException {
        out.writeInt( header.majorVersionNumber );
        out.writeInt( header.minorVersionNumber );
        out.writeInt( header.minorMinorVersionNumber );
        out.writeInt( header.bufferType );
        out.writeInt( header.bufferDataPresent );
        out.writeInt( header.size );
        out.writeInt( header.start );
        control.writePoint3d( out, header.lowerBound );
        control.writePoint3d( out, header.upperBound );
    }
    
    private void readCompressedGeometryHeader( DataInput in ) throws IOException {
        header.majorVersionNumber = in.readInt();
        header.minorVersionNumber = in.readInt();
        header.minorMinorVersionNumber = in.readInt();
        header.bufferType = in.readInt();
        header.bufferDataPresent = in.readInt();
        header.size = in.readInt();
        header.start = in.readInt();
        header.lowerBound = control.readPoint3d( in );
        header.upperBound = control.readPoint3d( in );
    }
     
    public SceneGraphObject createNode( Class j3dClass ) {
        
        return createNode( j3dClass, new Class[] { CompressedGeometryHeader.class,
                                                    bytes.getClass(),
                                                    Boolean.TYPE },
                                      new Object[] { header,
                                                     bytes,
                                                     new Boolean(isByReference) } );
    }
    
    protected SceneGraphObject createNode() {
        return new CompressedGeometry( header, bytes, isByReference );
    }

}
