/*
 *  @(#)GeometryArrayState.java 1.6 01/02/07 14:45:57
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io.state.javax.media.j3d;

import java.io.*;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.SceneGraphObject;
import javax.vecmath.*;
import com.sun.j3d.demos.utils.scenegraph.io.retained.Controller;
import com.sun.j3d.demos.utils.scenegraph.io.retained.SymbolTableData;

/**
 * @author Paul Byrne
 * @version	1.6, 02/07/01
 */
public abstract class GeometryArrayState extends GeometryState {

    protected int vertexFormat;
    protected int vertexCount;
    protected int texCoordSetCount;
    protected int[] texCoordSetMap;
    
    public GeometryArrayState( SymbolTableData symbol, Controller control ) {
	super( symbol, control );
    }

    public void writeObject( DataOutput out ) throws IOException {
        super.writeObject( out );
        
	if ((vertexFormat & GeometryArray.COLOR_3) !=0 ) {
	    //System.out.println("COLOR 3");
	    float[] colors3 = new float[ vertexCount*3 ];
	    ((GeometryArray)node).getColors( 0, colors3 );

	    writeFloatArray( out, colors3 );
	} else if ( (vertexFormat & GeometryArray.COLOR_4)!=0 ) {
	    //System.out.println("COLOR 4");
	    float[] colors = new float[ vertexCount*4 ];
	    ((GeometryArray)node).getColors( 0, colors );

	    writeFloatArray( out, colors );
	}


	if ((vertexFormat & GeometryArray.COORDINATES)!=0 ) {
	    //System.out.println("COORDS");
	    float[] points = new float[ vertexCount*3 ];
	    ((GeometryArray)node).getCoordinates( 0, points );

	    writeFloatArray( out, points );
	}

	if ((vertexFormat & GeometryArray.NORMALS)!=0) {
	    //System.out.println("NORMALS");
	    float[] normals = new float[ vertexCount*3 ];
	    ((GeometryArray)node).getNormals( 0, normals );

	    writeFloatArray( out, normals );
	}

	if ((vertexFormat & GeometryArray.TEXTURE_COORDINATE_2)!=0) {
	    float[] textureCoords = new float[ vertexCount*2 ];
            for(int set=0; set<texCoordSetCount; set++) {
                ((GeometryArray)node).getTextureCoordinates( set, 0, textureCoords );

                writeFloatArray( out, textureCoords );
            }
	} else if ((vertexFormat & GeometryArray.TEXTURE_COORDINATE_3)!=0) {
	    float[] textureCoords = new float[ vertexCount*3 ];
            for(int set=0; set<texCoordSetCount; set++) {
                ((GeometryArray)node).getTextureCoordinates( set, 0, textureCoords );

                writeFloatArray( out, textureCoords );
            }
	}

    }

    public void readObject( DataInput in ) throws IOException {
        super.readObject( in );
        
       if ((vertexFormat & GeometryArray.COLOR_3) !=0 ) {
           float[] colors3 = readFloatArray( in );
           ((GeometryArray)node).setColors( 0, colors3 );
       } else if ( (vertexFormat & GeometryArray.COLOR_4)!=0 ) {
           float[] colors = readFloatArray( in );
           ((GeometryArray)node).setColors( 0, colors );
       }

       if ((vertexFormat & GeometryArray.COORDINATES)!=0 ) {
           float[] points = readFloatArray(in );
           ((GeometryArray)node).setCoordinates( 0, points );
       }

       if ((vertexFormat & GeometryArray.NORMALS)!=0) {
           float[] normals = readFloatArray( in );
           ((GeometryArray)node).setNormals( 0, normals );
       }

       if ((vertexFormat & GeometryArray.TEXTURE_COORDINATE_2 |
           vertexFormat & GeometryArray.TEXTURE_COORDINATE_3)!=0 ) {
           for(int set=0; set<texCoordSetCount; set++) {
               float[] textureCoords = readFloatArray( in );
               ((GeometryArray)node).setTextureCoordinates( set, 0, textureCoords );
           }
       }
    }

    protected void writeConstructorParams( DataOutput out ) throws
								IOException {
	vertexCount = ((GeometryArray)node).getVertexCount();
	vertexFormat = ((GeometryArray)node).getVertexFormat();
        texCoordSetCount = ((GeometryArray)node).getTexCoordSetCount();
        texCoordSetMap = new int[ ((GeometryArray)node).getTexCoordSetMapLength() ];
        
        ((GeometryArray)node).getTexCoordSetMap( texCoordSetMap );
        
	out.writeInt(vertexCount);
	out.writeInt(vertexFormat);
        out.writeInt( texCoordSetCount );
        out.writeInt( texCoordSetMap.length );
        for(int i=0; i<texCoordSetMap.length; i++)
            out.writeInt( texCoordSetMap[i] );
        super.writeConstructorParams( out );
    }

    protected void readConstructorParams( DataInput in ) throws
							IOException {
       // Load VertexCount and format first beause
       // SceneGraphObjectState will call createNode which
       // requires them
       vertexCount = in.readInt();
       vertexFormat = in.readInt();
       texCoordSetCount = in.readInt();
       texCoordSetMap = new int[in.readInt()];
       for(int i=0; i<texCoordSetMap.length; i++)
           texCoordSetMap[i] = in.readInt();
       
       super.readConstructorParams( in );
    }
    
    protected void writeFloatArray( DataOutput out, float[] array ) throws IOException {
        ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
        DataOutputStream dataOut = new DataOutputStream( byteStream );

        dataOut.writeInt( array.length );
        for(int i=0; i<array.length; i++)
            dataOut.writeFloat( array[i] );
        dataOut.close();
        
        out.writeInt( byteStream.size() );
        out.write( byteStream.toByteArray() );
    }
    
    protected float[] readFloatArray( DataInput in ) throws IOException {
        byte[] buffer = new byte[ in.readInt() ];
        in.readFully( buffer );
        ByteArrayInputStream byteStream = new ByteArrayInputStream( buffer );
        DataInputStream dataIn = new DataInputStream( byteStream );
        
        float[] array = new float[ dataIn.readInt() ];
        for(int i=0; i<array.length; i++)
            array[i] = dataIn.readFloat();
        
        dataIn.close();
        
        return array;
    }
}
