/*
 *  @(#)GroupState.java 1.5 01/03/02 15:33:57
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io.state.javax.media.j3d;

import java.io.*;
import java.util.Enumeration;
import javax.media.j3d.Group;
import javax.media.j3d.Node;
import javax.media.j3d.SceneGraphObject;
import com.sun.j3d.demos.utils.scenegraph.io.retained.Controller;
import com.sun.j3d.demos.utils.scenegraph.io.retained.SymbolTableData;

public class GroupState extends NodeState {
    
    protected SceneGraphObjectState[] groupChildren;        // State calssed for all the children during load

    public GroupState( SymbolTableData symbol, Controller control ) {
        super(symbol, control);        
    }

  public void writeObject( DataOutput out) throws IOException {
    super.writeObject( out );

    control.writeBounds( out, ((Group)node).getCollisionBounds() );
    
    int numChildren;
    if (processChildren())
        numChildren = ((Group)node).numChildren();
    else
        numChildren = 0;
    
    out.writeInt( numChildren );
        
    for(int i=0; i<numChildren; i++ ) {
        control.writeObject( out, control.createState( ((Group)node).getChild(i) ) );
    }
    
    out.writeBoolean( ((Group)node).getAlternateCollisionTarget() );
  }

  public void readObject( DataInput in ) throws IOException {
      super.readObject( in );

    ((Group)node).setCollisionBounds( control.readBounds( in ));

    int numChildren = in.readInt();
    groupChildren = new SceneGraphObjectState[ numChildren ];
    for(int i=0; i<numChildren; i++ ) {
        groupChildren[i] = control.readObject( in );
        ((Group)node).addChild( (Node)groupChildren[i].getNode() );
    }
    
    ((Group)node).setAlternateCollisionTarget( in.readBoolean() );
  }
  
  /**
   * Returns true if the groups children should be saved.
   *
   * This is overridden by 'black box' groups such a geometry primitives
   */
  protected boolean processChildren() {
      return true;
  }
  
  public void buildGraph() {
      super.buildGraph();
      
      for(int i=0; i<groupChildren.length; i++) {
          if (!groupChildren[i].getSymbol().graphBuilt) {
              groupChildren[i].getSymbol().graphBuilt = true;
              groupChildren[i].buildGraph();
          }
      }
  }
  
  public void cleanup() {
      for(int i=0; i<groupChildren.length; i++) {
          groupChildren[i].cleanup();
          groupChildren[i] = null;
      }
      
      groupChildren = null;
      super.cleanup();
  }
    
    protected javax.media.j3d.SceneGraphObject createNode() {
        return new Group();
    }

  
}
