/*
 *  @(#)IndexedGeometryArrayState.java 1.4 01/02/07 14:45:58
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io.state.javax.media.j3d;

import java.io.*;
import javax.media.j3d.IndexedGeometryArray;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.SceneGraphObject;
import javax.vecmath.*;
import com.sun.j3d.demos.utils.scenegraph.io.retained.Controller;
import com.sun.j3d.demos.utils.scenegraph.io.retained.SymbolTableData;

/**
 * @author Paul Byrne
 * @version	1.4, 02/07/01
 */
public abstract class IndexedGeometryArrayState extends GeometryArrayState {

    protected int indexCount;
    
    public IndexedGeometryArrayState(SymbolTableData symbol,Controller control) {
	super( symbol, control );
    }

    public void writeObject( DataOutput out ) throws IOException {
        super.writeObject( out );
        
        int[] indices = new int[ ((IndexedGeometryArray)node).getIndexCount() ];
        
	if ((vertexFormat & GeometryArray.COLOR_3 |
             vertexFormat & GeometryArray.COLOR_4) !=0 ) {
	    //System.out.println("COLOR 3");
	    ((IndexedGeometryArray)node).getColorIndices( 0, indices );

	    writeIntArray( out, indices );
	}

	if ((vertexFormat & GeometryArray.COORDINATES)!=0 ) {
	    //System.out.println("COORDS");
	    ((IndexedGeometryArray)node).getCoordinateIndices( 0, indices );

	    writeIntArray( out, indices );
	}

	if ((vertexFormat & GeometryArray.NORMALS)!=0) {
	    //System.out.println("NORMALS");
	    ((IndexedGeometryArray)node).getNormalIndices( 0, indices );

	    writeIntArray( out, indices );
	}

	if ((vertexFormat & GeometryArray.TEXTURE_COORDINATE_2 |
             vertexFormat & GeometryArray.TEXTURE_COORDINATE_3)!=0) {            
            for(int i=0; i<((IndexedGeometryArray)node).getTexCoordSetCount(); i++) {
	        ((IndexedGeometryArray)node).getTextureCoordinateIndices( i, 0, indices );

	        writeIntArray( out, indices );
            }
	}

    }

    public void readObject( DataInput in ) throws IOException {
        super.readObject( in );
      
        int[] indices = new int[indexCount];
        
        if ((vertexFormat & GeometryArray.COLOR_3 |
             vertexFormat & GeometryArray.COLOR_4) !=0 ) {
           readIntArray( in, indices );
           ((IndexedGeometryArray)node).setColorIndices( 0, indices );
       }

       if ((vertexFormat & GeometryArray.COORDINATES)!=0 ) {
           readIntArray( in, indices );
           ((IndexedGeometryArray)node).setCoordinateIndices( 0, indices );
       }

       if ((vertexFormat & GeometryArray.NORMALS)!=0) {
           readIntArray( in, indices );
           ((IndexedGeometryArray)node).setNormalIndices( 0, indices );
       }

       if ((vertexFormat & GeometryArray.TEXTURE_COORDINATE_2 |
           vertexFormat & GeometryArray.TEXTURE_COORDINATE_3)!=0 ) {
           for(int i=0; i<texCoordSetCount; i++) {
                readIntArray( in, indices );
                ((IndexedGeometryArray)node).setTextureCoordinateIndices( i, 0, indices );
           }
       }
    }

    protected void writeConstructorParams( DataOutput out ) throws
								IOException {
        super.writeConstructorParams( out );
        out.writeInt( ((IndexedGeometryArray)node).getIndexCount() );
    }

    protected void readConstructorParams( DataInput in ) throws
							IOException {
        super.readConstructorParams( in );
        indexCount = in.readInt();
    }
    
    protected void writeIntArray( DataOutput out, int[] array ) throws IOException {
        ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
        DataOutputStream dataOut = new DataOutputStream( byteStream );

        dataOut.writeInt( array.length );
        for(int i=0; i<array.length; i++)
            dataOut.writeInt( array[i] );
        dataOut.close();
        
        out.writeInt( byteStream.size() );
        out.write( byteStream.toByteArray() );
    }
    
    private void readIntArray( DataInput in, int[] array ) throws IOException {
        byte[] buffer = new byte[ in.readInt() ];
        in.readFully( buffer );
        ByteArrayInputStream byteStream = new ByteArrayInputStream( buffer );
        DataInputStream dataIn = new DataInputStream( byteStream );
        
        for(int i=0; i<array.length; i++)
            array[i] = dataIn.readInt();
        
        dataIn.close();
    }

}
