/*
 *  @(#)LightState.java 1.5 01/03/02 15:34:12
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io.state.javax.media.j3d;

import java.io.*;
import javax.media.j3d.Light;
import javax.media.j3d.BoundingLeaf;
import javax.media.j3d.Group;
import javax.vecmath.Color3f;
import com.sun.j3d.demos.utils.scenegraph.io.retained.Controller;
import com.sun.j3d.demos.utils.scenegraph.io.retained.SymbolTableData;

/**
 * @author Paul Byrne
 * @version	1.5, 03/02/01
 */
public abstract class LightState extends LeafState {

    private int boundingLeaf = 0;
    private int[] scope;

    public LightState( SymbolTableData symbol, Controller control ) {
	super( symbol, control );
        
    }

    public void writeObject( DataOutput out ) throws IOException {
	super.writeObject( out );
        
	scope = new int[ ((Light)node).numScopes() ];
	for(int i=0; i<((Light)node).numScopes(); i++) {
	    scope[i] = control.getSymbolTable().addReference( ((Light)node).getScope(i) );;
	}
        boundingLeaf = control.getSymbolTable().addReference( ((Light)node).getInfluencingBoundingLeaf() );
        
	Color3f color = new Color3f();
	((Light)node).getColor( color );
	control.writeColor3f( out, color );

	out.writeBoolean( ((Light)node).getEnable() );
        
        out.writeInt( boundingLeaf );
        control.writeBounds( out, ((Light)node).getInfluencingBounds() );
        
        out.writeInt( scope.length );
        for( int i=0; i<scope.length; i++) {
           out.writeInt( scope[i] );
        }
    }

    public void readObject( DataInput in ) throws IOException {
       super.readObject(in);
       ((Light)node).setColor( control.readColor3f(in) );
       ((Light)node).setEnable( in.readBoolean() );

       boundingLeaf = in.readInt();

       ((Light)node).setInfluencingBounds( control.readBounds(in) );

        scope = new int[in.readInt()];
        for(int i=0; i<scope.length; i++) {
            scope[i] = in.readInt();
        }

    }
    
    public void buildGraph() {
        super.buildGraph();
        ((Light)node).setInfluencingBoundingLeaf( (BoundingLeaf)control.getSymbolTable().getJ3dNode( boundingLeaf ));
        for(int i=0; i<scope.length; i++) {
            ((Light)node).addScope( (Group)control.getSymbolTable().getJ3dNode( scope[i] ));
        }
    }
    
}
