/*
 *  @(#)MorphState.java 1.3 01/03/02 15:34:36
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io.state.javax.media.j3d;

import java.io.IOException;
import java.io.DataInput;
import java.io.DataOutput;
import javax.media.j3d.Morph;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.Appearance;
import com.sun.j3d.demos.utils.scenegraph.io.retained.Controller;
import com.sun.j3d.demos.utils.scenegraph.io.retained.SymbolTableData;

/**
 * @author Paul Byrne
 * @version	1.3, 03/02/01
 */
public class MorphState extends LeafState {

    private int[] geometry;
    private double[] weights;
    private int appearance;
    
    public MorphState(SymbolTableData symbol,Controller control) {
        super( symbol, control );
        
        if (node!=null) {
            appearance = control.getSymbolTable().addReference( ((Morph)node).getAppearance() );
            weights = ((Morph)node).getWeights();
            geometry = new int[weights.length];
            for(int i=0; i<weights.length; i++)
               geometry[i] = control.getSymbolTable().addReference( ((Morph)node).getGeometryArray(i) );
        }
    }
    
    public void writeObject( DataOutput out ) throws IOException {
        super.writeObject( out );
        
        control.writeBounds( out, ((Morph)node).getCollisionBounds() );
        
        out.writeInt( appearance );
        out.writeBoolean( ((Morph)node).getAppearanceOverrideEnable() );
        
        out.writeInt( geometry.length );
        for(int i=0; i<geometry.length; i++) {
           out.writeInt( geometry[i] );
           out.writeDouble( weights[i] );
        }
    }
    
    public void readObject( DataInput in ) throws IOException {
        super.readObject( in );
        ((Morph)node).setCollisionBounds( control.readBounds( in ));
        appearance = in.readInt();
        ((Morph)node).setAppearanceOverrideEnable( in.readBoolean() );
        
        geometry = new int[ in.readInt() ];
        for(int i=0; i<geometry.length; i++) {
            geometry[i] = in.readInt();
            weights[i] = in.readDouble();
        }
        
        ((Morph)node).setWeights( weights );
    }
    
    public void buildGraph() {
        super.buildGraph();
        ((Morph)node).setAppearance( (Appearance)control.getSymbolTable().getJ3dNode( appearance ) );
        
        GeometryArray[] geom = new GeometryArray[ geometry.length ];
        for(int i=0; i<geometry.length; i++) {
            geom[i] =  (GeometryArray)control.getSymbolTable().getJ3dNode( geometry[i] );
        }
        
        ((Morph)node).setGeometryArrays( geom );
    }
    
    protected javax.media.j3d.SceneGraphObject createNode() {
        return new Morph( null );
    }


}
