/*
 *  @(#)RotPosScalePathInterpolatorState.java 1.5 01/03/02 15:34:35
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */package com.sun.j3d.demos.utils.scenegraph.io.state.javax.media.j3d;

import java.io.IOException;
import java.io.DataInput;
import java.io.DataOutput;
import javax.media.j3d.RotPosScalePathInterpolator;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.vecmath.Point3f;
import javax.vecmath.Quat4f;
import com.sun.j3d.demos.utils.scenegraph.io.retained.Controller;
import com.sun.j3d.demos.utils.scenegraph.io.retained.SymbolTableData;

/**
 * @author Paul Byrne
 * @version	1.5, 03/02/01
 */
public class RotPosScalePathInterpolatorState extends PathInterpolatorState {

    private int target;
    private Point3f[] positions;
    private Quat4f[] quats;
    private float[] scales;
    private Transform3D axisOfTranslation;
    
    public RotPosScalePathInterpolatorState(SymbolTableData symbol,Controller control) {
        super( symbol, control );
        
        if (node!=null)
            target = control.getSymbolTable().addReference( ((RotPosScalePathInterpolator)node).getTarget() );
    }
    
    public void writeObject( DataOutput out ) throws IOException {
        super.writeObject( out );
        
        out.writeInt( target );
    }
    
    public void readObject( DataInput in ) throws IOException {
        super.readObject( in );
        
                
        target = in.readInt();
    }
    
    public void writeConstructorParams( DataOutput out ) throws IOException {
        super.writeConstructorParams( out );
        
        control.writeTransform3D( out, ((RotPosScalePathInterpolator)node).getAxisOfRotPosScale() );
                
        positions = new Point3f[ knots.length ];
        quats = new Quat4f[ knots.length ];
        scales = new float[ knots.length ];
        for(int i=0; i<positions.length; i++) {
            positions[i] = new Point3f();
            quats[i] = new Quat4f();
        }

        ((RotPosScalePathInterpolator)node).getPositions( positions );
        ((RotPosScalePathInterpolator)node).getQuats( quats );
        ((RotPosScalePathInterpolator)node).getScales( scales );
        for(int i=0; i<positions.length; i++) {
            control.writePoint3f( out, positions[i] );
            control.writeQuat4f( out, quats[i] );
            out.writeFloat( scales[i] );
        }
    }
    
    public void readConstructorParams( DataInput in ) throws IOException {
        super.readConstructorParams( in );
        
        axisOfTranslation =  control.readTransform3D( in );

        positions = new Point3f[ knots.length ];
        quats = new Quat4f[ knots.length ];
        scales = new float[ knots.length ];
        for(int i=0; i<positions.length; i++) {
            positions[i] = control.readPoint3f( in );
            quats[i] = control.readQuat4f( in );
            scales[i] = in.readFloat();
        }
        
    }

    
    public void buildGraph() {
        super.buildGraph();
        
        ((RotPosScalePathInterpolator)node).setTarget( (TransformGroup)control.getSymbolTable().getJ3dNode( target ));
    }
    
    public SceneGraphObject createNode( Class j3dClass ) {
        return createNode( j3dClass, new Class[] { javax.media.j3d.Alpha.class,
                                                    TransformGroup.class,
                                                    Transform3D.class,
                                                    knots.getClass(),
                                                    quats.getClass(),
                                                    positions.getClass(),
                                                    scales.getClass() },
                                      new Object[] { null,
                                                     null,
                                                     axisOfTranslation,
                                                     knots,
                                                     quats,
                                                     positions,
                                                     scales } );
                                                    
    }
    
    protected javax.media.j3d.SceneGraphObject createNode() {
        return new RotPosScalePathInterpolator( null, null, axisOfTranslation, knots, quats, positions, scales );
    }


}
