/*
 *  @(#)Shape3DState.java 1.5 01/03/02 15:34:23
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io.state.javax.media.j3d;

import java.io.IOException;
import java.io.DataInput;
import java.io.DataOutput;
import javax.media.j3d.Shape3D;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.Geometry;
import javax.media.j3d.Appearance;
import com.sun.j3d.demos.utils.scenegraph.io.retained.Controller;
import com.sun.j3d.demos.utils.scenegraph.io.retained.SymbolTableData;

/**
 * @author Paul Byrne
 * @version	1.5, 03/02/01
 */
public class Shape3DState extends LeafState {

    private int[] geometry;
    private int appearance;
    
    public Shape3DState( SymbolTableData symbol, Controller control ) {
        super( symbol, control );
        
        if (node!=null) {
            appearance = control.getSymbolTable().addReference( ((Shape3D)node).getAppearance() );
            int length = ((Shape3D)node).numGeometries();
            geometry = new int[length];
            for(int i=0; i<length; i++)
               geometry[i] = control.getSymbolTable().addReference( ((Shape3D)node).getGeometry(i) );
        }
    }
    
    public void writeObject( DataOutput out ) throws IOException {
        super.writeObject( out );
        
        control.writeBounds( out, ((Shape3D)node).getCollisionBounds() );
        
        out.writeInt( appearance );
        
        out.writeInt( geometry.length );
        for(int i=0; i<geometry.length; i++)
           out.writeInt( geometry[i] );
    }
    
    public void readObject( DataInput in ) throws IOException {
        super.readObject( in );
        ((Shape3D)node).setCollisionBounds( control.readBounds( in ));
        appearance = in.readInt();
        
        geometry = new int[ in.readInt() ];
        for(int i=0; i<geometry.length; i++) {
            geometry[i] = in.readInt();
        }
    }
    
    public void buildGraph() {
        super.buildGraph();
        ((Shape3D)node).setAppearance( (Appearance)control.getSymbolTable().getJ3dNode( appearance ) );
        
        for(int i=0; i<geometry.length; i++) {
            ((Shape3D)node).addGeometry( (Geometry)control.getSymbolTable().getJ3dNode( geometry[i] ) );
        }
    }
    
    protected javax.media.j3d.SceneGraphObject createNode() {
        return new Shape3D();
    }


}
