/*
 *  @(#)TextureAttributesState.java 1.5 01/03/02 15:34:29
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io.state.javax.media.j3d;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import com.sun.j3d.demos.utils.scenegraph.io.retained.Controller;
import com.sun.j3d.demos.utils.scenegraph.io.retained.SymbolTableData;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.TextureAttributes;
import javax.media.j3d.Transform3D;
import javax.vecmath.Color4f;
import javax.vecmath.Matrix4d;

public class TextureAttributesState extends NodeComponentState {
    
    public TextureAttributesState( SymbolTableData symbol, Controller control ) {
        super(symbol, control);
    }
    
    public void writeObject( DataOutput out ) throws IOException {
        super.writeObject( out );
        TextureAttributes attr = (TextureAttributes)node;
        Color4f clr = new Color4f();
        Matrix4d mat = new Matrix4d();
        Transform3D trans = new Transform3D();
        int tableComponents = attr.getNumTextureColorTableComponents();
        int tableSize = attr.getTextureColorTableSize();
        int[][] colorTable = new int[tableComponents][tableSize];
        
        out.writeInt( attr.getPerspectiveCorrectionMode() );
        attr.getTextureBlendColor( clr );
        control.writeColor4f( out, clr );
        out.writeInt( tableComponents );
        out.writeInt( tableSize );
        attr.getTextureColorTable( colorTable ); 
        for(int i=0; i<tableComponents; i++)
            for(int j=0; j<tableSize; j++)
                out.writeInt( colorTable[i][j] );
        
        out.writeInt( attr.getTextureMode() );
        attr.getTextureTransform( trans );
        trans.get( mat );
        control.writeMatrix4d( out, mat );
    }
    
    public void readObject( DataInput in ) throws IOException {
        super.readObject( in );
        TextureAttributes attr = (TextureAttributes)node;
        attr.setPerspectiveCorrectionMode( in.readInt() );
        attr.setTextureBlendColor( control.readColor4f(in) );
        int tableComponents = in.readInt();
        int tableSize = in.readInt();
        int[][] colorTable = new int[tableComponents][tableSize];
        for(int i=0; i<tableComponents; i++)
            for(int j=0; j<tableSize; j++)
                colorTable[i][j] = in.readInt();
        if (tableComponents!=0)
            attr.setTextureColorTable( colorTable );
        attr.setTextureMode( in.readInt() );
        Matrix4d mat = control.readMatrix4d( in );
        Transform3D trans = new Transform3D( mat );
        attr.setTextureTransform( trans );
    }
    
    protected javax.media.j3d.SceneGraphObject createNode() {
        return new TextureAttributes();
    }

    
}

