/*
 *  @(#)TextureState.java 1.6 01/03/02 15:34:30
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.io.state.javax.media.j3d;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import com.sun.j3d.demos.utils.scenegraph.io.retained.Controller;
import com.sun.j3d.demos.utils.scenegraph.io.retained.SymbolTableData;
import javax.media.j3d.SceneGraphObject;
import javax.media.j3d.ImageComponent;
import javax.media.j3d.Texture;
import javax.vecmath.Color4f;

public abstract class TextureState extends NodeComponentState {
    
    private int[] imageComponents;
    protected int width;
    protected int height;
    protected int format;
    protected int mipMapMode;
    
    
    public TextureState( SymbolTableData symbol, Controller control ) {
        super(symbol, control);
        
        if (node!=null) {       // Node is null during a load
            ImageComponent[] images = ((Texture)node).getImages();
            imageComponents = new int[ images.length ];
            for(int i=0; i<images.length; i++) {
                imageComponents[i] = control.getSymbolTable().addReference( images[i] );
            }
        }
        
    }
    
    public void writeConstructorParams( DataOutput out ) throws IOException {
        super.writeConstructorParams( out );
        out.writeInt( ((Texture)node).getMipMapMode() );
        out.writeInt( ((Texture)node).getWidth() );
        out.writeInt( ((Texture)node).getHeight() );
        out.writeInt( ((Texture)node).getFormat() );
    }
    
    public void readConstructorParams( DataInput in ) throws IOException {
        super.readConstructorParams( in );
        
        mipMapMode = in.readInt();
        width = in.readInt();
        height = in.readInt();
        format = in.readInt();
    }
    
    public void writeObject( DataOutput out ) throws IOException {
        super.writeObject( out );
        Texture attr = (Texture)node;
        Color4f clr = new Color4f();
        attr.getBoundaryColor( clr );
        control.writeColor4f( out, clr );
        out.writeInt( attr.getBoundaryModeS() );
        out.writeInt( attr.getBoundaryModeT() );
        out.writeBoolean( attr.getEnable() );

        out.writeInt( imageComponents.length );
        for(int i=0; i<imageComponents.length; i++)
            out.writeInt( imageComponents[i] );
        
        out.writeInt( attr.getMagFilter() );
        out.writeInt( attr.getMinFilter() );
    }
    
    public void readObject( DataInput in ) throws IOException {
        super.readObject( in );
        Texture attr = (Texture)node;
        attr.setBoundaryColor( control.readColor4f( in ));
        attr.setBoundaryModeS( in.readInt() );
        attr.setBoundaryModeT( in.readInt() );
        attr.setEnable( in.readBoolean() );

        imageComponents = new int[ in.readInt() ];
        for(int i=0; i<imageComponents.length; i++)
            imageComponents[i] = in.readInt();
        
        attr.setMagFilter( in.readInt() );
        attr.setMinFilter( in.readInt() );
    }
    
    public void addSubReference() {
        for( int i=0; i<imageComponents.length; i++)
            control.getSymbolTable().incNodeComponentRefCount( imageComponents[i] );
    }
    
    public void buildGraph() {
        for(int i=0; i<imageComponents.length; i++) {
            ((Texture)node).setImage( i, (ImageComponent)control.getSymbolTable().getJ3dNode( imageComponents[i] ));

            /*
             // Debug aid - paints the textures image in a new frame
            final java.awt.image.BufferedImage im = ((javax.media.j3d.ImageComponent2D)((javax.media.j3d.Texture2D)node).getImage( i )).getImage();
            javax.swing.JFrame f = new javax.swing.JFrame( (width+" "+height) );
            javax.swing.JPanel p = new javax.swing.JPanel() {
	    public void paint(java.awt.Graphics g ) {
		super.paint(g);
		g.drawImage( im, 10, 10, null );
	    }

            };

            f.getContentPane().add(p);
            f.setSize( 500, 200 );
            f.setLocation( 0, 200 );
            f.show();
             */
        }
    }
    
}

