/*
 *  @(#)ChangePolygonAttributes.java 1.2 00/12/19 17:24:42
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.traverser;

import javax.media.j3d.PolygonAttributes;
import javax.media.j3d.Appearance;

/** 
 *
 * @author  paulby
 * @version 
 */
public class ChangePolygonAttributes extends Object {

  /**
    * Traverse the graph setting the polygon mode in all Polygon Attributes
    * object that exist
    */
  public static void setPolygonMode( javax.media.j3d.Node treeRoot,
                                     final int polygonMode ) {
    setPolygonMode( treeRoot, polygonMode, false );
  }                     
                 
  /**
    * Traverse the graph setting the polygon mode in all PolygonAttributes.
    * If createAttribute is false on PolygonAttribute objects that already
    * exist are modified.
    * If craeteAttribute is true, PolygonAttributes and Appearance objects will
    * be created as necessary
    */
  public static void setPolygonMode( javax.media.j3d.Node treeRoot,
                                     final int polygonMode, 
                                     final boolean createAttribute ) {
  
    AppearanceChangeProcessor processor = new AppearanceChangeProcessor() {
      public void changeAppearance( javax.media.j3d.Shape3D shape,
                                    javax.media.j3d.Appearance app ) {
        if (app==null)
          if (createAttribute) {
            app = new Appearance();
            shape.setAppearance( app );
          } else 
            return;
        PolygonAttributes poly = app.getPolygonAttributes();
        if (poly!=null)
          poly.setPolygonMode( polygonMode );
        else if (createAttribute) {
          poly = new PolygonAttributes();
          poly.setPolygonMode( polygonMode );
          app.setPolygonAttributes( poly );
        }
      }
    };
    
    scanTree( treeRoot, processor );
  }
  
  /**
    * Traverse the graph setting the cullFace in all Polygon Attributes
    * object that exist
    */
  public static void setCullFace( javax.media.j3d.Node treeRoot,
                                     final int cullFace ) {

    setCullFace( treeRoot, cullFace, false );                                  
  }
                                    
  /**
    * Traverse the graph setting the cullFace in all PolygonAttributes.
    * If createAttribute is false on PolygonAttribute objects that already
    * exist are modified.
    * If craeteAttribute is true, PolygonAttributes and Appearance objects will
    * be created as necessary
    */
  public static void setCullFace( javax.media.j3d.Node treeRoot,
                                     final int cullFace,
                                     final boolean createAttribute ) {
  
    AppearanceChangeProcessor processor = new AppearanceChangeProcessor() {
      public void changeAppearance( javax.media.j3d.Shape3D shape,
                                    javax.media.j3d.Appearance app ) {
        if (app==null)
          if (createAttribute) {
            app = new Appearance();
            shape.setAppearance( app );
          } else 
            return;
        PolygonAttributes poly = app.getPolygonAttributes();
        if (poly!=null)
          poly.setCullFace( cullFace );
        else if (createAttribute) {
          poly = new PolygonAttributes();
          poly.setCullFace( cullFace );
          app.setPolygonAttributes( poly );
        }
      }
    };
    
    scanTree( treeRoot, processor );
  }  
  
  private static void scanTree( javax.media.j3d.Node treeRoot, 
                                AppearanceChangeProcessor processor ) {
    try {
      Class shapeClass = Class.forName("javax.media.j3d.Shape3D");
    
      TreeScan.findNode( treeRoot, shapeClass, processor, false, true );
    } catch(Exception e ) {
      e.printStackTrace();
      System.out.println( "ERROR ChangePolygonAttributes, SceneGraph contains"+
                          " Live or compiled nodes, without correct capabilities");
    }
  }
  
}
