/*
 *  @(#)GeometryArrayStripifier.java 1.3 00/12/19 17:24:44
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.traverser;

import java.util.HashMap;
import javax.media.j3d.TriangleArray;
import javax.media.j3d.Shape3D;
import javax.media.j3d.GeometryArray;
import com.sun.j3d.utils.geometry.GeometryInfo;
import com.sun.j3d.utils.geometry.Stripifier;
import com.sun.j3d.demos.utils.geometry.GeometryInfoGenerator;
import com.sun.j3d.demos.utils.capability.Capabilities;

/**
 * Takes each TriangleArray geometry in the
 * scene graph and stripifies the data
 *
 * @author  paulby
 * @version 1.3, 12/19/00
 */
public class GeometryArrayStripifier extends Object {


    public static void stripify( javax.media.j3d.Node[] treeRoots ) {
        StripifyProcessor processor = new StripifyProcessor() {
            private GeometryInfo geomInfo = new GeometryInfo( GeometryInfo.TRIANGLE_ARRAY );
            private Stripifier stripifier = new Stripifier();
            private HashMap map = new HashMap();    // Mapping of original
                                                    // TriangleArrays to new
                                                    // StripArrays
            
            public void stripifyGeometry( javax.media.j3d.Shape3D shape ) {
                for( int i=0; i<shape.numGeometries(); i++) {
                    if (shape.getGeometry(i)!=null &&
                        shape.getGeometry(i) instanceof GeometryArray) {

                        GeometryArray geom = (GeometryArray)shape.getGeometry( i );

                        GeometryArray previousConvert = (GeometryArray)map.get( geom );

                        if (previousConvert!=null) {
                            shape.setGeometry( previousConvert, i );
                        } else if (geom instanceof TriangleArray) {
                            GeometryInfoGenerator.create( geomInfo, geom );
                            stripifier.stripify( geomInfo );
                            GeometryArray newGeom = geomInfo.getGeometryArray();
                            shape.setGeometry( newGeom, i );
                            map.put( geom, newGeom );
                            
                            int[] capabilities = Capabilities.getCapabilities( geom );
                            for(int c=0; c<capabilities.length; c++)
                                newGeom.setCapability( capabilities[c] );
                        }
                    }
                }
            }
        };
        
        for(int i=0; i<treeRoots.length; i++)
            scanTree( treeRoots[i], processor );
    }
    
  /**
   * Stripify all TriangleArray geometry in the scene graph
   *
   * @param treeRoot, root of Scene Graph
   */
    public static void stripify( javax.media.j3d.Node treeRoot ) {        
        stripify( new javax.media.j3d.Node[] { treeRoot } );
    }
    
    private static void scanTree( javax.media.j3d.Node treeRoot, StripifyProcessor processor ) {
        try {
            TreeScan.findNode( treeRoot, javax.media.j3d.Shape3D.class, processor, false, true );
        } catch(Exception e ) {
            e.printStackTrace();
            System.out.println( "ERROR GeometryArrayStripifier, SceneGraph contains Live or compiled nodes");
        }
    }
    
}
