/*
 *  @(#)GeometrySizeProcessor.java 1.8 00/12/19 17:24:45
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.traverser;

import java.util.HashSet;
import java.util.Enumeration;
import javax.media.j3d.*;

/** 
 * Count the number of Geometry primitives in all the Shape3D nodes supplied by
 * TreeScan
 *
 * The number of Triangles, Quads, Points and Lines are available
 */
public class GeometrySizeProcessor implements ProcessNodeInterface {

  private long triangleCount;
  private long quadCount;
  private long lineCount;
  private long pointCount;
  
  private long vertexInStrips;
  private long vertexInArrays;
  
  private long stripCount;

  private int shape3Dcount;

  private HashSet appearanceSet = new HashSet();
  
  /** Creates new TriangleCountProcessor, sets initial triangleCount to 0 */
  public GeometrySizeProcessor() {
    triangleCount = 0;
    pointCount = 0;
    quadCount = 0;
    lineCount = 0;
    vertexInStrips = 0;
    vertexInArrays = 0;
    stripCount = 0;
    shape3Dcount = 0;
    appearanceSet.clear();
  }
  
  /**
    * Process the Shape3D node, increment triangleCount by the number
    * of triangles in the Geometry of the node
    * @param node must be a Shape3D node
    */
  public void processNode(javax.media.j3d.Node node) {
    if (!javax.media.j3d.Shape3D.class.isAssignableFrom( node.getClass() ))
      throw new RuntimeException("Node class "+node.getClass().getName()+" is not a (subclass) of Shape3D as expected");
    
    shape3Dcount++;
    appearanceSet.add( ((Shape3D)node).getAppearance() );
    
    Enumeration e = ((Shape3D)node).getAllGeometries();
    
    while(e.hasMoreElements()) {
        Geometry geom = (Geometry)e.nextElement();

        if (geom instanceof GeometryStripArray) {
          int numStrips = ((GeometryStripArray)geom).getNumStrips();
          int vertexCounts[] = new int[numStrips];
          ((GeometryStripArray)geom).getStripVertexCounts( vertexCounts );
          for( int i=0; i<numStrips; i++)
            vertexInStrips += vertexCounts[i];

          stripCount += numStrips;

          if (geom instanceof TriangleFanArray) {
            for(int i=0; i<numStrips; i++)
              triangleCount += vertexCounts[i] - 2;
          } else if (geom instanceof TriangleStripArray) {
            for(int i=0; i<numStrips; i++)
              triangleCount += vertexCounts[i] - 1;
          } else if (geom instanceof LineStripArray) {
            for(int i=0; i<numStrips; i++)
              lineCount += vertexCounts[i] - 1;
          } else
            unsupportedGeom( geom );
        } else if (geom instanceof IndexedGeometryStripArray) {
          int numStrips = ((IndexedGeometryStripArray)geom).getNumStrips();
          int vertexCounts[] = new int[numStrips];
          ((IndexedGeometryStripArray)geom).getStripIndexCounts( vertexCounts );

          if (geom instanceof IndexedTriangleFanArray) {
            for(int i=0; i<numStrips; i++)
              triangleCount += vertexCounts[i] - 2;
          } else if (geom instanceof IndexedTriangleStripArray) {
            for(int i=0; i<numStrips; i++)
              triangleCount += vertexCounts[i] - 1;
          } else if (geom instanceof IndexedLineStripArray) {
            for(int i=0; i<numStrips; i++)
              lineCount += vertexCounts[i] - 1;
           } else
            unsupportedGeom( geom );
        } else if (geom instanceof IndexedGeometryArray) {
          if (geom instanceof IndexedTriangleArray) {
            triangleCount += ((IndexedGeometryArray)geom).getIndexCount()/3;
            vertexInArrays += ((IndexedGeometryArray)geom).getIndexCount();
          } else if (geom instanceof IndexedQuadArray) {
            quadCount += ((IndexedGeometryArray)geom).getIndexCount()/4;
            vertexInArrays += ((IndexedGeometryArray)geom).getIndexCount();
          } else if (geom instanceof IndexedLineArray) {
            lineCount += ((IndexedGeometryArray)geom).getIndexCount()/2;
            vertexInArrays += ((IndexedGeometryArray)geom).getIndexCount();
          } else if (geom instanceof IndexedPointArray) {
            pointCount += ((IndexedGeometryArray)geom).getIndexCount();
            vertexInArrays += ((IndexedGeometryArray)geom).getIndexCount();
          } else
            unsupportedGeom(geom);
        } else if (geom instanceof LineArray) {
          lineCount += ((LineArray)geom).getVertexCount()/2;
          vertexInArrays += ((LineArray)geom).getVertexCount(); 
        } else if (geom instanceof TriangleArray) {
          triangleCount += ((TriangleArray)geom).getVertexCount()/3;
          vertexInArrays += ((TriangleArray)geom).getVertexCount(); 
        } else if (geom instanceof QuadArray) {
          quadCount += ((QuadArray)geom).getVertexCount()/4;
          vertexInArrays += ((QuadArray)geom).getVertexCount(); 
        } else if (geom instanceof PointArray) {
          pointCount += ((PointArray)geom).getVertexCount();
          vertexInArrays += ((PointArray)geom).getVertexCount(); 
        } else
          unsupportedGeom( geom );
    }
  }
  
  /**
    * Display the Unsupported Geometry error message to stderr
    */
  private void unsupportedGeom( Geometry geom ) {
      if (geom==null)
          return;
      else
          System.err.println("GeometrySizeProcessor: Unsupported Geometry Type "+
            geom.getClass().getName() );
  }
  
  /** Getter for property lineCount.
   * @return Value of property lineCount.
   */
  public long getLineCount() {
    return lineCount;
  }
  /** Setter for property lineCount.
   * @param lineCount New value of property lineCount.
   */
  public void setLineCount(long lineCount) {
    this.lineCount = lineCount;
  }
  /** Getter for property quadCount.
   * @return Value of property quadCount.
   */
  public long getQuadCount() {
    return quadCount;
  }
  /** Setter for property quadCount.
   * @param quadCount New value of property quadCount.
   */
  public void setQuadCount(long quadCount) {
    this.quadCount = quadCount;
  }
  /** Getter for property pointCount.
   * @return Value of property pointCount.
   */
  public long getPointCount() {
    return pointCount;
  }
  /** Setter for property pointCount.
   * @param pointCount New value of property pointCount.
   */
  public void setPointCount(long pointCount) {
    this.pointCount = pointCount;
  }
  /** Getter for property triangleCount.
   * @return Value of property triangleCount.
   */
  public long getTriangleCount() {
    return triangleCount;
  }
  /** Setter for property triangleCount.
   * @param triangleCount New value of property triangleCount.
   */
  public void setTriangleCount(long triangleCount) {
    this.triangleCount = triangleCount;
  }

    public int getShape3DCount() {
        return shape3Dcount;
    }
    
    public void setShape3DCount( int count ) {
        shape3Dcount = count;
    }
    
    public int getAppearanceCount() {
        return appearanceSet.size();
    }
  
  /**
    * Return the percentage of Vertex which are in strips
    */
  public float getStripPercentage() {
    if ((vertexInStrips + vertexInArrays)==0)
      return 0;
    else
      return ((float)vertexInStrips)/(vertexInStrips + vertexInArrays) * 100f;
  }
  
  /** 
    * Return the average length of the Strips
    */
  public float getAverageStripLength() {
    if (stripCount==0)
      return 0.0f;
    else
      return (float)vertexInStrips/(float)stripCount;
  }
}
