/*
 *  @(#)GeometryStatistics.java 1.3 00/12/19 17:24:46
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.scenegraph.traverser;

/** 
 *
 * @author  paulby
 * @version 
 *
 * Provide statistics on the Geometry in a given scene graph
 */
public class GeometryStatistics extends Object {
  
  /**
    * Print the geometry statistics for the all SceneGraph in treeRoots to
    * stdout. Information on all geometry nodes in the Scene Graph will
    * be used, even nodes under disabled Switch nodes.
    *
    * treeRoot and all of it's children must be non-live and non-compiled
    * if they are not an error message will be displayed
    */
  public static void printStatistics( javax.media.j3d.Node[] treeRoots ) {
    GeometrySizeProcessor processor = new GeometrySizeProcessor();
    TreeScan scan = new TreeScan();
    int triangleCount=0;
    int quadCount=0;
    int lineCount=0;
    int pointCount=0;
    
    try {
      Class geometryClass = Class.forName("javax.media.j3d.Shape3D");
    
      for(int i=0; i<treeRoots.length; i++) {
        scan.findNode( treeRoots[i], geometryClass, processor, false, false );
      }
            
      System.out.println("Triangle Count : "+ processor.getTriangleCount());
      System.out.println("Quad Count     : "+ processor.getQuadCount());
      System.out.println("Line Count     : "+ processor.getLineCount());
      System.out.println("Point Count    : "+processor.getPointCount() );
      
      System.out.println("Percentage of Vertex in Strips "+
                                processor.getStripPercentage() +"%");
      System.out.println("Average Strip Length "+
                                processor.getAverageStripLength() );
    } catch( javax.media.j3d.CapabilityNotSetException e ) {
      System.out.println("Error, SceneGraph is either live or compiled");
    } catch( ClassNotFoundException ex ) {
      ex.printStackTrace();
      System.exit(1);
    }
  }

  /**
    * Print the geometry statistics for the SceneGraph treeRoot to
    * stdout. Information on all geometry nodes in the Scene Graph will
    * be used, even nodes under disabled Switch nodes.
    *
    * treeRoot and all of it's children must be non-live and non-compiled
    * if they are not an error message will be displayed
    */
  public static void printStatistics( javax.media.j3d.Node treeRoot ) {
      printStatistics( new javax.media.j3d.Node[] { treeRoot } );
  }
  
}
