/*
 *  @(#)FlyBehavior.java 1.10 00/12/19 17:29:38
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.vpbehaviors;

import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;
import java.awt.AWTEvent;
import java.awt.Component;
import java.awt.Cursor;
import javax.swing.SwingUtilities;

import javax.media.j3d.WakeupOnAWTEvent;
import javax.media.j3d.WakeupOnElapsedFrames;
import javax.media.j3d.WakeupOr;
import javax.media.j3d.WakeupCriterion;
import javax.media.j3d.WakeupCondition;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.Transform3D;
import javax.media.j3d.View;

import javax.vecmath.Vector3f;

/**
 *<P>This behavior will allow the user to move to any point with any
orientation in 3 space.</P>
<P>All controls are provided by the mouse. Each mouse button
generates a different type of motion while the button is pressed, the
distance of the mouse cursor from the center of the canvas3D will
control the speed of motion. The behavior of each button is described
below with reference to the View coordinate system.</P>
<P>Left Mouse Button - Moving the mouse up and down will move the
view forward and backward. Moving the mouse left and right will yaw
the view around the Y axis.</P>
<P>Right Mouse Button - Moving the mouse up and down will move the
view up and down the Y axis. Movig the mouse left and right will
translate the view along the X axis.</P>
<P>Middle Mouse Button - Moving the mouse up and down will pitch the
view around the X axis. Moving the mouse left and right will roll the
view around the Z axis.</P>

 * Canvas3D size changes are tracked
 *
 * @author Paul Byrne
 * @version	1.10, 12/19/00
 */
public class FlyBehavior extends MouseBehavior {

    private Transform3D velocityTransform;
    private Transform3D yawTransform;
    private Transform3D rollTransform;
    private Transform3D pitchTransform;
    private Transform3D newTargetTransform = new Transform3D();
    

    private Vector3f velocity = new Vector3f();
    private float yawAngle = 0f;
    private float pitchAngle = 0f;
    private float rollAngle = 0f;
    
    public FlyBehavior( View view ) {
        super(view);
    }
    
    /**
     * Canvas3D size will be calculated on MouseEvent.MOUSE_ENTERED
     * or component resize
     */
    public FlyBehavior() {
        super();
    }
         
    public void initialize() {
        rollTransform = new Transform3D();
        pitchTransform = new Transform3D();
        yawTransform = new Transform3D();
        targetTransform = new Transform3D();
        velocityTransform = new Transform3D();
        
        super.initialize();
    }

    protected void processMouseEvent( MouseEvent evt ) {
        
        float offsetX = (evt.getX() - canvasCenter.x) / canvasCenter.x;
        float offsetY = (evt.getY() - canvasCenter.y) / canvasCenter.y;
        float factorX=0f;
        float factorY=0f;

        if (Math.abs(offsetX)-deadXSize > 0f) {
            factorX = (float)Math.pow( Math.abs(offsetX)-deadXSize, 2f );
            motion = true;
        } else {
            factorX = 0f;
        }

        if (Math.abs(offsetY)-deadYSize > 0f) {                
            factorY = (float)Math.pow( Math.abs(offsetY)-deadYSize, 2f );
            motion = true;
        } else {
            factorY = 0f;
        }

        if (offsetX>0f)
        factorX = -factorX;

        if (offsetY<0f)
            factorY = -factorY;
        
        if (SwingUtilities.isLeftMouseButton( evt ) ) {
            if (evt.getID()==MouseEvent.MOUSE_RELEASED) {
                yawAngle = 0;
                velocity.z = 0;
                motion = false;
            } else {
                //System.out.println( factorX +" "+factorY );
                yawAngle = MAX_ANGLE * factorX;
                velocity.z = MAX_VELOCITY * factorY;
            }
        } else if ( SwingUtilities.isRightMouseButton( evt )) {
            if (evt.getID()==MouseEvent.MOUSE_RELEASED) {
                velocity.x = 0;
                velocity.y = 0;
                motion = false;
            } else {
                velocity.x = MAX_VELOCITY * factorX;
                velocity.y = MAX_VELOCITY * factorY;
            }
        } else if ( SwingUtilities.isMiddleMouseButton( evt )) {
            if (evt.getID()==MouseEvent.MOUSE_RELEASED) {
                pitchAngle = 0f;
                rollAngle = 0f;
                motion = false;
            } else {
                pitchAngle = MAX_ANGLE * factorY;
                rollAngle = MAX_ANGLE * factorX;
            }
        } else { 
            yawAngle = 0f;
            pitchAngle = 0f;
            rollAngle = 0f;
            velocity.x = 0;
            velocity.z = 0;
            velocity.y = 0;
            motion = false;
        } 
    }

    protected void integrateTransforms() {
        if (!collisionEnabled()) {
            yawTransform.rotY( yawAngle );
            pitchTransform.rotX( pitchAngle );
            rollTransform.rotZ( rollAngle );

            velocityTransform.set( velocity );
            velocityTransform.mul( yawTransform );
            velocityTransform.mul( pitchTransform );
            velocityTransform.mul( rollTransform );

            targetTG.getTransform( targetTransform );
            targetTransform.mul( yawTransform );
            targetTransform.mul( pitchTransform );
            targetTransform.mul( rollTransform );
            targetTransform.mul( velocityTransform );
            targetTG.setTransform( targetTransform );
        } else {
            targetTG.getTransform( targetTransform );
            SweptVolumeCollision collision = collisionControl.getCollisions( targetTransform, newTargetTransform, velocity, rollAngle, pitchAngle, yawAngle );
            targetTG.setTransform( newTargetTransform );
        }

    }

}

