/*
 *  @(#)SweptVolume.java 1.4 00/12/19 17:29:47
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.vpbehaviors;

import javax.vecmath.Vector3f;

/**
 * Describes the set of rays cast in each of six directions to determine if
 * the view platform is in collision with scene geometry. It is not intended
 * for collision detection between the view platform and 'fast moving' animations
 * in the application.
 *
 * The arrays of rays consist of Vector3f pairs describing the start end
 * end of each ray. During collision detection the start of the ray is transformed
 * by the current ViewPlatform transform and the end of the ray is transformed 
 * by the next ViewPlatform transform.
 *
 * The implementation of VPCollisionInterface will determine which rays are
 * cast, the intent is that only subset of the rays will be cast per frame.
 * For example when the ViewPlatform is moving forward ONLY the frontRays
 * will be cast.
 *
 *
 * @author Paul Byrne
 * @version	1.4, 12/19/00
 */
public abstract class SweptVolume {

    public Vector3f[] frontRays;          // List of rays for this volume
    public Vector3f[] backRays;          // List of rays for this volume
    public Vector3f[] leftRays;          // List of rays for this volume
    public Vector3f[] rightRays;          // List of rays for this volume
    public Vector3f[] upRays;          // List of rays for this volume
    public Vector3f[] downRays;          // List of rays for this volume
}

