/*
 *  @(#)VPCollisionInterface.java 1.5 00/12/19 17:29:50
 *
 * Copyright (c) 2000-2001 Sun Microsystems, Inc. All Rights Reserved.
 */
package com.sun.j3d.demos.utils.vpbehaviors;

import java.util.ArrayList;

import javax.media.j3d.BranchGroup;
import javax.media.j3d.Transform3D;
import javax.media.j3d.Locale;
import javax.vecmath.Vector3f;

/**
 * Interface for all collision detection implementations
 */
public interface VPCollisionInterface {
    
    /**
      * Add a BranchGroup to check for collision. 
      * setCapabilites() should be called before the BranchGroup is
      * added.
      *
      * @param bg BranchGroup to add to set of BranchGroups checked for
      * collision
      */
    public void addCollisionBG( BranchGroup bg );
    
    /**
      * Check for collision with all geometry in <code>locale</code>
      * Ensure the capability bits in each branchGraph are set correctly
      * by calling setCapabilities()
      *
      * @param locale Collision detection will be performed on all geometry in 
      * the locale.
      *
      * @see setCapabilities( BranchGroup branchGroup )
      */
    public void setCollisionLocale( Locale locale );
    
    /**
      * Set the capabilities in the BranchGroup to allow this collision 
      * detection to work. The branchGroup must not be live or compiled.
      *
      * @param branchGroup Root of graph in which all necessary capabilities
      * will be set
      */
    public void setCapabilities( BranchGroup branchGroup );
    
    /**
     * Set the SweptVolume to be used in collision calculations
     */
    public void setVPSweptVolume( SweptVolume volume );
    
    /**
     * Get the SweptVolume used in collision calculations
     */
    public SweptVolume getVPSweptVolume();
    
    /**
      * Get the number of collision BranchGroups that are checked for
      * collision
      */
    public int getCollisionBGCount();

    /**
      * Get the BranchGroup at the specified index that is used for
      * collision detection
      */
    public BranchGroup getCollisionBG( int index );
    
    /**
     * Check for collision when casting the swept volume from currentLocation to currentLocation
     * modified by the roll, pitch, yaw and velocity
     * 
     * nextLocation will return the next NON COLLISION location, this will be implementation dependent.
     * 
     * If collision occurred the data for all the collisions will be returned in
     * SweptVolumeCollision. If no collision occurred null will be returned.
     */
    public SweptVolumeCollision getCollisions( Transform3D currentLocation, Transform3D nextLocation,
    Vector3f velocity, float roll, float pitch, float yaw );

}
